
/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"


/* don't #include specs.h, it has code in it... */

/* Patch history, starting from v2.1:
   2.11:  no changes here!
   2.12:  still calm
   2.13:
          got rid of leave()
	  replaced fifos with socketpair
	  replaced system() with exec() and fork()
	  replaced gets() with fgets()
   2.14:
          headers moved to src/shared
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <errno.h>


#define indicatefilm "-MakeIntoFilm"

typedef char * String;

#define strdup(s) (String)strcpy((String)calloc(strlen(s) +1, sizeof(char)), s)


void myexec(String command)
{
  String *arg = NULL;
  String s = strtok(command, " ");
  int n = 1;
  
  while (s) {
    arg = (String *) realloc(arg, ++n * sizeof(String));
    arg[n - 2] = s;
    s = strtok(NULL, " ");
  }
  arg[n - 1] = NULL;
  execvp(arg[0], arg);
  perror(arg[0]); 
  printf("%s failed with error %d\n", arg[0], errno);
}

/* filmbasin library tag [-f frames] [-s simbasin] [-p plotbasin] [-a] */

void main(int argc, char **argv)
{
  String tag;
  int i, nframes, args;
  char plot[1000], sim[1000], plotargs[1000], simargs[1000], library[1000];
  char temp[1000];
  int filedes[2];
  char option;
  FILE *fp; 

  printf("\n\
Strata version %g:  filmbasin\n\
Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris.\n\
Filmbasin and the rest of Strata come with ABSOLUTELY NO WARRANTY.\n\
They are free software, and you are welcome to redistribute them under\n\
certain conditions; for details of either of these subjects please see\n\
the Gnu General Public License, of which you whould have received copies\n\
in the user manual and the source code.\n\n", STRATA_VERSION);

  if (argc < 3 || '-' == argv[2][0]) {
    printf
      ("syntax:  filmbasin library tag [-f frames] [-s sim] [-p plot] [-a]\n");
    exit(1);
  }

  tag = argv[2];

  strcpy(library, argv[1]);
  strcpy(temp, argv[1]);
  i = strlen(argv[1]);
  if (i > 4 && !strcmp(argv[1] + i - 4, ".dat")) 
    library[i - 4] = '\0';
  else
    strcat(temp, ".dat");
  fp = fopen(temp, "r");
  if (!fp) {
    printf("Error:  cannot read %s\n", temp);
    exit(2);
  }
  fclose(fp);

  strcpy(plot, "plotbasin");
  strcpy(sim, "simbasin");
  nframes = 0;
  args = 0;

  option = '\0';
  for (i = 3; i < argc; ++i)
    if ('-' == argv[i][0]) {
      if (!strcmp(argv[i], "-f"))
	option = 'f';
      else if (!strcmp(argv[i], "-s"))
	option = 's';
      else if (!strcmp(argv[i], "-p"))
	option = 'p';
      else if (!strcmp(argv[i], "-a")) {
	args = 1;
	option = '\0';
      }
      else
	printf("option \"%s\" unknown\n", argv[i]);
    }
    else {
      if ('f' == option) 
	sscanf(argv[i], "%d", &nframes);
      else if ('s' == option)
	sscanf(argv[i], "%s", sim);
      else if ('p' == option)
	sscanf(argv[i], "%s", plot);
      else 
	printf("Don't know what to do with \"%s\" argument\n", argv[i]);
      option = '\0';
    }

  /* won't handle quoted arguments correctly, most likely */
  if (args) {
    printf("Plotter arguments:  ");
    fgets(plotargs, sizeof(plotargs) / sizeof(char), stdin);
    printf("Simulator arguments:  ");
    fgets(simargs, sizeof(simargs) / sizeof(char), stdin);
  }
  else {
    strcpy(plotargs, "");
    strcpy(simargs, "");
  }


  /* arrange for interprocess communication */
  if (socketpair(AF_UNIX, SOCK_STREAM, 0, filedes) < 0) {
    perror("Filmbasin: ");
    printf("Filmbasin:  socketpair() failed on %d\n", errno);
    exit(3);
  }


  /* really ought to call sigaction() and do checks for various
     failure modes between children processes... */
    
  /* start the simulator and plotter processes */
  if (fork()) {
    close(filedes[1]);
    sprintf(temp, "%s %s.film.%s %s %d %s", 
	    plot, library, tag, indicatefilm, filedes[0], plotargs);
    myexec(temp);
  }
  else {
    close(filedes[0]);
    sprintf(temp, "%s %s %s %s %d %d %s", 
	    sim, library, indicatefilm, tag, nframes, filedes[1], simargs);
    myexec(temp);
  }
} 

