/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"

/* If you want to use a simulator other than simbasin or a plotter other 
   than plotbasin with filmbasin, you'll need the specs for how each is
   supposed to behave in the interactions with each other and filmbasin. 
   Here's pseudocode that ought to get it across; you'll probably want
   to read film.c and see what filmbasin is doing.                    */


/* SIMULATOR */

/* for films, syntax is
   sim (library) -MakeIntoFilm (tag) (number of frames) (socket) (other args)
   #frames will be passed by filmbasin as zero if no -f argument
   was specified for filmbasin; in this case the simulator is expected
   to set the #frames = #timelines.     */


Boolean movie;
int nframes;
char *tag;

void main(int argc, char **argv)
{
  char temp[1000], library[1000];
  int sock;
  FILE *fp;

  strcpy(library, argv[1]);
  cut_off_dat_suffix_if_present(library);

  if (movie = (argc > 2 && !strcmp(argv[2], "-MakeIntoMovie"))) { 
    tag = argv[3];
    sscanf(argv[4], "%d", &nframes);
    sscanf(argv[5], "%d", &sock);
  }

  read_library();

  deal_with_parameters();

  if (movie) printf("\n");
  printf("Delta x %.1g; delta t %.1g (%.1g)\n", 
	 spatial_distance, time_per_iteration, 
	 time_per_iteration * iterations_per_timestep);

  if (do_thermal_stuff) {
    whatever;
  }
  else {
    /* tell plotter there's no thermal data */
    sand_conductivity = shale_conductivity = fluid_conductivity = 0;
  }

  if (movie) {
    /* turn library name into (library).film.(tag) */
    strcat(library, ".film.");
    strcat(library, tag);

    if (nframes > ntimelines) 
      printf("Lowering # frames to # timelines, %d.\n", nframes = ntimelines);
    else if (!nframes)
      nframes = ntlines;
    else if (ntimesteps % nframes)
      printf("Warning:  # frames is not a factor of # timesteps (%d).\n", 
	     ntimesteps);
    movie_next_time = movie_time_delta = 
      ntimesteps * time_per_iteration / nframes;
    printf("Taking snapshot every %.1g years\n", movie_time_delta);

    /* write the initial dat.out file, with ages as zeros */
    sprintf(temp, "%sdat.out", library);
    fp = fopen(temp, "w");
    fprintf(fp, "%d %g\n", model_number, version_number); 
    fprintf(fp, "%g %d %d %d %g 0 0 0 0 0 0\n",
	    spatial_distance, number_spatial_nodes, ntimesteps,
	    iterations_per_timeline, time_per_iteration);
    fprintf(fp, "%d %g %g %g %g\n",
	    whether_compaction, diffusion_nonmarine, diffusion_marine,
	    sand_composition_cutoff, composition_decay_constant);
    fprintf(fp, "%g %g %g\n", sand_thermal_conductivity, 
	    shale_thermal_conductivity, fluid_thermal_conductivity);
    fclose(fp);

    /* filmbasin has created a socket pair for the plotter and simulator 
       to communicate with; each has been given its half with the other
       half closed */

    /* tell the plotter how many digits the highest age has, for slicelist */
    sprintf(temp, "%2d", 
	    (int) floor(log10((double)(ntimesteps * time_per_iteration))) + 1);
    write(sock, temp, 2);

    /* tell the plotter how many frames */
    sprintf(temp, "%d", nframes);
    strcat(temp, ".");
    write(sock, temp, strlen(temp));
  }


  timelines_done = 0;
  current_time = 0;
  for (i = 1; i <= ntimesteps; ++i) {
    do_an_iteration();

    if (!(i % timesteps_per_timeline)) 
      timelines_done++;

    current_time = i * time_per_iteration;

    if (movie && current_time >= movie_next_time) {
      movie_next_time += movie_time_delta;
      do_movie_save(sock);
    }
  }
  
  /* simulation over */

  if (movie) {
    read(sock, temp, 1); /* listen for plotter's last words */
    write(sock, "q", 1); /* tell plotter to quit */
    close(sock);
  }
}

extern void usual_save(FILE *);

void do_movie_save(int sock)
{
  int i;
  char temp[1000];
  FILE *fp;

  /* wait for signal that plotter is done w files */
  if (read(sock, temp, 1) != 1) {
    /* cheat to make the main loop exit right away, after
       which it will die a horrible death */
    ntimesteps = -1;
    return;
  }

  sprintf(temp, "%ss.out", library);
  fp = fopen(temp, "w");

  for (i = 0; i < 6; ++i)    /* faking #s of timelines in each slice */
    fprintf(fp, "%d ", timelines_done);

  for (i = 0; i < timelines_done; ++i)
    fprintf(fp, "%d ", shore_boundary[i]);

  for (i = 0; i < timelines_done; ++i)
    fprintf(fp, "%lf ", sealevel[i]);
  fclose(fp);

  sprintf(temp, "%sdat.out", library);
  fp = fopen(temp, "w");

  fprintf(fp, "%d %g\n", model_number, version_number);
  fprintf(fp, "%g %d %d %d %g\n", 
	  spatial_distance, number_spatial_nodes, ntimesteps,
	  iterations_per_timeline, time_per_iteration);
  for (i = 0; i < 6; ++i)     /* faking age of each timeslice */
    fprintf(fp, "%g ", current_time);
  fprintf(fp, "\n");
  fprintf(fp, "%d %g %g %g %g\n", 
	  whether_compaction, diffusion_nonmarine, diffusion_marine,
	  sand_composition_cutoff, composition_decay_constant);
  fprintf(fp, "%g %g %g\n", sand_thermal_conductivity, 
	  shale_thermal_conductivity, fluid_thermal_conductivity);
  fclose(fp);

  sprintf(temp, "%sn.out", library);
  if (fp = fopen(temp, "w")) {
    usual_save(fp);
    fclose(fp);
  }
  else {
    printf("Fatal error opening %s for output.  Film of %s exiting.\n",
	   temp, library);
    exit(1);
  }

  /* signal to plotter that new files are ready for plotting */
  write(sock, "p", 1);
}

extern void write_datum(FILE *, double);

void usual_save(FILE *fp)
{
  int i, j, k;
  Boolean have_data[7];
  
  /* elevation  */
  have_data[0] = TRUE;
        
  /* porosity, if and only if compaction is on */
  have_data[1] = whether_compaction;  

  /* water depth */
  have_data[2] = TRUE;        

  /* max space from neighboring timeline, meters */
  have_data[3] = TRUE;        

  /* prevailing diffusion constant */
  have_data[4] = TRUE;        

  /* temperature, if and only if thermal calculations are being done */
  have_data[5] = did_thermal; 

  /* Sum TTI, if and only if thermal calculations are being done */
  have_data[6] = did_thermal; 
  
  for (i = 0; i < 7; ++i)  /* loop through the data fields */
    if (have_data[i])      /* skip those not calculated */
      for (j = 0; j < timelines_done; ++j)
	for (k = 0; k < number_spatial_nodes; ++k)
	  write_datum(fp, basin_data[i][j][k]);
}
  

void write_datum(FILE *fp, double x)
{
  Boolean negmantis, negexpon;   /* mantis / exponent negative? */
  unsigned int mantis, expon;
  unsigned char buf[3];
  double y;

  if (negmantis = (x < 0))
    x = -x;
  else if (!x) {
    fwrite("\0\0\0", 1, 3, fp);
    return;
  }

  y = log(x) / .69314718 - 17;   /* binary log(x) = ln(x) / ln(2)  */
  if ((int)y == y) ++y;
  if (negexpon = (y < 0))
    expon = -y;
  else
    expon = y;

  if (expon > 31) 
    if (negexpon) {
      fwrite("\0\0\0", 1, 3, fp);
      return;
    }
    else {
      printf("Simulator crashes and burns:  can't handle %g (too big)\n", x);
      exit(1);
    }
  
  if (negexpon)
    mantis = x * (1 << expon);
  else
    mantis = ((int)x) >> expon;

  buf[1] = (mantis & 65280) >> 8;
  buf[2] = mantis & 255;

  buf[0] = expon;
  if (negexpon) buf[0] += 32;
  if (negmantis) buf[0] += 64;
  if (mantis & 65536) buf[0] += 128;

  fwrite(buf, 1, 3, fp);
}


/* PLOTTER */
/* for films, syntax is
   plotter (library).film.(tag) -MakeIntoMovie (other arguments) 
   i.e. filmbasin takes the library and the tag and gloms them
   and the .film. middle together and passes that to the plotter
   as a single string, since the plotter doesn't need the pieces */


Boolean movie;
char *name;
int sock;

void main(int argc, char **argv)
{
  char temp[1000];
  FILE *fp;

  movie = (argc > 2 && !strcmp(argv[2], "-MakeIntoMovie"));

  if (movie) {
    name = argv[1];
    sscanf(argv[3], "%d", &sock); /* socket that filmbasin set up */
    
    /* get #digits */
    read(sock, temp, 2);
    temp[2] = '\0';
    sscanf(temp, "%d", &max_number_digits_in_timeslice_age);

    /* get #frames */
    i = 0;
    while ((read(sock, temp + i, 1) == 1) && temp[i++] != '.');
    temp[--i] = '\0';
    sscanf(temp, "%d", &nframes);
    read_dat_dot_out_file();
  }

  /* set up the console etc; set the callback for the list of ages
     such that when one is selected, movie_main_loop() is entered
     instead of the plotter's usual main loop */
}

extern Display *display;

void movie_main_loop(Widget parent, XtPointer callData, XtPointer clientData)
{
  XEvent event;
  int n, ndigits, m;
  char temp[1000];
  FILE *fp;
  
  /* tell simulator it can start */
  write(sock, "s", 1);
  
  /* #digits in frame numbers... NOT #digits in timeslice ages */
  ndigits = (int) floor(log10((double)nframes)) + 1;
  
  /* simulator writes a "p" to indicate the plotter should plot, and
     a "q" to indicate it should quit */
  n = 0;
  while (read(sock, temp, 1) == 1 && temp[0] != 'q') {
    ++n;

    /* plotbasin needs this, or Sync; you may or may not */
    XFlush(display);
    while (XCheckMaskEvent(display, ExposureMask, &event))
      XtDispatchEvent(&event);
    
    read_dat_dot_out_file();
    read_s_dot_out_file();
    
    sprintf(temp, "%sn.out", name);
    fp = fopen(temp, "r");
    read_basin_data_raw_bytes(fp);
    fclose(fp);

    /* done reading; signal to simulator */
    write(sock, "x", 1);

    turn_raw_bytes_into_numbers();

    /* plotbasin needs this (and not Flush); you may or may not */
    XSync(display, False);
    while (XCheckMaskEvent(display, ExposureMask, &event))
      XtDispatchEvent(&event);

    check_data_scales();  /* if you do scaling of color data, etc */
    
    display_basin_picture();
    display_timeslice_ages(parent);

    /* plotbasin needs this */
    XSync(display, False);

    sprintf(temp, "%s.%0*d.gif", name, ndigits, n);
    dump_canvas_area_to_gif(temp); 
  }
  /* all done! */

  movie_cleanup();

  /* clean up X stuff */
  XtDestroyWidget(top_level_shell);
  XtDestroyApplicationContext(app);
  XFlush(display);

  exit(0);
}
 
void movie_cleanup()
{
  char temp[1000];

  close(sock);
 
  /* lose the files the simulator left */
  sprintf(temp, "%sdat.out", library);
  unlink(temp);
  sprintf(temp, "%sn.out", library);
  unlink(temp);
  sprintf(temp, "%ss.out", library);
  unlink(temp);
}
