

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h"
#include "boomheader.h"
#include <math.h>


/* only figure once per slice; 
   don't call when slice == -1 */

double kmarine, knonmarine;
double **k[NSLICES];  /* k[slice][node][layer] */
double **cond[NSLICES];  /* heat conductivity [slice][node][layer] */
double condsand, condfluid, condshale;
double compcut, compdecay;

void boomk()
{
  int n, layer;
  double x, max, min;

  if (k[slice]) return;  /* only do this once per slice... */
  
  boomalloc(&k[slice]);
  
  if (compcut < 0) {
    x = knonmarine - kmarine;
    for (n = 0; n < nnodes; ++n)
      for (layer = 0; layer < timeloop[slice]; ++layer) 
	k[slice][n][layer] = 
	  (timeline[slice][4][layer][n] - kmarine) / x;
  }
  else {
    for (n = 0; n < nnodes; ++n)
      for (layer = 0; layer < timeloop[slice]; ++layer)
	if ((x = (timeline[slice][2][layer][n] - compcut)) < 0)
	  k[slice][n][layer] = 1;
	else
	  k[slice][n][layer] = exp(- compdecay * x);
  }

  max = k[slice][0][0];
  min = k[slice][0][0];
  for (n = 0; n < nnodes; ++n)
    for (layer = 0; layer < timeloop[slice]; ++layer)
      if (k[slice][n][layer] > max)
	max = k[slice][n][layer];
      else if (k[slice][n][layer] < min)
	min = k[slice][n][layer];

  datascale[COMPOS][0].end[0][slice] = min;
  datascale[COMPOS][0].end[1][slice] = max;

  boomalloc(&cond[slice]);
  x = condsand - condshale;
  for (n = 0; n < nnodes; ++n)
    for (layer = 0; layer < timeloop[slice]; ++layer)
      cond[slice][n][layer] = condfluid * timeline[slice][1][layer][n] +
	(1-timeline[slice][1][layer][n]) * (condshale + x* k[slice][n][layer]);
  max = cond[slice][0][0];
  min = cond[slice][0][0];
  for (n = 0; n < nnodes; ++n)
    for (layer = 0; layer < timeloop[slice]; ++layer)
      if (cond[slice][n][layer] > max)
	max = cond[slice][n][layer];
      else if (cond[slice][n][layer] < min)
	min = cond[slice][n][layer];
  datascale[CONDUCT][0].end[0][slice] = min;
  datascale[CONDUCT][0].end[1][slice] = max;
}


