

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h"
#include "boomheader.h"


extern double boomresp(int, double, Boolean);

double **sig[NSLICES][FUNCS];  /* [slice][func][node][time] */
double maxsig[NSLICES][FUNCS][2];  /* last is watermode */

int funcmode;
Boolean watermode;

static struct {
  double *var;
  double old[NSLICES][FUNCS];
} boompar[] = {
  {&vfluid}, {&vsand}, {&vshale},
  {&rhofluid}, {&rhosand}, {&rhoshale}, 
  {&nu}, {&phi}};

static double boom_sig_base;  
static double boom_sig_slope; 
static int boom_sig_number; 

void boomsig(void)
{
  int n, i;
  double unit, t, maxdelay, s, arg, high, low, maxt;
  Boolean bool;

  bool = movie;
  if (!sig[slice][funcmode]) bool = True;
  for (i = 0; i < 8; ++i)
    if (*boompar[i].var != boompar[i].old[slice][funcmode])
      boompar[i].old[slice][funcmode] = *boompar[i].var, bool = True;
  
  if (!bool) return;

  if (sig[slice][funcmode]) {
    for (n = 0; n < nnodes; ++n)
      free(sig[slice][funcmode][n]);
    free(sig[slice][funcmode]);
  }

  sig[slice][funcmode] = (double **) calloc(nnodes, sizeof(double *));

  maxdelay = 0;
  for (n = 0; n < nnodes; ++n)
    if (delay[slice][n][0] > maxdelay)
      maxdelay = delay[slice][n][0];

  unit = SIGINV / nu;
  boom_sig_number = (int)(maxdelay / unit) + 2 * SIGPTS + 2;
  boom_sig_base = - (1 + phi) * SIGPTS * unit;
  boom_sig_slope = 
    (maxdelay + 2 * SIGPTS * unit) / (double)(boom_sig_number-1);
  maxt = boom_sig_base + boom_sig_slope * (boom_sig_number - 1);

  for (i = 0; i < 2; ++i)
    maxsig[slice][funcmode][i] = 0;

  progressname("convolving");

  for (n = 0; n < nnodes; ++n) {
    progressvalue((double)n/(double)nnodes);

    high = nu * (maxt - delay[slice][n][timeloop[slice] - 1]) + phi;
    low = nu * (boom_sig_base - delay[slice][n][0]) + phi;
    if (high < 0) high = -high;
    if (low < 0) low = -low;
    bool = (low < high);

    sig[slice][funcmode][n] = (double *) 
      calloc(boom_sig_number, sizeof(double));
    for (i= 0, t= boom_sig_base; i < boom_sig_number; ++i, t+= boom_sig_slope){
      s = boomresp(n, t, bool); 
      if (s > maxsig[slice][funcmode][0])
	maxsig[slice][funcmode][0] = s;
      else if (-s > maxsig[slice][funcmode][0])
	maxsig[slice][funcmode][0] = -s;
      if (!watermode)
	sig[slice][funcmode][n][i] = s;
      arg = nu * (t - delay[slice][n][timeloop[slice] - 1]) + phi;
      if (arg >= -1 && arg <= 1)
	s += rc[slice][n][timeloop[slice] - 1] *
	  wave[funcmode].val[(int)(SIGPTS * (arg + 1))];
      if (s > maxsig[slice][funcmode][1])
	maxsig[slice][funcmode][1] = s;
      else if (-s > maxsig[slice][funcmode][1])
	maxsig[slice][funcmode][1] = -s;
      if (watermode)
	sig[slice][funcmode][n][i] = s;
    }
  } 

  signalends();

  progressclear();
}


double seismicsignal(int n, double t)
{
  int i;
  double s, arg;
  
  i = (t - boom_sig_base) / boom_sig_slope; 

  if (i < 0 || i >= boom_sig_number)
    return 0;
   
  s = sig[slice][funcmode][n][i];
  
  if (watermode)
    if (((arg = (nu * (t - delay[slice][n][timeloop[slice] - 1]) + phi)) <= 1)
	&& (arg >= -1))
      s += rc[slice][n][timeloop[slice] - 1] *
	wave[funcmode].val[(int)(SIGPTS * (arg + 1))];
  
  return s;
}


double boomresp(int node, double t, Boolean bottumup)
{
  int layer, limit;
  double response, arg;
  
  limit = timeloop[slice] - 1;
  
  arg = 0;
  response = 0;
  if (bottumup) {
    for (layer = 0; arg <= 1 && layer < limit; ++layer)
      if (((arg = (nu * (t - delay[slice][node][layer]) + phi)) <= 1)
	  && (arg >= -1))
	response += rc[slice][node][layer] * 
	  wave[funcmode].val[(int)(SIGPTS * (arg + 1))];
  }
  else 
    for (layer = limit - 1; arg >= -1 && layer >= 0; --layer)
      if (((arg = (nu * (t - delay[slice][node][layer]) + phi)) >= -1)
	  && (arg <= 1))
	response += rc[slice][node][layer] *
	  wave[funcmode].val[(int)(SIGPTS * (arg + 1))];
  
  return response;
}

  
