

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h"


extern void colorsig(void);

extern double **v[], **rho[], **imped[], **delay[];
extern double **k[], **cond[], **thgrad[];

Boolean logdisp;
double nodenum;

void colorlogs()            
{
  int i, j, major, minor;
  twodoub *block;  /* [i][0] x, [i][0] y */
  double ***boomdat, boommax, boommin, boomrange;
  double x, max;

  major = logdata / 100;
  minor = logdata % 100;

  if (SIGNAL == major) {
    colorsig();
    return;
  }

  /* allocate space to compose figures to be drawn */
  block = (twodoub *) calloc(timeloop[slice] * tlstep + 5, sizeof(*block));

  /* indicate in the Progress box that logging is going on */
  progressname("logging");

  switch (major) {     
    /* seismic max and min determination all done at once, below */
  case VELOCITY:
    boom('V');
    boomdat = v;
    setcolor("red"); 
    break;
  case DENSITY:
    boom('D');
    boomdat = rho;
    setcolor("blue");
    break;
  case IMPED: 
    boom('I');
    boomdat = imped;
    setcolor("purple");
    break;
  case COMPOS:
    boom('C');
    boomdat = k;
    setcolor("green");
    break;
  case FACIES:
    if (datascale[FACIES][minor].scaled[1])
      max = TWOMAX(datascale[FACIES][minor].use[0], 
		   datascale[FACIES][minor].use[1]);
    else
      max = TWOMAX(datascale[FACIES][minor].end[0][slice], 
		   datascale[FACIES][minor].end[1][slice]);
    setcolor("grey");
    break;
  case CONDUCT:
    boom('C');
    boomdat = cond;
    setcolor("grey");
    break;
  case THGRAD:
    boomdat = thgrad;
    setcolor("grey");
    break;
  case POROSITY: 
  case DIFFUSION:
  case THERM:
  case TTI:
  case ISOTOPE:
    setcolor("grey");
    break;
  default:  
    setcolor("gold");
    break;
  }
  if (major != SIGNAL) {  /* not that this gets called then anyway */
    if (datascale[major][minor].scaled[1]) {
      boommax = datascale[major][minor].use[1];
      boommin = datascale[major][minor].use[0];
    }
    else {
      boommax = datascale[major][minor].end[1][slice];
      boommin = datascale[major][minor].end[0][slice];
    }
    if (COMPOS == major)
      boommax = -boommax;
  }
  boomrange = boommax - boommin;
  
  if (TWTIME == axismode)
    boom('T');
  
  for (i = 1; i < nnodes; i += nodenum) {
    /* indicate progress in the Progress box */
    progressvalue((double)i / nnodes);   

    block[0][0] = i * delta;
    
    /* find how far from the baseline the log should extend there,
       according to the type of data being displayed */
    for (j = 1; j < timeloop[slice]; ++j) {
      switch (major) {
      case FACIES:
	x = timeline[slice][2][j][i] / max;
	break;
      case POROSITY:
	x = (timeline[slice][1][j][i] - boommin) / boomrange;
	break;
      case DIFFUSION:
	x = timeline[slice][4][j][i] / boommax;
	break;
      case COMPOS: case VELOCITY: case DENSITY: case IMPED: 
      case CONDUCT: case THGRAD:
	x = (boomdat[slice][i-1][j-1] - boommin) / boomrange;
	break;
      case THERM:
	x = (timeline[slice][5][j][i] - boommin) / boomrange;
	break;
      case TTI:
	x = (timeline[slice][6][j][i] - boommin) / boomrange;
	break;
      case ISOTOPE:
	x = (isotope[minor][j] - boommin) / boomrange;
	break;
      }
      
      block[j][0] = (i + datascale[major][minor].amp * x) * delta;
            
      /* set the vertical positions of the corners of the trapezoid */
      if (DEPTH == axismode)
	block[j][1] = timeline[slice][0][j][i];
      else  /* TWTIME == axismode */
	block[j][1] = delay[slice][i][j];
    }
    block[0][1] = block[1][1];
    block[timeloop[slice]][0] = block[0][0];
    block[timeloop[slice]][1] = block[timeloop[slice]-1][1];
    if (logdisp)
      draw(canvas, timeloop[slice] + 1, block, 'P');
    draw(canvas, timeloop[slice] + 1, block, 'L');
  }

  progressclear();   /* clear the Progress box */
  
  free(block);
}
