

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h"
#include "boomheader.h"


extern double depthtotime(double depth, int node);
extern Boolean reset_param(double *);
extern double seismicsignal(int, double);

Boolean sigmode;


void signalends(void)
{
  if (datascale[SIGNAL][0].use[0] <
      (datascale[SIGNAL][0].end[0][slice] = 
       maxsig[slice][funcmode][watermode ? 1 : 0])) {
    datascale[SIGNAL][0].use[0] = datascale[SIGNAL][0].end[0][slice];
    reset_param(&datascale[SIGNAL][0].use[0]);
  }
}


void colorsig(void)
{
  int i, n, height;
  Dimension dim;
  double x, t, vert, dvert, sigmax, vertmax, vertmin;
  twodoub *pt[2];

  XtVaGetValues(canvas, XtNheight, &dim, NULL);
  height = dim;

  for (i = 0; i < 2; ++i) 
    pt[i] = (twodoub *) calloc(height + 2, sizeof(*pt[i]));

  boom('S');

  signalends();

  sigmax = (datascale[SIGNAL][0].scaled[1]
	    ? datascale[SIGNAL][0].use[0] 
	    : maxsig[slice][funcmode][watermode ? 1 : 0]);

  if (TWTIME == axismode) {
    vertmin = twtmin;
    vertmax = twtmax;
  }
  else {  /* DEPTH */
    vertmin = ymax;
    vertmax = ymin;
  }
  dvert = (vertmax - vertmin) / (double)height;

  progressname("tracing");

  for (n = 0; n < nnodes; n += nodenum) {
    progressvalue((double)n/(double)nnodes);
    pt[0][0][0] = pt[1][0][0] = pt[0][height + 1][0] = pt[1][height + 1][0] 
      = n * delta;
    pt[0][0][1] = pt[1][0][1] = vertmin;
    pt[0][height + 1][1] = pt[1][height + 1][1] = vertmax; 
    vert = vertmin;
    for (i = 0; i < height; ++i) {
      pt[0][i + 1][1] = pt[1][i + 1][1] = vert;
      t = ((TWTIME == axismode) ? vert : depthtotime(vert, n));
      if ((x = datascale[SIGNAL][0].amp * seismicsignal(n, t) / sigmax) > 0) {
	pt[0][i + 1][0] = (n + x) * delta;
	pt[1][i + 1][0] = n * delta;
      }
      else {
	pt[0][i + 1][0] = n * delta;
	pt[1][i + 1][0] = (n + x) * delta;
      }
      vert += dvert;
    }  
    if (sigmode) {
      setcolor("white");
      draw(canvas, height + 2, pt[0], 'P');
      setcolor("black"); /* needs to be 2nd */
      draw(canvas, height + 2, pt[0], 'L');
      draw(canvas, height + 2, pt[1], 'P');
    }
    else {
      setcolor("blue");
      draw(canvas, height + 2, pt[0], 'P');
      setcolor("red");
      draw(canvas, height + 2, pt[1], 'P');
      setcolor("black"); 
    }
    pt[0][0][0] = pt[0][1][0] = n * delta;
    pt[0][0][1] = vertmin;
    pt[0][1][1] = vertmax;
    /* color should already be black */
    draw(canvas, 2, pt[0], 'L'); 
  }

  progressclear();

  for (i = 0; i < 2; ++i) 
    free(pt[i]);
}
