

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h"
#include <math.h>
#include "Xj/Canvas.h"

/* When color data is to be displayed --- water depth, porosity, diffusion,
   etc --- this routine is what does it.  It colors in the spaces in the 
   canvas appropriately and puts up the colorbar and percentbar at the top 
   to indicate what the colors mean.  It prints the units used on the right 
   of the horizbar at the top.  For Wheeler diagrams it also puts in time 
   indications, on the left, and a sealevel graph, on the right.*/

extern void interpolate(void);
extern double mapfacies(double, int);

extern double **k[], **v[], **rho[], **imped[], **delay[];
extern double **cond[], **thgrad[];
extern XFontStruct *fontstruct;

double seascale, isoscale[100];
double wheeleruse;

void colorspace()            
{
  int i, j, a, b, major, minor;
  double x, y, z;
  twodoub *block, quad[4];
  double ***boomdat, boommax, boommin, boomrange, mult, units;
  Dimension width, height;
  Arg args[2];
  XPoint pt[5];
  char s[100], note[100];
  Pixmap pmap, pmap2, pmap3;
  Pixel hiatusgrey, vacuitygrey, grey;
  Datascale *ds;

  major = data[wheeler] / 100;
  minor = data[wheeler] % 100;
  ds = &datascale[major][minor];

  if (SIGNAL == major) {
    interpolate();
    return;
  }
  
  /* allocate space to compose figures to be drawn; must be at least 
     timeloop[slice] * tlstep to be able to draw all the pts in the 
     sealevel or shoreboundary */
  block = (twodoub *) calloc(timeloop[slice] * tlstep + 5, sizeof(*block));
  
  XtSetArg(args[0], XtNheight, &height);
  XtSetArg(args[1], XtNwidth, &width);
  XtGetValues(colorbar, args, 2);
  
  /* find the greys that indicate accumulation lacunae */
  if (wheeler) {
    hiatusgrey = setcolor("DimGrey");
    vacuitygrey = setcolor("grey");
  }
  
  /* indicate in the Progress box that coloring is going on */
  progressname("coloring");
  
  pmap = CanvasPixmap(colorbar);
  
  /* paint spectrum in colorbar; %s marked much later */
  a = (int) (width * .05);
  b = (int) (width * .95);
  y = (double) (b - a);
  pt[0].y = pt[1].y = pt[4].y = height;
  pt[2].y = pt[3].y = 0;
  for (i = a; i < b; ++i) {
    x = (i - a) / y;
    if (ACCUM == major || MAXACCUM == major || DIFFUSION == major || 
	THERM == major || TTI == major || THGRAD == major || CONDUCT == major
	|| (FACIES == major && 1 == modelnum))
      x = 1 - x;
    XSetForeground(display, gc, topixel(x));
    pt[0].x = pt[3].x = pt[4].x = i;
    pt[1].x = pt[2].x = i + 1;
    XFillPolygon(display, pmap, gc, pt, 5, Complex, CoordModeOrigin);
  }

  switch (major) {
  case VELOCITY:
    boom('V');
    boomdat = v;
    break;
  case DENSITY:
    boom('D');
    boomdat = rho;
    break;
  case IMPED: 
    boom('I');
    boomdat = imped;
    break;
  case COMPOS:
    boom('C');
    boomdat = k;
    /* scaling overrides (then switches) if on */
    boommax = 0;
    boommin = 1;
    break;
  case CONDUCT:
    boom('C');
    boomdat = cond;
    break;
  case THGRAD:
    boomdat = thgrad;
    break;
  default:
    /* others don't care about this */
    break;
  }
  
  
  if (ds->scaled[0]) {
    boommax = ds->use[1];
    boommin = ds->use[0];
  }
  else {
    boommax = ds->end[1][slice];
    boommin = ds->end[0][slice];
  }
  if (COMPOS == major) {
    x = boommin;
    boommin = boommax;
    boommax = x;
  }
  boomrange = boommax - boommin;
  
  if (ACCUM == major) 
    mult = .0001 / (timedelt * tlstep);  /* meters --> mm / yr  */
  
  if (TWTIME == axismode)
    boom('T');

  for (i = 0; i < nnodes - 1; ++i) {
    /* indicate progress in the Progress box */
    progressvalue((double)i / (double)nnodes);   
    
    /* set the right and left edges of the trapezoids to be colored in */
    quad[0][0] = quad[3][0] = i * delta;
    quad[1][0] = quad[2][0] = (i + 1) * delta;
    
    /* find how far along the spectrum the area should be colored, according
       to the type of data being displayed */
    for (j = 1; j < timeloop[slice]; ++j) {
      switch (major) {
      case FACIES:
	x = mapfacies
	  ((timeline[slice][2][j][i+1] + timeline[slice][2][j][i]
	    + timeline[slice][2][j-1][i+1] +timeline[slice][2][j-1][i]) / 4, 
	   0); /* fill, not log */
	break;
      case POROSITY:
	x = (timeline[slice][1][j][i+1] + timeline[slice][1][j][i]
	     + timeline[slice][1][j-1][i+1] +timeline[slice][1][j-1][i] ) /4;
	if (ds->scaled[0]) 
	  x = (x - ds->use[0]) / boomrange;
	break;
      case ACCUM:
	x = 1 - pow((mult / boommax) * 
		    (timeline[slice][0][j][i] - timeline[slice][0][j-1][i]),
		    (1.0 / ds->power[0]));
	break;
      case MAXACCUM:
	x = 1 - pow(timeline[slice][3][j][i] / boommax, (1.0 / ds->power[0]));
	break;
      case DIFFUSION:
	x = 1 - pow(timeline[slice][4][j][i] / boommax, ds->power[0]);
	break;
      case THERM:
	x = 1 - ((timeline[slice][5][j][i] - boommin) / boomrange);
	break;
      case TTI:
	x = 1 - ((timeline[slice][6][j][i] - boommin) / boomrange);
	break;
      case COMPOS: case VELOCITY: case DENSITY: case IMPED: 
      	x = (boomdat[slice][i][j] - boommin) / boomrange;
	break;
      case CONDUCT: case THGRAD:
	x = 1 - ((boomdat[slice][i][j] - boommin) / boomrange);
	break;
      case ISOTOPE:
	x = (isotope[minor][j] - boommin) / boomrange;
	break;
      }
      
      /* Wheeler displays use greys for accumulation lacunae */
      if (wheeler && (timeline[slice][0][j][i] == timeline[slice][0][j-1][i])
	  && (((grey = (timeline[slice][3][j-1][i] ? vacuitygrey : hiatusgrey))
	       != vacuitygrey) || major != MAXACCUM))
	XSetForeground(display, gc, grey);
      else {
	if (x < 0) x = 0;
	else if (x > 1) x = 1;
	if (FACIES == major && 1 == modelnum) 
	  x = 1 - x;
	XSetForeground(display, gc, topixel(x));
      }      
      
      /* set the vertical positions of the corners of the trapezoid */
      if (WHEELER == axismode) {
	quad[0][1] = quad[1][1] = ymin + wheeleruse *.01 * (ymax - ymin) 
	  * (double)(j-1) / (timeloop[NSLICES-1]-1);
	quad[2][1] = quad[3][1] = ymin + wheeleruse * .01 * (ymax - ymin) 
	  * (double)j / (timeloop[NSLICES-1]-1);
      }
      else if (DEPTH == axismode) {
	quad[0][1] = timeline[slice][0][j-1][i];
	quad[1][1] = timeline[slice][0][j-1][i+1];
	quad[2][1] = timeline[slice][0][j][i+1];
	quad[3][1] = timeline[slice][0][j][i];
      }
      else { /* TWTIME == axismode */
	quad[0][1] = delay[slice][i][j-1];
	quad[1][1] = delay[slice][i+1][j-1];
	quad[2][1] = delay[slice][i+1][j];
	quad[3][1] = delay[slice][i][j];
      }
      draw(canvas, 4, quad, 'P');  /* draw the polygon (the trapezoid) */
    }
  }

  progressclear();   /* clear the Progress box */
  
  if (wheeler) {
    pmap = CanvasPixmap(canvas);

    /* reference lines and labels */
    XtGetValues(canvas, args, 2);
    setcolor("white");
    setgroundcolor("black", False);
    x = .1 * timedelt * tlstep * timeloop[5];  /* 10% total time */
    a = log10(x);
    b = pow(10.0, (double)a);    
    for (i = 1; i <= 10; ++i) {
      sprintf(s, "%2de%d", (int)(i * x / b), a);
      XDrawImageString(display, pmap, gc, 0, 
		       (j = height * (1 -(double)(i * wheeleruse) / 1e3)) +4,
		       s, strlen(s));
      XDrawLine(display, pmap, gc, XTextWidth(fontstruct, s, strlen(s)), 
		j, width, j);
    }
    
    /* things going evenly along y (t) */
    for (i = 0; i < timeloop[slice]; ++i)
      block[i][1] = ymin + 
	wheeleruse * .01 * (ymax - ymin) * (double) i / (timeloop[5] - 1);

    /* axis for sealevel, isotopes */
    setcolor("purple");
    quad[0][0] = quad[1][0] = xmax - .1 * (xmax - xmin);
    quad[0][1] = ymin;
    quad[1][1] = ymin + wheeleruse * .01 * (ymax-ymin) *
      ((double)(timeloop[slice] - 1) / (timeloop[5] - 1));
    draw(canvas, 2, quad, 'T');

    /* shoreline */
    if (shorevis) { 
      setcolor("white");
      for (i = 0; i < timeloop[slice]; ++i) 
	block[i][0] = shoreboundary[i] * delta;
      draw(canvas, timeloop[slice], block, 'L');
    }
    
    /* sealevel */    
    setcolor("SkyBlue");
    for (i = 0; i < timeloop[slice]; ++i) 
      block[i][0] = xmax - .1 * (xmax - xmin) * (1 + sealevel[i] / seascale);
    draw(canvas, timeloop[slice], block, 'T');
    /* scale, scale label */  
    quad[0][0] = xmax - .1 * (xmax - xmin);
    quad[1][0] = xmax - .2 * (xmax - xmin);
    quad[0][1] = quad[1][1] = ymin + wheeleruse * .0005 * (ymax - ymin);
    draw(canvas, 2, quad, 'T');
    sprintf(s, "+%d m", (int)seascale);
    XDrawImageString(display, pmap, gc, width - XTextWidth(fontstruct, s,
							   strlen(s)), 
		     (int)((double)height * (1 - .0005 * (double)wheeleruse)),
		     s, strlen(s));
    
    /* isotope, done like sealevel */
    if (ISOTOPE == major) {
      setcolor("LimeGreen");
      for (i = 0; i < timeloop[slice]; ++i) 
	block[i][0] = xmax - .1 * (xmax - xmin) * 
	  (1 + isotope[minor][i] / isoscale[minor]);
      draw(canvas, timeloop[slice], block, 'T');
      /* scale, scale label */
      quad[0][0] = xmax - .1 * (xmax - xmin);
      quad[1][0] = xmax - .2 * (xmax - xmin);
      quad[0][1] = quad[1][1] = ymin + wheeleruse * .0015 * (ymax - ymin);
      draw(canvas, 2, quad, 'T');
      sprintf(s, "+%d %s", (int)isoscale[minor], isounits[minor]);
      XDrawImageString(display, pmap, gc, width - XTextWidth(fontstruct, s,
							     strlen(s)), 
		       (int)((double)height * (1-.0015 * (double)wheeleruse)),
		       s, strlen(s));
    }    
  }
  
  /* set up the colorbar and percentbar; put unit display in horizbar */
  XtGetValues(colorbar, args, 2);
  setcolor("black");
  pmap = CanvasPixmap(colorbar);
  pmap2 = CanvasPixmap(percentbar);
  pmap3 = CanvasPixmap(horizbar); 
  switch (major) {
  case FACIES:
    x = (boommax > boommin ? boommax : boommin);
    if (x > 10000) 
      units = pow(10.0, 3.0 * (((int)log10(x)) / 3));
    else
      units = 0;
    if (modelnum)
      sprintf(s, "color units:  %%sand");
    else 
      if (units)
	sprintf(s, "color units:  10e%d meters", (int)log10(units));
      else
	sprintf(s, "color units:  meters");
    XDrawString(display, pmap3, gc, 
                width - XTextWidth(fontstruct, s, strlen(s)) - 20, 10,     
  		s, strlen(s));
    for (x = 0; x < 1.1; x += .1) {
      if (x < (z = ds->boundary)) 
	y = - (1 - pow(x / z, 1.0 / ds->power[i = 0]));
      else
	y = pow((x - z) / (1 - z), 1.0 / ds->power[i = 1]);
      y *= (ds->scaled[0] ? ds->use[i] : ds->end[i][slice]);
      if (units)
	y /= units;
      if (modelnum)
        sprintf(s, "%d%%",(int)(100 * y));
      else 
	if (y < 10 && y > -10)
	  sprintf(s, "%+.2f", y);
	else
	  sprintf(s, "%d", (int)y);
      i = (.9 * x + .05) * width;
      XDrawLine(display, pmap, gc, i, 0, i, height);
      XDrawString(display, pmap2, gc, 
		  i - XTextWidth(fontstruct, s, strlen(s)) /2,
  		  15, s, strlen(s));
    }
    break;
  case POROSITY:
    for (x = 0; x < 1.1; x += .1) {
      y = (ds->scaled[0] ? ds->use[0] + x * (ds->use[1] - ds->use[0]) : x);
      sprintf(s, "%d%%", (int)(100 * y));
      i = (.9 * x + .05) * width;
      XDrawLine(display, pmap, gc, i, 0, i, height);
      XDrawString(display, pmap2, gc, 
		  i - XTextWidth(fontstruct, s, strlen(s)) / 2,
		  15, s, strlen(s));
    }
    break;
  case DIFFUSION:
    if (boommax > 10000) {
      units = pow(10.0, 3.0 * (((int)log10(boommax)) / 3));
      sprintf(s, "color units:  10e%d meters^2 / year", (int)log10(units));
    }
    else {
      units = 0;
      sprintf(s, "color units:  meters^2 / year");
    }
    XDrawString(display, pmap3, gc,    
                width - XTextWidth(fontstruct, s, strlen(s)) - 20, 10,
                s, strlen(s));
    for (x = 0; x < 1.1; x += .1) {
      y = boommax *  pow(x, 1.0 / ds->power[0]);
      if (units)
	y /= units;
      if (y < 10)
	sprintf(s, "%.2f", y);
      else
	sprintf(s, "%d", (int)y);
      i = (.9 * x + .05) * width;
      XDrawLine(display, pmap, gc, i, 0, i, height);
      XDrawString(display, pmap2, gc, 
		  i - XTextWidth(fontstruct, s, strlen(s)) / 2,
		  15, s, strlen(s));
    }
    break;
  case ACCUM:  
  case MAXACCUM:
    if (boommax > 10000) {
      units = pow(10.0, 3.0 * (((int)log10(boommax)) / 3));
      sprintf(s, "color units:  10e%d mm / yr", (int)log10(units));
    }
    else {
      units = 0;
      sprintf(s, "color units:  millimeters / year");
    }
    XDrawString(display, pmap3, gc,
                width - XTextWidth(fontstruct, s, strlen(s)) - 20, 10,
                s, strlen(s));
    for (x = 0; x < 1.1; x += .1) {
      y = boommax * pow(x, ds->power[0]);
      if (units)
	y /= units;
      if (ACCUM == major) 
	y /= mult;
      if (y < 10)
	sprintf(s, "%.2f", y);
      else
	sprintf(s, "%d", (int)y);
      i = (.9 * x + .05) * width;
      XDrawLine(display, pmap, gc, i, 0, i, height);
      XDrawString(display, pmap2, gc, 
		  i - XTextWidth(fontstruct, s, strlen(s)) / 2,
		  15, s, strlen(s));
    }
    break;
  case COMPOS: case VELOCITY:  case DENSITY:  case IMPED: 
  case CONDUCT: case THERM: case TTI: case THGRAD:
  case ISOTOPE:
    if (boommax > 10000) {
      units = pow(10.0, 3.0 * (((int)log10(boommax)) / 3));
      sprintf(note, " 10e%d", (int)log10(units));
    }
    else {
      units = 0;
      strcpy(note, "");
    }
    switch (major) {
    case COMPOS: strcpy(s, "color units: %sand"); break;
    case VELOCITY: sprintf(s, "color units: %s meters / sec", note); break;
    case DENSITY: sprintf(s, "color units: %s kg / meter^3", note); break;
    case IMPED: sprintf(s, "color units: %s kg / (meter^2 sec)", note); break;
    case CONDUCT: sprintf(s, "color units: %s watts / meter deg C",note);break;
    case THERM: sprintf(s, "color units: %s deg C", note); break;
    case TTI: sprintf(s, "color units: %s megayears", note); break;
    case THGRAD: sprintf(s, "color units: %s deg C / meter", note); break;
    case ISOTOPE: sprintf(s, "color units: %s %s",note,isounits[minor]); break;
    }
    XDrawString(display, pmap3, gc, 
		width - XTextWidth(fontstruct, s, strlen(s)) - 20, 10,
		s, strlen(s));
    for (x = 0; x < 1.1; x += .1) {
      y = boommin + x * boomrange;
      if (units)
	y /= units;
      if (COMPOS == major)
	y *= 100;
      if (major != COMPOS && y < 10 && y > -10)
	sprintf(s, "%.2f", y);
      else
	sprintf(s, "%d", (int)y);
      i = (.9 * x + .05) * width;
      XDrawLine(display, pmap, gc, i, 0, i, height);
      XDrawString(display, pmap2, gc,
		  i - XTextWidth(fontstruct, s, strlen(s)) / 2,
		  15, s, strlen(s));
    }
    break;
  case SIGNAL:
    /* colorspace() doesn't handle this, interpolate() does */
    break;
  }

  free(block);
}
