

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h"
#include <math.h>
#include "Xj/Canvas.h"


extern double **k[], **v[], **rho[], **imped[], **cond[], **thgrad[];
extern char dot[];  /* from picture.c */
extern XFontStruct *fontstruct;

extern double vertpos(int t, int n);
extern void dealcontour(void);

void contour()
{
  int i, j, n, a, b, major, minor;
  double step, min, max, level, **dat, f, g, mult, div, corner[2][2];
  double units, x;
  twodoub p[4];
  Boolean figuredat, anycon, nobase;
  XGCValues gcvalues;
  unsigned long gcmask;
  char s[100], t[100];
  Dimension width;

  major = condata / 100;
  minor = condata % 100;

  figuredat = False;
  switch (major) {
  case COMPOS:
    boom('C');
    dat = k[slice];
    break;
  case VELOCITY:
    boom('V');
    dat = v[slice];
    break;
  case DENSITY:
    boom('D');
    dat = rho[slice];
    break;
  case IMPED:
    boom('I');
    dat = imped[slice];
    break;
  case CONDUCT:
    boom('C');
    dat = cond[slice];
    break;
  case THGRAD:
    dat = thgrad[slice];
    break;
  case ISOTOPE:
    return;  /* shouldn't even call here! 
		ISOTOPE is NOT a contour-able datum!  think about it. */
    break;
  default:
    figuredat = True;
  }

  dealcontour();
  if ((step = datascale[major][minor].interval) <= 0)
    return;

  if (figuredat) {
    dat = (double **) calloc(nnodes, sizeof(double *));
    for (i = 0; i < nnodes; ++i)
      dat[i] = (double *) calloc(timeloop[slice], sizeof(double));
    
    switch (major) {
    case FACIES:
      for (i = 0; i < nnodes; ++i)
	for (j = 0; j < timeloop[slice]; ++j)
	  dat[i][j] = timeline[slice][2][j][i];
      break;
    case POROSITY: 
      for (i = 0; i < nnodes; ++i)
	for (j = 0; j < timeloop[slice]; ++j)
	  dat[i][j] = timeline[slice][1][j][i];
      break;
    case ACCUM:
      for (i = 0; i < nnodes; ++i) {
	dat[i][0] = 0;
	for (j = 1; j < timeloop[slice]; ++j)
	  dat[i][j] = timeline[slice][0][j][i] - timeline[slice][0][j - 1][i];
      }
      break;
    case MAXACCUM:
      for (i = 0; i < nnodes; ++i) 
	for (j = 0; j < timeloop[slice]; ++j)
	  dat[i][j] = timeline[slice][3][j][i];
      break;
    case DIFFUSION:
      for (i = 0; i < nnodes; ++i)
	for (j = 0; j < timeloop[slice]; ++j)
	  dat[i][j] = timeline[slice][4][j][i];
      break;
    case THERM:
      for (i = 0; i < nnodes; ++i)
	for (j = 0; j < timeloop[slice]; ++j)
	  dat[i][j] = timeline[slice][5][j][i];
      break;
    case TTI:
      for (i = 0; i < nnodes; ++i)
	for (j = 0; j < timeloop[slice]; ++j)
	  dat[i][j] = timeline[slice][6][j][i];
      break;
    }
  }

  if (TWTIME == axismode)
    boom('T');
  
  max = step * (int)(datascale[major][minor].end[1][slice] / step);
  if (condata != FACIES) {
    min = step * (int) (datascale[major][minor].end[0][slice] / step);
    if (min > 0)
      min = 0;
  }
  else
    min = -step * (int)(datascale[major][minor].end[0][slice] / step);
  
  mult = step / (double)nnodes;
  div = max - min + step;

  progressname("contouring");

  gcmask = GCLineStyle;
  gcvalues.line_style = LineOnOffDash;
  XChangeGC(display, gc, gcmask, &gcvalues);
  XSetDashes(display, gc, 0, dot, 2);
  setcolor("DimGrey");
  nobase = True;
  anycon = False;
  for (level = min; level <= max; level += step) {
    for (i = 1; i < nnodes; ++i) {
      progressvalue((level - min + i * mult) / div);
      for (j = 1; j < timeloop[slice]; ++j) {
	n = 0;
	for (a = 0; a < 2; ++a)
	  for (b = 0; b < 2; ++b)
	    corner[a][b] = vertpos(j - 1 + b, i - 1 + a);
	
	f = dat[i-1][j-1], g = dat[i-1][j];
	if ((f > level && g < level) || (f < level && g > level)) {
	  p[n][0] = (i-1) * delta;
	  p[n][1] = ((f - level) * corner[0][1] + (level - g) * corner[0][0])
	    / (f - g);
	  ++n;
	}
	
	f = g, g = dat[i][j];
	if ((f > level && g < level) || (f < level && g > level)) {
	  p[n][0] = (i - 1 + ((f - level) / (f - g))) * delta;
	  p[n][1] = ((f - level) * corner[1][1] + (level - g) * corner[0][1])
	    / (f - g);
	  ++n;
	}
	
	f = g, g = dat[i][j-1];
	if ((f > level && g < level) || (f < level && g > level)) {
	  p[n][0] = i * delta;
	  p[n][1] = ((f - level) * corner[1][0] + (level - g) * corner[1][1])
	    / (f - g);
	  ++n;
	}
	
	f = g, g = dat[i-1][j-1];
	if ((f > level && g < level) || (f < level && g > level)) {
	  p[n][0] = (i - 1 + ((level - g) / (f - g))) * delta;
	  p[n][1] = ((f - level) * corner[0][0] + (level - g) * corner[1][0])
	    / (f - g);
	  ++n;
	}
	
	if (n > 0) {
	  draw(canvas, n, p, (n < 4) ? 'L' : 'P');
	  anycon = True;
	}
      }
    }
    if (nobase && anycon) {
      gcvalues.line_style = LineSolid;
      XChangeGC(display, gc, gcmask, &gcvalues);
      if ((x = level) > 10000) {
	units = pow(10.0, 3.0 * (((int)log10(x)) / 3));
	x /= units;
      }
      else
	units = 0;
      strcpy(s, "base contour:  ");
      if (x < 10 && x > -10)
	sprintf(t, "%.2f", x ? x : 0);
      else
	sprintf(t, "%d", (int)x);
      strcat(s, t);
      if (units) {
	sprintf(t, "e%d", (int)log10(units));
	strcat(s, t);
      }
      XtVaGetValues(horizbar, XtNwidth, &width, NULL);
      XDrawString(display, CanvasPixmap(horizbar), gc,
		  width - XTextWidth(fontstruct, s, strlen(s)) - 20, 30,
		  s, strlen(s)); 
      nobase = False;
    }
  }  
  
  progressclear();
  
  /* just in case... */
  gcvalues.line_style = LineSolid;
  XChangeGC(display, gc, gcmask, &gcvalues);
  
  if (figuredat) {
    for (i = 0; i < nnodes; ++i)
      free(dat[i]);
    free(dat);
  }
}  
