

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h" 
#include "Xj/Canvas.h"


/* draw in the canvas of the Canvas widget passed as 'widget' either 
   a series of line segments (if 'figure' is 'L') or a filled polygon
   (if 'figure' is 'P'); the x, y of the nth point are 'array'[n-1][0, 1]
   and the total number of pts is 'numpts.'  Or thick lines, for 'T'.

   The figures in 'array' are in the simulation's units:  probably meters,
   horizontally; probably meters for Depth and seconds for Twtime vertically.
   They get converted to pixels.  */

double yshift, vexag;
double xmin, xmax, ymin, ymax;
double twtmin, twtmax;
int axismode;

void draw(Widget widget, int numpts, twodoub array[], char figure)
{
  XPoint *pt;
  int i, width, height;
  double del;
  Dimension dim1, dim2;
  Pixmap pmap = CanvasPixmap(widget);
  XGCValues gcvalues;
  unsigned long gcmask;

  pt = (XPoint *) calloc(numpts+1, sizeof(XPoint)); 
  XtVaGetValues(widget, XtNwidth, &dim1, XtNheight, &dim2, NULL);
  width = dim1;
  height = dim2;

  del = xmax - xmin;
  for (i = 0; i < numpts; ++i) 
    pt[i].x = width * (array[i][0] - xmin) / del;
  if (axismode != TWTIME) {
    del = ymax - ymin;
    for (i = 0; i < numpts; ++i)
      pt[i].y = height * (ymax - array[i][1] + yshift * !wheeler) / del;
  }
  else {
    del = twtmax - twtmin;
    for (i = 0; i < numpts; ++i)
      pt[i].y = height * (array[i][1] - twtmin) / del;
  }

  switch (figure) {
  case 'P':
    XFillPolygon(display, pmap, gc, pt, numpts, Complex, CoordModeOrigin);
    break;
  case 'L':
    XDrawLines(display, pmap, gc, pt, numpts, CoordModeOrigin);
    break;
  case 'T':
    gcmask = GCLineWidth;
    gcvalues.line_width = 3;
    XChangeGC(display, gc, gcmask, &gcvalues);
    XDrawLines(display, pmap, gc, pt, numpts, CoordModeOrigin);
    gcvalues.line_width = 0;
    XChangeGC(display, gc, gcmask, &gcvalues);
    break;
  default:
    printf("Fatal error:  can't %c-draw\n", figure);
    exit(1);
    break;
  }

  free(pt);
}
