

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include <stdio.h>
#include <stdlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>



/* define, and for some set, the global variables of the program */

#define ABOVE .25              /* fraction of view above zero level          */
#define NSLICES 6              /* number of slices                           */
#define SIMDATA 7              /* # of data fields simulator gives           */


#define TWOMAX(A, B) ((A)> (B) ? (A) : (B))
#define THREEMAX(A, B, C) ((A) > (B) ? TWOMAX((A), (C)) : TWOMAX((B), (C)))
#define TWOMIN(A, B) ((A) < (B) ? (A) : (B))
#define THREEMIN(A, B, C) ((A) < (B) ? TWOMIN((A), (C)) : TWOMIN((B), (C)))

/* I'd use typedef enum for these various lists, which mostly just have to
   by interally distinguishable, but some machines don't know it. 
   Note in some cases specific #s are important, typically where the last 
   or penultimate one 'counts' the others. */

/* axis modes */

enum {WHEELER = 0, DEPTH, TWTIME};  /* WHEELER must be 0 */

/* distinct types of data (fill and log) */
enum {FACIES = 0, POROSITY, ACCUM, MAXACCUM, DIFFUSION, VELOCITY, DENSITY,
	IMPED, COMPOS, SIGNAL, CONDUCT, THERM, THGRAD, TTI, 
        ISOTOPE,  /* could be multiple subtypes */
	NODATA, OFF};  /* nodata counts, off counts incl nodata */

/* defined spectra */
enum {FULL = 0, SMALL, GREY, REDBLUE, 
      NSPECTRA}; 

typedef double twodoub[2];

extern char datacomposlabel[]; /* set in parameters.c */

extern int axismode;          /* vertical axis:  DEPTH, TWTIME, or WHEELER  */

extern int compaction;       /* whether porosity is meaningful             */

extern double compcut, compdecay;

extern int condata;           /* as data, logdata for contour display       */

/* heat conductivities; 0s for off */
extern double condsand, condshale, condfluid; 

extern double cut[NSLICES];            /* basin slice ages             */

extern int data[2];         /* color data being shown, normal and wheeler */

typedef struct {
  String label[2];
  int major[2];    /* FACIES, ISOTOPE, etc */
  int minor[2];    /* which facies, which isotope, etc; usually zero */
  Widget obj[3];
} Datachoice;
extern Datachoice datachoice[100];  
int nchoices;

typedef struct {
  double end[2][NSLICES], use[2], power[2], boundary;
  Boolean scaled[2];  /* 0 fill, 1 log */
  int amp; /* log amp */
  double interval;  /* contour interval */
} Datascale;
extern Datascale datascale[100][100]; /* scaling info, [gentype][subtype] */

extern double delta;                   /* spacing of nodes               */

extern Boolean frozen;        /* do not update screen when param changes    */

extern Boolean frozechange;           /* holding a change frozen           */

extern char isoname[100][100];
extern char isounits[100][100];
extern double *isotope[100];
extern int nisotopes;

extern String library;       /* library to read from, inc'g path; no .dat  */

extern Boolean linesvis;              /* timelines visible                 */

extern int logdata;                   /* like "data" for log display       */

extern int modelnum;           /* 0 = Peter Flemings', 1 = Jen Carlson's     */

extern Boolean movie;                 /* are we making a movie?            */

extern int nisotopes;                 /* currently 0 or 1 */

extern int nnodes;                    /* number of nodes                   */

extern double nodenum;                 /* display logs every nodenum nodes  */

extern double plotstep;                /* plot every n timelines (int)     */

extern Boolean reflecmode;            /* show reflection coeffs            */

extern double *sealevel;               /* sea level curve                   */

extern int *shoreboundary;            /* boundary of rt shoreline          */

extern Boolean shorevis;              /* shoreline visible                */

extern int slice;                /* which slice the user has selected, 0+ */

extern Boolean sigvis;                /* seismic signal visible            */

extern double timedelt;                /* duration of each time STEP        */

extern double **timeline[NSLICES][SIMDATA];
                               /* 0 elevation, 1 porosity, 2 water depth,
				  3 max space (accum), 4 diff const,
				  5 temperature, 6 TTI                 */

extern int timeloop[NSLICES];         /* # of timelines in the cut          */

extern int tlstep;                    /* extra resolution for shore and sea  */

extern double twtmin, twtmax;          /* min & max twtime for that mode    */

extern Boolean usecolor;       /* whether to use color instead of monochrome */

extern double version;                /* simulator version number          */

extern double vexag;                   /* vertical exaggeration for plot    */

extern Boolean watermode;          /* include seabottom reflection signals */

extern int wheeler;                   /* 1 if in wheeler diagram else 0   */

extern double wheeleruse;         /* % of height for oldest wheeler to use */

extern double xmax, xmin, ymax, ymin;  /* sides of plotted window         */

extern double yshift;                  /* shift the vertical axis          */


/*                X stuff                                              */

extern XtAppContext app;               

extern Display *display;       

extern GC gc;

extern Widget alsograb[];    /* widgets to also grab when using Messenger  */

extern Widget anglebox, anglebar;
extern Widget canvas;                 /* the main canvas (drawing area)  */
extern Widget colorbar;      /* the slim canvas displaying sample spectrum */
extern Widget condatabtn;
extern Widget condial, conval;
extern Widget filldatabtn;
extern Widget logdatabtn, logampbtn;
extern Widget percentbar;        /* slim canvas labelling colorbar with %s  */
extern Widget horizbar;      /* the canvas with sample horizontal lengths  */
extern Widget scalefill, scalelog;
extern Widget toplevel;               /* the top-level (Shell) widget    */
extern Widget vertbar;       /* canvas displaying sample vertical lengths  */
extern Widget viewport;

extern Pixel black, white;

extern void boom(char);
extern void draw(Widget, int, twodoub a[], char);
extern void picture(void);
extern void progressname(String);
extern void progressvalue(double);
extern void progressclear(void);
extern Pixel setcolor(String), setgroundcolor(String, Boolean);
extern Pixel topixel(double);

extern Boolean XjMessenger(Widget, String, Boolean, char a[], char s[],
			   Widget, Widget *);
