

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h"
#include <X11/Xatom.h> 
#include <math.h>
#include <string.h>


#define STRATA_DEBUG 0

#define MAXCOLS 256  /* per mode */

Boolean greyvis, usecolor;
Colormap cmap;
int spectrum[NSPECTRA][MAXCOLS];
int ncols[NSPECTRA];
int spectralmode = FULL;
int screenint, screendepth;
Visual *visual;
Pixel black, white;

/* better include Black and White! */
String colornames[] = { "Black", "Blue", "DimGrey", "Gold", "Green", "Grey",
			"LightGrey", "LimeGreen", "Purple", "Red", "SkyBlue",
			"Violet", "White", "Yellow" };

static struct {
  int map;
  struct {
    double r;
    double g;
    double b;
  } spec[10];
} bow[] = {
  {FULL, {
    {.5, 0, 0},
    {1, 0, 0},
    {1, .5, 0},
    {1, 1, 0},   /* SMALL starts here, 3/8 of the way in; '5/8' used below */
    {.5, 1, .5},
    {0, 1, 1},
    {0, .5, 1},
    {0, 0, 1},
    {0, 0, .5},
    {-1, -1, -1}}},
  {REDBLUE, {
    {1, 0, 0},
    {1, 1, 1},
    {0, 0, 1},
    {-1, -1, -1}}},
  {GREY, {
    {1, 1, 1},
    {.25, .25, .25},
    {-1, -1, -1}}} };
/* SMALL does not get its own bow; it's a subbow of FULL */

void initialize()
{
  XColor color, exact;
  int i, j, k, m, n, p;
  Window win;
  XVisualInfo visinfo;

  screenint = DefaultScreen(display);
  win = XRootWindow(display, screenint);
  visual = DefaultVisual(display, screenint);
  visinfo.visualid = visual->visualid;
  screendepth = XGetVisualInfo(display, VisualIDMask, &visinfo, &n)->depth;

  greyvis = (StaticGray == visual->class || GrayScale == visual->class);

  cmap = XCreateColormap(display, win, visual, AllocNone);

  for (i = 0; i < XtNumber(colornames); ++i) {
    n = XAllocNamedColor(display, cmap, colornames[i], &exact, &color);
    if (STRATA_DEBUG && !n) printf("Failed to initialize %s\n", colornames[i]);
    if (!strcasecmp(colornames[i], "black")) black = color.pixel;
    if (!strcasecmp(colornames[i], "white")) white = color.pixel;
  }

  n = pow(2.0, (double)screendepth) - XtNumber(colornames);
  if (greyvis) {
    ncols[GREY] = TWOMIN(MAXCOLS, n);
    ncols[FULL] = ncols[REDBLUE] = ncols[SMALL] = 0;
  }
  else {
    ncols[FULL] = TWOMIN(MAXCOLS, n / 2);
    ncols[GREY] = ncols[REDBLUE] = TWOMIN(MAXCOLS, (n - ncols[FULL]) / 2);
    ncols[SMALL] = ncols[FULL] * 5 / 8;  /* just copies index from FULL */
  }  

  color.flags = DoRed | DoGreen | DoBlue;
  for (i = 0; i < XtNumber(bow); ++i) 
    if (ncols[bow[i].map]) {
      for (n = 0; bow[i].spec[n].r >= 0; ++n);
      m = ncols[bow[i].map] / (n - 1) - 1;
      p = 0;
      for (j = 1; j < n; ++j) 
	for (k = 0; k <= m; ++k) {
	  color.red =   (k * bow[i].spec[j].r + (m - k) * bow[i].spec[j-1].r) 
	    * 65535 / m;
	  color.blue =  (k * bow[i].spec[j].b + (m - k) * bow[i].spec[j-1].b)
	    * 65535 / m;
	  color.green = (k * bow[i].spec[j].g + (m - k) * bow[i].spec[j-1].g)
	    * 65535 / m;
	  XAllocColor(display, cmap, &color);
	  spectrum[bow[i].map][p++] = color.pixel;
	}
      ncols[bow[i].map] = p;
    }
  n = ncols[FULL] - ncols[SMALL];
  for (i = 0; i < ncols[SMALL]; ++i)
    spectrum[SMALL][i] = spectrum[FULL][n + i];
  
  if (STRATA_DEBUG) {
    switch(visual->class) {
    case StaticGray:  printf("StaticGray "); break;
    case StaticColor: printf("StaticColor "); break;
    case GrayScale:  printf("GrayScale "); break;
    case PseudoColor: printf("PseudoColor "); break;
    case DirectColor: printf("DirectColor "); break;
    case TrueColor: printf("TrueColor "); break;
    }
    printf("%d, %d (%d)\n", screendepth, ncols[FULL], ncols[GREY]);
  }    

  XtVaSetValues(toplevel, XtNcolormap, cmap, NULL);
}


Pixel topixel(double x) 
{
  return spectrum[spectralmode][(int)(x * (ncols[spectralmode] - 1))];
}


Pixel setgroundcolor(String name, Boolean fore) 
{
  XColor exact, color;

  if (XAllocNamedColor(display, cmap, name, &exact, &color)) {
    if (fore)
      XSetForeground(display, gc, color.pixel);
    else
      XSetBackground(display, gc, color.pixel);
    
    return color.pixel;
  }

  if (STRATA_DEBUG)
    printf("Error: plotbasin unable to use \"%s\"\n", name);
  return black;
}


Pixel setcolor(String name) { return setgroundcolor(name, True); }


void setspectrum()   /* caller is responsible for calling picture() */
{
  int major = data[wheeler] / 100;
  
  if (greyvis || !usecolor)
    spectralmode = GREY;
  else 
    if (modelnum && FACIES == major && !datascale[FACIES][0].scaled[0])
      spectralmode = SMALL;
    else if (SIGNAL == major)
      spectralmode = REDBLUE;
    else
      spectralmode = FULL;
}
