

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h"
#include "boomheader.h"
#include <math.h>
#include "Xj/Canvas.h"

extern XFontStruct *fontstruct;
extern int screendepth;
extern Visual *visual;

extern double depthtotime(double depth, int node);
extern double seismicsignal(int, double);

void interpolate()
{
  Dimension dim1, dim2;
  int width, height;
  int i, n, m, m1, m2, step, nz, nmin, nmax, nstart;
  double t, r1, r2, r, dr, z, sigmax, y;
  XPoint pt[5];
  Pixmap pmap, pmap2;
  XImage *image;
  char s[100];

  boom('S');
  signalends();
  sigmax = (datascale[SIGNAL][0].scaled[0] 
	    ? datascale[SIGNAL][0].use[0] 
	    : datascale[SIGNAL][0].end[0][slice]);

  /* paint spectrum in colorbar */
  XtVaGetValues(colorbar, XtNheight, &dim1, XtNwidth, &dim2, NULL);
  height = dim1;
  width = dim2;
  pmap = CanvasPixmap(colorbar);
  m1 = .05 * width;
  m2 = .95 * width;
  r = m2 - m1;
  pt[0].y = pt[1].y = pt[4].y = height;
  pt[2].y = pt[3].y = 0;
  for (i = m1; i < m2; ++i) {
    XSetForeground(display, gc, topixel((i - m1) / r));
    pt[0].x = pt[3].x = pt[4].x = i;
    pt[1].x = pt[2].x = i + 1;
    XFillPolygon(display, pmap, gc, pt, 5, Complex, CoordModeOrigin);
  } 
  /* put lines in colorbar, put %s in percentbar */
  pmap2 = CanvasPixmap(percentbar);
  setcolor("black");
  for (z = 0; z < 1.1; z += .1) {
    n = (.9 * z + .05) * width;
    t = (2 * z - 1);
    if (t < 0)
      r = -pow(-t, 1 / datascale[SIGNAL][0].power[0]);
    else
      r = pow(t, 1 / datascale[SIGNAL][0].power[0]);
    r *= sigmax;
    sprintf(s, "%1.1f%%", 100 * r);
    XDrawLine(display, pmap, gc, n, 0, n, height);
    XDrawString(display, pmap2, gc,
		n - XTextWidth(fontstruct, s, strlen(s)) / 2,
		15, s, strlen(s));
  }
  sprintf(s, "color units:  percentage of pulse amp");
  XDrawString(display, CanvasPixmap(horizbar), gc,
	      width - XTextWidth(fontstruct, s, strlen(s)) - 20, 
	      10, s, strlen(s));

  XtVaGetValues(canvas, XtNheight, &dim1, XtNwidth, &dim2, NULL);
  height = dim1;
  width = dim2;

  nmin = xmin / delta /* +1 */;
  nmax = xmax / delta;
  z = width / (double)(nmax - nmin);
  nstart = (nmin < 0 ? 0 : nmin);
  if (nmax > nnodes) nmax = nnodes;

  image = XCreateImage(display, visual, screendepth, ZPixmap, 0, None, 
		       width, height, 32, 0);
  image->data = XtMalloc(image->bytes_per_line * height);

  progressname("interpolating");

  for (i = 0; i < height; ++i) {
    progressvalue((double)i / height);
    if (TWTIME == axismode) 
      t = twtmin + (twtmax - twtmin) * (double)i / (double)(height - 1);
    else {  /* DEPTH */
      y = ymax + (ymin - ymax) * (double)i / (double)(height - 1);
      t = depthtotime(y, nstart);
    }
    r2 = seismicsignal(nstart, t) / sigmax;
    for (n = nstart; n < nmax; ++n) {
      r1 = r2;
      if (TWTIME == axismode)
	r2 = seismicsignal(n, t) / sigmax;
      else  /* DEPTH */
	r2 = seismicsignal(n, depthtotime(y, n)) / sigmax;
      if (datascale[SIGNAL][0].power[0] != 1)
	if (r2 > 0)
	  r2 = pow(r2, 1 / datascale[SIGNAL][0].power[0]);
	else
	  r2 = -pow(-r2, 1 / datascale[SIGNAL][0].power[0]);
      dr = r2 - r1;

      nz = (n - nmin) * z;
      step = (int)((n - nmin + 1) * z) - nz;
      for (m = 0; m < step; ++m) { 
	r = r1 + m * dr / step;
	XPutPixel(image, nz + m, i, topixel(.5 * (r + 1)));
      }
    }
  }
  XPutImage(display, CanvasPixmap(canvas), gc, image, 
	    0, 0, 0, 0, width, height);
  XtFree((XtPointer)image->data); 
  XFree((void *)image); 
  progressclear();
}

