

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h"
#include "Xj/Canvas.h"
#include <string.h>


/* dump (part) of the picture into an xwd file with user-specified name.
   Dump either entire window (entire), just the part of the canvas in view
   and the scrollbars (view), or the whole canvas (seen and unseen) but 
   naught else (canvas).  The movie routine uses the dump_canvas routine. 
   After any dump (except during movies) create a <dump name>.specs file
   with all possibly-relevant parameters.  */


extern void gifdump(Window, Pixmap, FILE *, Dimension, Dimension);
extern void Window_Dump(Display *, Window, Pixmap, unsigned, unsigned, FILE*); 
extern void do_saveparams(FILE *);

static void dump_win(Window win, Boolean isxwd, String s)
{
  FILE *fp;
  XEvent event;

  XMapRaised(display, XtWindow(toplevel));
  XSync(display, False);

  while (XCheckMaskEvent(display, ExposureMask, &event))
    XtDispatchEvent(&event);

  if (isxwd) {
    fp = fopen(s, "w");
    Window_Dump(display, win, (Pixmap)NULL, 0, 0, fp);
  }
  else {
    fp = fopen(s, "wb");
    gifdump(win, (Pixmap)NULL, fp, 0, 0);
  }

  fclose(fp);
}



static Dimension canvas_width, canvas_height;
Window toplevel_win;
static Pixmap canvas_pixmap;

void dump_canvas(Boolean isxwd, String s)
{
  FILE *fp;

  if (!movie || !canvas_pixmap) {
    XtVaGetValues(canvas, XtNwidth, &canvas_width, XtNheight, &canvas_height,
		  NULL);
    toplevel_win = XtWindow(toplevel);  /* has colormap installed */
    canvas_pixmap = CanvasPixmap(canvas);
  }
  
  if (isxwd) {
    fp = fopen(s, "w");
    Window_Dump(display, toplevel_win, canvas_pixmap, canvas_width, 
		canvas_height, fp);
  }
  else {
    fp = fopen(s, "wb");
    gifdump(toplevel_win, canvas_pixmap, fp,  canvas_width, canvas_height);
  }
  
  fclose(fp);
}


void specfile(String s) 
{
  FILE *fp;
  char f[1000];

  sprintf(f, "%s.specs", s);
  fp = fopen(f, "w");
  do_saveparams(fp);
  fclose(fp);
}


void makedump(Widget widget, XtPointer clientData, XtPointer callData)  
{
  char s[100];
  Widget w = XtParent(widget);
  Boolean isxwd;
  int n;

  if (XjMessenger(w, "Dump to                   ", True, s, 
		  NULL, NULL, alsograb)) 
    if ((n = strlen(s))) {
      isxwd = (n > 4 && !strcasecmp(s + n - 4, ".xwd"));
      XBell(display, 50);
      switch ((int)clientData) {
      case 0:  
	dump_win(XtWindow(toplevel), isxwd, s);  
	break;
      case 1:  
	dump_win(XtWindow(viewport), isxwd, s); 
	break;
      case 2:  
	dump_canvas(isxwd, s);  
	break;
      }
      XBell(display, 50);
      XBell(display, 50);
      specfile(s);  /* below */
    }
    else XjMessenger(w, "File unspecified --- undumped.", False, NULL,
		     NULL, NULL, alsograb);
}  
