 

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h"


/* initialize to defaults the parameters of the picture (not those of
   the simulation).  The paramstuff.c and params.h code allows the user
   to change these at or during runtime.  */

extern int funcmode;
extern Boolean logdisp, sigmode;
extern double vfluid, vsand, vshale;
extern double rhofluid, rhosand, rhoshale;
extern double nu, phi;

char datacomposlabel[15];

Datascale datascale[100][100];

/* omitted minors default to zero */
Datachoice datachoice[100] = {
  {{"Empty", "Water depth"}, {NODATA, FACIES}},
  {{"Water depth", "Accumulation"}, {FACIES, ACCUM}},
  {{"Porosity", "Max accum"}, {POROSITY, MAXACCUM}},
  {{"Diffusion", "Diffusion"}, {DIFFUSION, DIFFUSION}},
  {{datacomposlabel, ""}, {COMPOS, OFF}},
  {{"Velocity", ""}, {VELOCITY, OFF}},
  {{"Density", ""}, {DENSITY, OFF}},
  {{"Impedance", ""}, {IMPED, OFF}},
  {{"Signal", ""}, {SIGNAL, OFF}},
  {{"Heat cond.", ""}, {CONDUCT, OFF}},
  {{"Temperature", ""}, {THERM, OFF}},
  {{"Temp. grad.", ""}, {THGRAD, OFF}},
  {{"Sum TTI", ""}, {TTI, OFF}},
  {{NULL, NULL}} /* isotopes get inserted here */
};

int nchoices;


void parameters()
{               
  int i, j, k, major, minor;

  /* no slice selected yet */
  slice = -1;

  for (nchoices = 0; datachoice[nchoices].label[0] != NULL; ++nchoices);
  for (i = 0; i < nisotopes; ++i) {
    for (j = 0; j < 2; ++j) {
      datachoice[nchoices].label[j] = isoname[i];
      datachoice[nchoices].major[j] = ISOTOPE;
      datachoice[nchoices].minor[j] = i;
    }
    ++nchoices;
  }

  /* initialize to defaults: */
  data[0] = 100 * NODATA;   /* no color data in nonWheeler mode */
  data[1] = 100 * FACIES;   /* water depth data in Wheeler mode */

  /* generalities, for used (major, minor) combos */
  for (i = 0; i < nchoices; ++i) 
    for (j = 0; j < 2; ++j) {
      major = datachoice[i].major[j];
      minor = datachoice[i].minor[j];
      if (major < NODATA) {
	datascale[major][minor].amp = 10;
	datascale[major][minor].interval = 0;
	for (k = 0; k < 2; ++k)
	  datascale[major][minor].scaled[k] = False;
	datascale[major][minor].use[0] = +9e9;
	datascale[major][minor].use[1] = -9e9;
	for (k = 0; k < NSLICES; ++k) {
	  datascale[major][minor].end[0][k] = +9e9;
	  datascale[major][minor].end[1][k] = -9e9;
	}
      }
    }

  /* specifics */
  for (i = 0; i < 2; ++i) {
    datascale[FACIES][0].use[i] = 0;
    datascale[POROSITY][0].use[i] = datascale[COMPOS][0].use[i] = 1 - i;
    for (j = 0; j < NSLICES; ++j) {
      datascale[FACIES][0].end[i][j] = 0;
      datascale[POROSITY][0].end[i][j] = 1 - i;  /* default stretch min, max */
      datascale[COMPOS][0].end[i][j] = i;  /* default full scale (0, 1) */
    }
  }

  datascale[FACIES][0].boundary = .3;
  datascale[FACIES][0].power[0] = 1;
  datascale[FACIES][0].power[1] = 1;  
  datascale[DIFFUSION][0].power[0] = 1;
  datascale[ACCUM][0].power[0] = 2;
  datascale[MAXACCUM][0].power[0] = 2;
  datascale[SIGNAL][0].power[0] = 1;

  logdata = 100 * NODATA;
  condata = 100 * NODATA;

  sprintf(datacomposlabel, "Composition (%c)", compcut < 0 ? 'D' : 'W');

  axismode = DEPTH;
  frozen = False;
  funcmode = 0;
  linesvis = True;
  logdisp = False;
  nodenum = 10;
  shorevis = False;
  sigmode = False;
  sigvis = False;
  wheeler = 0;
  plotstep = 1;
  vexag = 100;
  watermode = False;
  wheeleruse = 50;

  twtmin = 0;
  twtmax = 1;

  vfluid = 1604;
  vsand = 5487;
  vshale = 4545;
  rhofluid = 1000;
  rhosand = 2650;
  rhoshale = 2500;
  nu = 30;
  phi = 0;

  xmax = nnodes * delta;
  xmin = 0;
  ymax = ABOVE * (xmax - xmin) / vexag;
  ymin = (ABOVE - 1) * (xmax - xmin) / vexag;
  yshift = 0;
}

