 

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h"
#include <X11/Shell.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/Toggle.h>

extern Widget XjCreateManagedHelpButton(String, Widget, String a[][2],
					String, String, ArgList, int);


/*  Set up the widget structure and callback functions for the Parameters
    window.  It's a bunch of Dialogs each controlling one of the parameters
    of the display, and buttons at the bottom for doing things with the set. 
    The parameters and associated info are defined in the params.h file.  */


extern Boolean havedata[SIMDATA];

Boolean report = False;   
/* report toggle; report invalids (rather than correcting) */

#include "params.h"

static int numparams;

/* do any of the displayed params differ from the current values? */
static Boolean anychange()
{
  int i;
  String new;
  char old[100];
  double x, y;
  
  for (i = 0; i < numparams; ++i) {
    new = XawDialogGetValueString(param[i].dialog);
    sprintf(old, "%g", *param[i].var); 
    sscanf(new, "%lf", &x);
    sscanf(old, "%lf", &y);
    if (x - y)
      return True;
  }
  return False;
}
  
  
/* shift param window focus */
static int hascaret;
static Widget paramshell;
static void docaret(int n) {
  n = (n + numparams) % numparams;
  if (hascaret == n)
    return;
  XawTextDisplayCaret(param[hascaret].val, False);
  hascaret = n;
  XawTextDisplayCaret(param[hascaret].val, True);
  XawTextSetInsertionPoint
    (param[hascaret].val,
     strlen(XawDialogGetValueString(param[hascaret].dialog)));
  XtSetKeyboardFocus(paramshell, param[hascaret].dialog);
}
static void takecaret(Widget w, XEvent *e, String *s, Cardinal *c)
{
  int n;
  sscanf(*s, "%d", &n);
  docaret(n);
}
static void shiftcaret(Widget w, XEvent *e, String *s, Cardinal *c)
{
  int n;
  sscanf(*s, "%d", &n);
  do 
    docaret(hascaret + n);
  while (!havedata[param[hascaret].havedataindex]);
}
  


/* If the displayed values are correct, pop down the parameter window */
static void leave(Widget widget, XtPointer clientData, XtPointer callData)
{
  if (anychange())
    XjMessenger(widget, "Please reset or implement changes.", False, NULL,
		 NULL, NULL, alsograb);
  else
    XtPopdown(paramshell);
}


/* actually perform the param save; called here and via movieloop */
void do_saveparams(FILE *fp)  
{
  char s[100];
  int i;

  for (i = 0; i < numparams; ++i)  
    if (strlen(strcpy(s, XawDialogGetValueString(param[i].dialog))))
      fprintf(fp, "%-32s %20s\n", param[i].name, s);
}


/* Save the currently displayed parameter values in user-named file. */
static void saveparams(Widget widget, XtPointer clientData, XtPointer callData)
{
  char s[100];
  FILE *fp;

  if (!XjMessenger(widget, "Save as            ", True, s, 
		   NULL, NULL, alsograb)) 
    return;

  if (strlen(s)) {
    fp = fopen(s, "w");
    do_saveparams(fp);
    fclose(fp);
  }
  else   
    XjMessenger(widget, "File unspecified --- unsaved", False, NULL, NULL, 
		NULL, alsograb);
}



/* load in the parameter values from a user-specified file.  any entries
   in the file that don't match param names are ignored.  any params wo
   entries in the file are unchanged.  changes are made as they are read
   so later entries override earlier ones.  format of file is that
   produced by do_saveparams(). */
static void loadparams(Widget widget, XtPointer clientData, XtPointer callData)
{
  char str[200], format[100], sx[20];
  FILE *fp;
  int i, m;
  double x;
  Arg arg;

  if (!XjMessenger(widget, "Load file", True, str, NULL, NULL, alsograb)) 
    return;

  if (strlen(str))
    if ((fp = fopen(str, "r")) != NULL) {
      while (fgets(str, 200, fp) != NULL)
        for (i = m = 0; i < numparams && !m; ++i) {
          if (m = 
	      sscanf(str, strcat(strcpy(format, param[i].name), " %lf"), &x)) {
            sprintf(sx, "%g", x);
            XtSetArg(arg, XtNvalue, (XtArgVal) sx);
            XtSetValues(param[i].dialog, &arg, 1);
          }
        }
      fclose(fp);
    }
    else XjMessenger(widget, "File not found.", False, NULL, NULL, 
		     NULL, alsograb);
  else 
    XjMessenger(widget, "File unspecified --- unloaded", False, NULL, NULL,
		NULL, alsograb);
}


/* set the displayed values to the current values; ask for confirmation
   first if there's a difference. */
static void reset(Widget widget, XtPointer clientData, XtPointer callData)
{
  int i;
  Arg arg;
  char value[30];

  if (anychange()) 
    if (XjMessenger(widget, "Changes made.  Reset?", True, NULL, NULL, 
		    NULL, alsograb))
      for (i = 0; i < numparams; ++i) { 
        sprintf(value, "%g", *param[i].var);
        XtSetArg(arg, XtNvalue, (XtArgVal) value);
        XtSetValues(param[i].dialog, &arg, 1);
      }
}

/* reset a single parameter in the window; used by readslice() and boom*()'s
   when datascales get stretched.  the return value is generally ignored. */
Boolean reset_param(double *var)
{
  int i;
  char value[30];

  for (i = 0; i < numparams && var != param[i].var; ++i);

  if (var != param[i].var)
    return False;         /* couldn't reset, no such variable */

  sprintf(value, "%g", *param[i].var);
  XtVaSetValues(param[i].dialog, XtNvalue, (XtArgVal)value, NULL);
  return True; /* found the variable */
}
 

/* set the parameter values equal to the displayed values.  ask first
   if there's a difference.  illegal values are either just made legal
   in some fashion (see the functions in params.h; method varies with
   each parameter), or reported if the 'report' toggle is set. */
static void implement(Widget widget, XtPointer clientData, XtPointer callData)
{
  int i;
  String value, err;
  char msg[100];
  double x;
  Arg arg;
  Boolean needpicture;

  if (anychange())
    if (XjMessenger(widget, "Changes made.  Implement?", True, NULL, NULL,
		    NULL, alsograb)){
      needpicture = False;
      for (i = 0; i < numparams; ++i) {
        value = XawDialogGetValueString(param[i].dialog);
        if (sscanf(value, "%lf", &x) == 1) {
          err = XtNewString((*param[i].correct)(x));
          if (report && strlen(err))
	    XjMessenger(widget, err, False, NULL, NULL, NULL, alsograb);
          else if (strlen(err)) {
	    XtSetArg(arg, XtNvalue, (XtArgVal) err);
	    XtSetValues(param[i].dialog, &arg, 1);
	    sscanf(err, "%lf", param[i].var);
            if (!needpicture) needpicture = (*param[i].relevant)();
          }
          else {
	    if (*param[i].var != x && !needpicture) 
	      needpicture = (*param[i].relevant)();
	    *param[i].var = x;
          }
          XtFree(err);
        }
        else {
          sprintf(msg, "%s invalid", param[i].name);
          XjMessenger(widget, msg, False, NULL, NULL, NULL, alsograb);
        }
      }
      if (needpicture) picture();
    }
}


/* toggle whether to report illegal values instead of just changing them */
static void setreport(Widget widget, XtPointer clientData, XtPointer callData)
{
  report = !report;
}


/* make the widgets */

#include "paramhelp.h"

XtActionsRec action_table[] = {
  {"takecaret", takecaret},
  {"shiftcaret", shiftcaret} };


void paramstuff(Widget parent, int argc, char **argv)
{
  Widget holder, box, col[3], fake;
  Widget exit_button, reset_button, imple_button, report_toggle;
  Widget save_button, load_button, help_button;
  int i, j, m, percol, nblank;
  Boolean parg;
  Arg arg[2];
  char value[30], str[200], format[100];
  FILE *fp;

  numparams = XtNumber(param);

  /* load initial parameter files, if any */
  parg = False;

  for (i = 2; i < argc; ++i)
    if ('-' == argv[i][0])
      parg = !strcmp(argv[i], "-p");
    else
      if (parg)
	if ((fp = fopen(argv[i], "r"))) {
	  while (fgets(str, 200, fp))
	    for (j = m = 0; j < numparams && !m; ++j)
	      m = sscanf(str, strcat(strcpy(format, param[j].name), " %lf"),
			 param[j].var);
	  fclose(fp);
	}
  
  XtAppAddActions(app, action_table, XtNumber(action_table));

  paramshell = XtVaCreatePopupShell("parameters", wmShellWidgetClass, parent,
				    XtNinput, True, 
				    NULL);
  
  holder = XtVaCreateManagedWidget("param holder", formWidgetClass, paramshell,
				   XtNdefaultDistance, 0, NULL);

  percol = numparams / 3;
  if (numparams % 3) ++percol;
  nblank = 3 * percol - numparams;

  for (i = 0; i < 3; ++i)
    col[i] = XtVaCreateManagedWidget("column", boxWidgetClass, holder,
				     XtNfromVert, NULL,
				     XtNfromHoriz, i ? col[i - 1] : NULL,
				     XtNbottom, XtChainBottom,
				     XtNtop, XtChainBottom,
				     XtNleft, XtChainLeft,
				     XtNright, XtChainLeft,
				     NULL);

  for (i = 0; i < numparams; ++i) {
    sprintf(value, "%g", *param[i].var);
    param[i].dialog = 
      XtVaCreateManagedWidget
	(param[i].name, dialogWidgetClass, col[i/percol],
	 XtNvalue, (XtArgVal) value, XtNlabel, param[i].name, 
	 XtNborderColor, white,
	 XtNmappedWhenManaged, havedata[param[i].havedataindex],  
	 NULL);
    sprintf(str, "<BtnDown> : takecaret(%d) \n\
                  <Enter>   : takecaret(%d) ", i, i);
    XtOverrideTranslations(param[i].dialog, XtParseTranslationTable(str));
    param[i].val = XtNameToWidget(param[i].dialog, "value");
    XtVaSetValues(param[i].val,
		  XtNborderColor, white,
		  XtNfromVert, NULL,
		  XtNdisplayCaret, False,
		  XtNfromHoriz, XtNameToWidget(param[i].dialog, "label"),
		  NULL);
    m = (i % percol == percol - 1);
    sprintf(str, "<Key> Return : shiftcaret(+1) \n\
                  <Key> Down   : shiftcaret(+1) \n\
                  <Key> Up     : shiftcaret(-1) \n\
                  <Key> Left   : shiftcaret(%d) \n\
                  <Key> Right  : shiftcaret(%d) ",
	    m && nblank && (i/percol == 1) ? 0 : -percol,
	    m && (i/percol + nblank > 1) ? (nblank - 1) * -percol : +percol);
    XtOverrideTranslations(param[i].val, XtParseTranslationTable(str));
  }

  /* let's be pretty */
  if (numparams % 3 > 0) {
    fake = XtVaCreateManagedWidget("space holder", dialogWidgetClass, col[2],
				   XtNvalue, "", XtNlabel, "     ",
				   XtNsensitive, False,
				   XtNborderColor, white,
				   NULL);
    XtVaSetValues(XtNameToWidget(fake, "value"), XtNborderColor, white,
		  XtNfromVert, NULL, XtNdisplayCaret, False,
		  XtNfromHoriz, XtNameToWidget(fake, "label"),
		  NULL);
    if (numparams % 3 < 2) {
      fake = XtVaCreateManagedWidget("space holder", dialogWidgetClass, col[2],
				     XtNvalue, "", XtNlabel, "     ",
				     XtNsensitive, False,
				     XtNborderColor, white,
				     NULL);
      XtVaSetValues(XtNameToWidget(fake, "value"), XtNborderColor, white,
		    XtNfromVert, NULL, XtNdisplayCaret, False,
		    XtNfromHoriz, XtNameToWidget(fake, "label"),
		    NULL);
    }
  }
  
  box = XtVaCreateManagedWidget("param box", formWidgetClass, holder,
				XtNfromVert, col[1], 
				XtNfromHoriz, col[0],
				XtNbottom, XtChainBottom, 
				XtNleft, XtChainLeft, XtNtop, XtChainBottom, 
				XtNright, XtChainLeft,
				NULL);
	    
  report_toggle = XtVaCreateManagedWidget("report", toggleWidgetClass, box,
					  XtNstate, report, 
					  XtNborderColor, white, 
					  NULL);
  XtAddCallback(report_toggle, XtNcallback, setreport, NULL);
  
  imple_button = 
    XtVaCreateManagedWidget("implement", commandWidgetClass, box, 
                            XtNfromHoriz, report_toggle,
			    XtNaccelerators, XtParseAcceleratorTable
			   ("#override <Key> Return : set() notify() unset()"),
			    XtNborderColor, white, NULL);
  XtInstallAccelerators
    (XtNameToWidget(param[numparams - 1].dialog, "value"), imple_button);
  XtAddCallback(imple_button, XtNcallback, implement, NULL);
  
  reset_button = XtVaCreateManagedWidget("reset", commandWidgetClass, box, 
					 XtNfromHoriz, imple_button, 
					 XtNborderColor, white, NULL);
  XtAddCallback(reset_button, XtNcallback, reset, NULL);
  
  save_button = XtVaCreateManagedWidget("save", commandWidgetClass, box,
					XtNfromHoriz, reset_button, 
					XtNborderColor, white, NULL);
  XtAddCallback(save_button, XtNcallback, saveparams, NULL);
  
  load_button = XtVaCreateManagedWidget("load", commandWidgetClass, box,
					XtNfromHoriz, save_button, 
					XtNborderColor, white, NULL);
  XtAddCallback(load_button, XtNcallback, loadparams, NULL);

  XtSetArg(arg[0], XtNfromHoriz, load_button);
  XtSetArg(arg[1], XtNborderColor, white);
  help_button = XjCreateManagedHelpButton("help", box, paramhelp, NULL,
					  "parameters", arg, 2);

  exit_button = XtVaCreateManagedWidget("exit", commandWidgetClass, box, 
					XtNfromHoriz, help_button, 
					XtNborderColor, white, NULL);
  XtAddCallback(exit_button, XtNcallback, leave, NULL);
}


