

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h"
#include "Xj/Canvas.h"

/*  If no slice has been selected yet (slice == -1) return.
    If the picture is frozen, note that a change has been frozen and return.
    Otherwise, draw the picture.

    Picture consists of the sealevel line in blue and the line of
    fixed-distance-from-earth's-core in dashed black, if in nonWheeler mode;
    the selected color data, if any; the timelines in black, if in nonWheeler
    and the lines are visible; and the shoreline in black, if visible. */

extern void colorlogs(void);
extern void colorsig(void);
extern void colorspace(void);
extern void contour(void);
extern void scalebars(void);
extern void setspectrum(void);
extern double vertpos(int t, int n);


extern Boolean reflecmode;
extern double **rc[NSLICES];
extern double **delay[NSLICES];

static char dash[] = {6, 3};
char dot[] = {1, 6};  /* also used by contour() */

int tlstep;
int slice;
int wheeler;
double plotstep;

void picture()                   
{
  int i, j, maxpts, last;
  Arg args[2];
  Dimension width, height;
  unsigned long mask;
  XGCValues values;
  twodoub *pt;

  if (-1 == slice) return;
  if (frozen) {
    frozechange = True;
    return;
  }
  frozechange = False;

  setspectrum();

  maxpts = ((timeloop[slice] * tlstep > nnodes) ? 
	    timeloop[slice] * tlstep : nnodes);
  pt = (twodoub *) calloc(maxpts, sizeof(*pt));

  scalebars(); /* put up the horiz & vert scalebars */

  ClearCanvas(canvas);
  ClearCanvas(colorbar);
  ClearCanvas(percentbar);

  if (DEPTH == axismode) {
    /* sealevel */
    setcolor("SkyBlue");
    pt[0][0] = xmin, pt[1][0] = xmax;
    pt[0][1] = pt[1][1] = sealevel[timeloop[slice] - 1];
    draw(canvas, 2, pt, 'L');

    /* zero level */
    XtSetArg(args[0], XtNwidth, &width);
    XtSetArg(args[1], XtNheight, &height);
    XtGetValues(canvas, args, 2);
    setcolor("black");
    mask = GCLineStyle, values.line_style = LineOnOffDash;
    XChangeGC(display, gc, mask, &values);
    XSetDashes(display, gc, 0, dash, 2);
    pt[0][1] = pt[1][1] = 0;
    draw(canvas, 2, pt, 'L');
    if (yshift) {
      XSetDashes(display, gc, 0, dot, 2);
      pt[0][1] = pt[1][1] = yshift;
      draw(canvas, 2, pt, 'L');
    }
    values.line_style = LineSolid;
    XChangeGC(display, gc, mask, &values);
  }

  if (data[wheeler] / 100 != NODATA) colorspace();  /* color data; show it */

  if (!wheeler && logdata / 100 != NODATA) colorlogs();

  if (!wheeler && condata / 100 != NODATA) contour();

  if (!wheeler) {
    if (linesvis) {     /* lines */
      setcolor("black");
      progressname("plotting");
      for (i = (timeloop[slice] - 1) % (int)plotstep; 
	   i < timeloop[slice]; i += plotstep) {
        progressvalue((double)i / timeloop[slice]);
	for (j = 0; j < nnodes; ++j) {
	  pt[j][0] = delta * j;
	  pt[j][1] = vertpos(i, j);
	}
	draw(canvas, nnodes, pt, 'L');
      }
      progressclear();
    }
    if (shorevis) {     /* shoreline */
      setcolor("yellow");
      for(i = 0; i < timeloop[slice]; ++i) { 
	pt[i][0] = delta * (shoreboundary[i] + 1);
	pt[i][1] = vertpos(i, shoreboundary[i]);
      }
      draw(canvas, timeloop[slice], pt, 'L');
    }
    if (reflecmode) {  /* reflection coeffs */
      boom('R');
      setcolor("grey");
      last = timeloop[slice] - (watermode ? 0 : 1);
      for (i = 0; i < nnodes; i += nodenum) {
	pt[0][0] = delta * i;
	for (j = 0; j < last; ++j) {
	  pt[1][0] = delta * (i + 100 * rc[slice][i][j]);
	  pt[0][1] = pt[1][1] = vertpos(j, i);
	  draw(canvas, 2, pt, 'L');
	}
      }
    }
  }

  if (sigvis && !wheeler) 
    colorsig();

  ExposeCanvas(canvas);
  ExposeCanvas(colorbar);
  ExposeCanvas(percentbar);
  ExposeCanvas(horizbar);

  free(pt);
}


double vertpos(int t, int n)
{
  if (DEPTH == axismode)
    return timeline[slice][0][t][n];
  else /* TWTIME; never gets called for wheeler */
    return delay[slice][n][t];
}


void drawpic(Widget widget, XtPointer clientData, XtPointer callData)
{
  picture();
}

