

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include "globals.h"
#include <X11/Xaw/List.h>

/*  read in the data output by the simulator for the particular slice the
    user has selected, unless it's been read in already in which case just
    use it.

    the read is done in two parts; the data is read as strings and then
    translated into numbers.  it's a LOT faster to do it this way.  */

extern Boolean reset_param(double *);
extern void thermgrad(void);

static double halftothe[40]; /* for use in decoding binary input */

void readslice_init()
{
  int n;

  if (!halftothe[0]) {
    halftothe[0] = 1;
    for (n = 1; n < 40; ++n)
      halftothe[n] = .5 * halftothe[n - 1];
  }
} 

extern void readslice_readnums(FILE *fp);
extern void readslice_usenums(void);
extern void readslice_checkscales(void);

/* small but not zero! */
#define VERYSMALL 1e-9 

static Boolean already[6];   /* has slice[n] data been read in already? */

void readslice(Widget widget, XtPointer clientData, XtPointer callData)
{
  int n, width, height;
  char lib[1000], temp[100];
  XEvent event;
  Dimension dim1, dim2;
  FILE *fp;

  readslice_init();

  if (slice != (n = XawListShowCurrent(widget)->list_index)) {
    if (already[n])  /* read the data in already; just use it. */
      slice = n;
    else {
      temp[0] = n + '1';
      temp[1] = '\0';
      strcpy(lib, library);
      strcat(lib, temp); 
      strcat(lib, ".out");
      if (!(fp = fopen(lib, "r"))) {
	/* data's not there; say this and go back to previously-used slice */
        strcat(strcpy(temp, lib), " not found.");
        XjMessenger(widget, temp, False, NULL, NULL, NULL, alsograb);
        /* set (highlight) the current entry */
	XtVaGetValues(widget, XtNwidth, &dim1, XtNheight, &dim2, NULL);
	width = dim1;
	height = dim2;
        event.type = ButtonPress;
        event.xbutton.x = (slice + .5) * width / NSLICES;
        event.xbutton.y = height / 2;
        XtCallActionProc(widget, "Set", &event, NULL, 0);
        return;
      }
      already[slice = n] = True;
      readslice_readnums(fp);   /* read in data as raw bytes (strings) */
      fclose(fp);
      readslice_usenums();      /* translate the strings to numbers    */
    }
    readslice_checkscales(); /* if necessary reset the color scales */
    picture();
  }                               
}  


typedef unsigned char holdtype[3];
static holdtype *hold;
Boolean havedata[SIMDATA];  /* get in crucialspecs() */
int nhavedata;              /* get in crucialspecs() */

void readslice_readnums(FILE *fp)     /* read in the data as bytes */
{
  int i, n;

  progressname("reading"); 

  hold = (holdtype *)
    calloc(nhavedata * timeloop[slice] * nnodes, sizeof(holdtype));
  
  /* this is done in blocks instead of all at once
     solely so that progress can be shown on the progress bar */
  n = nnodes * timeloop[slice];
  for (i = 0; i < nhavedata; ++i) {
    fread(hold[i * n], 1, 3 * n, fp);
    progressvalue((i+1.0) / (double)nhavedata);
  }
  
  progressclear();
}


void readslice_usenums()      /* translate the strings into numbers */
{ 
  int i, j, k, n;
  unsigned int expon, mantis;
  double x, t;
  /* max accum must convert meters --> mm / yr */
  double factor = .0001 / (timedelt * tlstep);  
  Datascale *ds, *ds2;

  if (!timeloop[slice])  /* only happens in movies */
    return;

  x = nhavedata * timeloop[slice] * nnodes - 1;
  progressname("processing");
  for (n = i = 0; i < SIMDATA; ++i)
    if (havedata[i])
      for (j = 0; j < timeloop[slice]; ++j) { 
	for (k = 0; k < nnodes; ++k, ++n)
	  if (hold[n][0] || hold[n][1] || hold[n][2]) {
	    mantis =
	      ((hold[n][0] & 128) << 9) | (hold[n][1] << 8) | hold[n][2];
	    expon = hold[n][0] & 31;
	    if (hold[n][0] & 32) 
	      t = mantis * halftothe[expon];
	    else
	      t = mantis << expon;
	    if (hold[n][0] & 64)
	      timeline[slice][i][j][k] = -t;
	    else
	      timeline[slice][i][j][k] = t; 
	  }
	  else
	    timeline[slice][i][j][k] = 0;
	progressvalue(n / x);
      }
  progressclear();

  free(hold);
  
  ds = &datascale[FACIES][0];
  if (modelnum) {
    ds->end[0][slice] = 0;
    ds->end[1][slice] = 1;
  }
  else {
    /* zero is always in the range, good model or not; have to use it */
    ds->end[0][slice] = ds->end[1][slice] = 0;
    for (j = 0; j < timeloop[slice]; ++j)
      for (k = 0; k < nnodes; ++k) 
	if (timeline[slice][2][j][k] > ds->end[1][slice])
	  ds->end[1][slice] = timeline[slice][2][j][k];
	else if (timeline[slice][2][j][k] < -ds->end[0][slice])
	  ds->end[0][slice] = -timeline[slice][2][j][k];
  }

  if (compaction) { /* if porosity is turned on */ 
    ds = &datascale[POROSITY][0];
    ds->end[0][slice] = ds->end[1][slice] = timeline[slice][1][0][0];
    for (j = 0; j < timeloop[slice]; ++j)
      for (k = 0; k < nnodes; ++k) 
	if (timeline[slice][1][j][k] > ds->end[1][slice])
	  ds->end[1][slice] = timeline[slice][1][j][k];
	else if (timeline[slice][1][j][k] < ds->end[0][slice])
	  ds->end[0][slice] = timeline[slice][1][j][k];
  }
  
  if (havedata[5]) { /* if thermal stuff is turned on */
    ds = &datascale[THERM][0];
    ds2 = &datascale[TTI][0];

    ds->end[0][slice] = ds->end[1][slice] = timeline[slice][5][0][0];
    ds2->end[0][slice] = ds2->end[1][slice] = timeline[slice][6][0][0];

    for (j = 0; j < timeloop[slice]; ++j)
      for (k = 0; k < nnodes; ++k) {
	if (timeline[slice][5][j][k] > ds->end[1][slice])
	  ds->end[1][slice] = timeline[slice][5][j][k];
	else if (timeline[slice][5][j][k] < ds->end[0][slice])
	  ds->end[0][slice] = timeline[slice][5][j][k];
	
	if (timeline[slice][6][j][k] > ds2->end[1][slice])
	  ds2->end[1][slice] = timeline[slice][6][j][k];
	else if (timeline[slice][6][j][k] < ds2->end[0][slice])
	  ds2->end[0][slice] = timeline[slice][6][j][k];
      }
    thermgrad();
  }

  /* really, it would be more convienient for simbasin to do this
     and spit out the max's, so that it only had to be done once.
     that's true for the rest, too... though they lack that
     extra subtraction.   */
  ds = &datascale[ACCUM][0];
  ds->end[1][slice] = 0;
  for (i = 1; i < timeloop[slice]; ++i)
    for (j = 1; j < nnodes; ++j)
      if ((x = timeline[slice][0][i][j] - timeline[slice][0][i - 1][j])
	  > ds->end[1][slice])
	ds->end[1][slice] = x;
  /* meters --> mm / yr */
  ds->end[1][slice] *= .001 / (timedelt * tlstep);  

  ds = &datascale[MAXACCUM][0];
  ds->end[1][slice] = 0;
  for (i = 1; i < timeloop[slice]; ++i)
    for (j = 1; j < nnodes; ++j)
      if ((timeline[slice][3][i-1][j-1] *= factor) > ds->end[1][slice])
	ds->end[1][slice] = timeline[slice][3][i - 1][j - 1];

  ds = &datascale[DIFFUSION][0];
  ds->end[0][slice] = 0;  /* stays that way, too */
  ds->end[1][slice] = 0;
  for (i = 0; i < timeloop[slice]; ++i)
    for (j = 0; j < nnodes; ++j)
      if ((timeline[slice][4][i][j]) > ds->end[1][slice])
	ds->end[1][slice] = timeline[slice][4][i][j];

  for (n = 0; n < nisotopes; ++n)
    for (i = 0; i < timeloop[slice]; ++i) {
      if (isotope[n][i] > datascale[ISOTOPE][n].end[1][slice])
	datascale[ISOTOPE][n].end[1][slice] = isotope[n][i];
      if (isotope[n][i] < datascale[ISOTOPE][n].end[0][slice])
	datascale[ISOTOPE][n].end[0][slice] = isotope[n][i];
    }

  for (i = 0; i < NODATA; ++i)
    for (n = 0; n < 100; ++n) { /* overkill to do ALL, but what the hell */
      ds = &datascale[i][n];
      switch (i) {
      case FACIES:
	for (j = 0; j < 2; ++j)
	  if (!ds->end[j][slice])
	    ds->end[j][slice] = VERYSMALL;
	break;
      case POROSITY: case THERM: case THGRAD: case TTI: 
      case ACCUM: case MAXACCUM: case DIFFUSION: case ISOTOPE:
	if (!(ds->end[1][slice] - ds->end[0][slice]))
	  if (ds->end[0][slice] > 0)
	    ds->end[0][slice] *= .99;
	  else if (ds->end[0][slice] < 0)
	    ds->end[0][slice] *= 1.01;
	  else
	    ds->end[1][slice] = VERYSMALL;
	break;
      }   
    }
}


/* if the new color data stretches the scales assigned to it,
   reset the parameters in the parameters window; draw the picture.  */

void readslice_checkscales()
{
  int i, j, k, major, minor;
  Datascale *ds;
  Boolean warn[100][2][2];  /* must use a warn mechanism because most data
			       changes sign */

  /* some things are done twice; that's OK */

  for (i = 0; i < nchoices; ++i)
    for (j = 0; j < 2; ++j) {
      major = datachoice[i].major[j];
      minor = datachoice[i].minor[j];
      ds = &datascale[major][minor];
      if (major != FACIES) {
	ds->end[0][slice] *= -1;
	ds->use[0] *= -1;
      }
      if (major < NODATA) 
	for (k = 0; k < 2; ++k)
	  if ((warn[i][j][k] = (ds->end[k][slice] > ds->use[k])))
	    ds->use[k] = ds->end[k][slice];
      if (major != FACIES) {
	ds->end[0][slice] *= -1;
	ds->use[0] *= -1;
      }
    }
    
  for (i = 0; i < nchoices; ++i)
    for (j = 0; j < 2; ++j) {
      major = datachoice[i].major[j];
      minor = datachoice[i].minor[j];
      for (k = 0; k < 2; ++k) 
	if (warn[i][j][k]) 
	  reset_param(&datascale[major][minor].use[k]);
    }
}  

  

