

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

#include <math.h>
#include "globals.h"
#include "boomheader.h"
#include "Xj/Canvas.h"


extern XFontStruct *fontstruct;

String theta[3][2] = {{"10", "red"}, {"1", "LimeGreen"}, {".1", "blue"}};

void scalebars()  /* draw & label the & scalebars and angles */
{                              
  XPoint pt[5], pos[100], neg[100];
  Dimension dim1, dim2;
  int size, gsize, width, cwidth, height, cheight;
  int i, y, side1, side2, p, n, ht, base;
  double pi = 3.1415926535, x, mult, step, gratio;
  double basescale, length;
  char s[7], units[10], also[10];
  Pixmap pmap, pmap2;

  /* do the horizontal scale bar */
  ClearCanvas(horizbar);
  pmap = CanvasPixmap(horizbar);
  XtVaGetValues(horizbar, XtNheight, &dim1, NULL);
  size = dim1;
  XtVaGetValues(canvas, XtNwidth, &dim1, NULL);
  gsize = dim1;
  gratio = (double)gsize / (xmax - xmin);
  basescale = pow(10.0, (double)(int)log10(xmax - xmin));
  for (i = 0; i < 2; ++i) {
    length = (i ? .1 : 1) * basescale;
    pt[0].x = pt[3].x = pt[4].x = 0;
    pt[1].x = pt[2].x = length * gratio;
    pt[0].y = pt[1].y = pt[4].y = (i ? 0 : size / 2);
    pt[2].y = pt[3].y = (i ? size / 2 : size);
    setcolor(i ? "grey" : "LightGrey");
    XFillPolygon(display, pmap, gc, pt, 5, Complex, CoordModeOrigin);
    setcolor("black");
    sprintf(s, "%d km", (int) (.001 * length));
    XDrawString(display, pmap, gc, pt[2].x + 5, pt[0].y + 20, s, strlen(s));
  }
  side1 = size / 2 + 2;
  side2 = size - 2;
  step = .1 * basescale * gratio;
  setcolor("black");
  for (i = 1, p = 0; i < 10; ++i) {
    p += step;
    XDrawLine(display, pmap, gc, p, side1, p, side2);
  }     
  ExposeCanvas(horizbar);

  /* do the vertical scalebar */
  ClearCanvas(vertbar);
  if (TWTIME == axismode) boom('T');
  pmap = CanvasPixmap(vertbar);
  XtVaGetValues(vertbar, XtNwidth, &dim1, NULL);
  size = dim1;
  XtVaGetValues(canvas, XtNheight, &dim1, NULL);
  gsize = dim1;
  if (DEPTH == axismode) {
    gratio = (double) gsize / (ymax - ymin);
    basescale = pow(10.0, (double)(int)log10(ymax - ymin));
    strcpy(units, "meters");
    strcpy(also, "");
    mult = 1;
  }
  else if (TWTIME == axismode) {
    gratio = (double) gsize / (twtmax - twtmin);
    mult = 1000;
    basescale = pow(10.0, (double)(int)log10(mult * (twtmax - twtmin))) / mult;
    strcpy(units, " ms");
    strcpy(also, "");
  }
  else {  /* WHEELER */
    x = timedelt * tlstep * timeloop[5] / (.01 * wheeleruse);
    gratio = (double) gsize / x;
    basescale = pow(10.0, (double)(int)log10(x));
    mult = 10.0 / basescale;
    sprintf(also, "e%d", (int)log10(1.0 / mult));
    strcpy(units, " yrs");
  }
  
  for (i = 0; i < 2; ++i) {
    length = (i ? .1 : 1) * basescale;
    pt[0].x = pt[3].x = pt[4].x = (i ? 0 : size / 2);
    pt[1].x = pt[2].x = (i ? size / 2 : size);
    pt[0].y = pt[1].y = pt[4].y = 0;
    pt[2].y = pt[3].y = length * gratio;
    setcolor(i ? "grey" : "LightGrey");
    XFillPolygon(display, pmap, gc, pt, 5, Complex, CoordModeOrigin);
    setcolor("black");
    sprintf(s, "%d%s", (int)(mult * length), also);
    XDrawString(display, pmap, gc, 
		pt[3].x + 1, pt[3].y + 20, s, strlen(s));
    XDrawString(display, pmap, gc,
		pt[3].x + 1, pt[3].y + 30, units, strlen(units));
  }
  side1 = size / 2 + 2;
  side2 = size - 2;
  step = .1 * basescale * gratio;
  setcolor("black");
  for (i = 1, p = 0; i < 10; ++i) {
    p += step;
    XDrawLine(display, pmap, gc, side1, p, side2, p);
  }
  ExposeCanvas(vertbar);

  /* do the angles */
  ClearCanvas(anglebox);
  ClearCanvas(anglebar);
  if (!wheeler) {
    pmap = CanvasPixmap(anglebox);
    pmap2 = CanvasPixmap(anglebar);
    XtVaGetValues(anglebox, XtNwidth, &dim1, XtNheight, &dim2, NULL);
    size = dim1;
    gsize = dim2;
    if (DEPTH == axismode)
      for (y = i = 0; i < 3; ++i) {
	setcolor(theta[i][1]);
	XDrawLine(display, pmap, gc, 0, 0, size,
		  (int)(vexag * size * tan(pi * atof(theta[i][0]) / 180)));
	XDrawString(display, pmap2, gc, y, 15, 
		    theta[i][0], strlen(theta[i][0]));
	y += XTextWidth(fontstruct, theta[i][0], strlen(theta[i][0]));
	XDrawString(display, pmap2, gc, y, 9, "o", 1);
	y += 15;
      }
    else  /* TWTIME */
      if (SIGNAL == logdata || reflecmode) {
	XtVaGetValues(canvas, XtNheight, &dim1, XtNwidth, &dim2, NULL);
	cheight = dim1;
	cwidth = dim2;
	XtVaGetValues(anglebox, XtNheight, &dim1, XtNwidth, &dim2, NULL);
	height = dim1;
	width = dim2;
      	boom('\0'); /* just want wavelet initialized */
       	base = width / 2;
	pos[0].x = neg[0].x = base;
	pos[0].y = neg[0].y = 0;
	pos[height + 1].x = neg[height + 1].x = base;
	pos[height + 1].y = neg[height + 1].y = height;
	mult = nu * (twtmax - twtmin) / (double)cheight;
	
	for (i = 0; i < height; ++i) {
	  pos[i + 1].y = neg[i + 1].y = i;
	  pos[i + 1].x = neg[i + 1].x = base;
	  n = SIGPTS * mult * (i - .5 * height);
	  if (n < -SIGPTS || n > SIGPTS)  
	    x = 0;
	  else
	    x = .4 * width * wave[funcmode].val[n + SIGPTS];
	  if (x > 0) 
	    pos[i + 1].x += x;
	  else 
	    neg[i + 1].x += x;
	}
	
	if (sigmode) {
	  setcolor("white");
	  XFillPolygon(display, pmap, gc, pos, height + 2, Complex,
		       CoordModeOrigin);
	  setcolor("black");
	  XDrawLines(display, pmap, gc, pos, height + 2, CoordModeOrigin);
	  XFillPolygon(display, pmap, gc, neg, height + 2, Complex,
		       CoordModeOrigin);
	}
	else {
	  setcolor("blue");
	  XFillPolygon(display, pmap, gc, pos, height + 2, Complex,
		       CoordModeOrigin);
	  setcolor("red");
	  XFillPolygon(display, pmap, gc, neg, height + 2, Complex,
		       CoordModeOrigin);
	}
	/* draw a reflec coeff of strength 10% at correct offset */
	setcolor("grey");
	p = 10 * delta * cwidth / (xmax - xmin);
	ht = height / 2 + cheight * (phi / nu) / (twtmax - twtmin);
	XDrawLine(display, pmap, gc, base, ht, base + p, ht);
	setcolor("black");
	XDrawLine(display, pmap, gc, base, 0, base, height);
      }
  }
  ExposeCanvas(anglebar);
  ExposeCanvas(anglebox);
}



