

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see strata.h                              */
#include "strata.h"
#include "license.h"

/* Patch History, starting from v2.1:
   2.11:  
           XawChain* changed to XtChain* for the benefit of XtR4
	   calloc()s in init() given explicitly casting
	   HelpButton.c got to #include <sys/unistd.h> for SEEK_*
   2.12:
           no changes here
   2.13:
           added -sync option to XSynchronize
	   changed scalebar.c's extra fontstruct decl to extern
	   movie handshaking tightened up
	   void main() --> int
   2.14:
           Xj moved to src/shared
	   isotopes added
	   don't loop on static visuals
	   unified colormap strategy; always display correctly (given focus)
	   bigger sealevel/isotope wheeler curves/axes/lines
	   easier cursor movement in params win (Enter / BtnDown / arrows)
*/


#define indicatefilm "-MakeIntoFilm"


#include "globals.h"
#include "plothelp.h"
#include "fallbacks.h"
#include "film.h"

#include <string.h>
#include <X11/Shell.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/List.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/Scrollbar.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/Viewport.h>

#include "Xj/Canvas.c"
#include "Xj/Messenger.c"
#include "HelpButton.c"

extern void crucialspecs(void);
extern void friendly(int);
extern void goodbye(Widget, XtPointer, XtPointer);
extern Colormap initialize(void);
extern void makedump(Widget, XtPointer, XtPointer);
extern void modelgood(void);
extern void movieloop(Widget slicelist, XtPointer, XtPointer);
extern void parameters(void);
extern void paramstuff(Widget, int, char **);
extern void readslice(Widget, XtPointer, XtPointer);
extern void readspecs(void);
extern void setaxis(Widget, XtPointer, XtPointer);
extern void setboolean(Widget, XtPointer, XtPointer);
extern void setcontour(Widget, XEvent *, String *, Cardinal *);
extern void setdata(Widget, XtPointer, XtPointer);
extern void setfrozen(Widget, XtPointer, XtPointer);
extern void setparams(Widget, XtPointer, XtPointer);
extern void setscaled(Widget, XtPointer, XtPointer);
extern void slices(Widget);
extern void update(Widget, XtPointer, XtPointer);

extern int slicelistdigits, nframes;
extern int sock;
extern Boolean logdisp, sigmode, reflecmode, greyvis;
extern int funcmode;

XFontStruct *fontstruct;

Display *display;
XtAppContext app;
GC gc = NULL;
Widget canvas, alsograb[3], toplevel, viewport;
Widget horizbar, vertbar, anglebox, anglebar, colorbar, percentbar;
Widget logampbtn, logdatabtn, conval, condatabtn, condial, filldatabtn;
Widget scalefill, scalelog;
String library;

void logamp(Widget w, XEvent *e, String *s, Cardinal *c)
{
  char label[100];
  int major = logdata / 100;
  int minor = logdata % 100;
  
  if (!strcmp(*s, "plus"))
    ++datascale[major][minor].amp;
  else if (!strcmp(*s, "minus")) {
    if (--datascale[major][minor].amp < 1) 
      datascale[major][minor].amp = 1;
  }
  else
    datascale[major][minor].amp = 10;
  
  sprintf(label, "%d", datascale[major][minor].amp);
  XtVaSetValues(w, XtNlabel, (XtArgVal)label, NULL);
}

XtActionsRec action[] = {
  {"logamp", logamp},
  {"setcontour", setcontour}};


#define APPNAME "plotbasin"
#define APPNAMEUP "Plotbasin"

/* create the widgets, add their callbacks etc */
void main(int argc, char **argv)
{
  Widget box, console, slicelist, dumpmenu;
  Widget axisbtn, axismenu;
  Widget fillbox, filldatamenu;
  Widget logbox, logdatamenu;
  Widget conbox, condatamenu;
  Widget param_button, dump_button, update_button, exit_button, help_button;
  Widget freezetog;
  Widget boolbtn, boolmenu;
  Widget progressbox, scrollprogress, labelprogress;
  int i, n;
  Dimension width, height;
  XSizeHints sizehints;
  char option, temp[1000];
  unsigned long gcmask;
  XGCValues gcvalues;

  if (argc < 2) friendly(1); 
  if (!strcasecmp(argv[1], "help")) friendly(2);
  if (!strcmp(argv[1], "-p")) friendly(1);

  /* cut .dat if present */
  n = strlen(argv[1]);
  if (n > 4 && !strcmp(argv[1] + n - 4, ".dat")) {
    strcpy(temp, argv[1]);
    temp[n - 4] = '\0';
    library = XtNewString(temp);
  }
  else
    library = argv[1];

  movie = False;
  option = '\0';
  for (i = 2; i < argc; ++i) {
    if (!strcmp(argv[i], indicatefilm)) {
      movie = True;
      option = '\0';
    }
    else if (!strcmp(argv[i], "-p")) 
      option = 'p';
    else if ('p' == option) 
      continue;  /* do nothing; load later, in paramstuff() */
  }

  if (movie) {
    sscanf(argv[3], "%d", &sock);
    read(sock, temp, 2);
    temp[2] = '\0';
    sscanf(temp, "%d", &slicelistdigits);
    i = 0;
    while ((read(sock, temp + i, 1) == 1) && temp[i++] != '.');
    temp[--i] = '\0';
    sscanf(temp, "%d", &nframes);
    crucialspecs();
  }
  else {
    friendly(0);
    readspecs();
  }

  parameters();

  /* top-level shell of the application */
  sprintf(temp, "Strata %s %s", APPNAMEUP, library);
  toplevel = XtVaAppInitialize(&app, APPNAME, (XrmOptionDescList) NULL, 0,
			       &argc, argv, fallbacks, 
			       XtNinput, True, XtNtitle, temp, 
			       NULL);
  display = XtDisplay(toplevel);
  XtAppAddActions(app, action, XtNumber(action));
  initialize();  /* do immediately to get colormap straight */
  
  /* box to hold boxes of buttons and toggles */
  box = XtVaCreateManagedWidget("box", formWidgetClass, toplevel, 
				XtNdefaultDistance, 0, NULL);

  /* bar for numerical labels of angles in anglebox */
  anglebar = XtVaCreateManagedWidget("anglebar", canvasWidgetClass, box,
				     XtNheight, 20, XtNwidth, 80,
				     XtNtop, XtChainTop,
				     XtNbottom, XtChainTop,
				     XtNright, XtChainLeft,
				     XtNleft, XtChainLeft,
				     XtNfromVert, NULL,
				     XtNfromHoriz, NULL,
				     NULL);

  /* box to display angles */
  anglebox = XtVaCreateManagedWidget("anglebox", canvasWidgetClass, box,
                                     XtNheight, 82, XtNwidth, 80,
                                     XtNtop, XtChainTop,
                                     XtNbottom, XtChainTop,
                                     XtNright, XtChainLeft,
                                     XtNleft, XtChainLeft,
                                     XtNfromVert, anglebar,
                                     XtNfromHoriz, NULL,
                                     NULL);

  /* vertical scale bar */
  vertbar = XtVaCreateManagedWidget("vertbar", canvasWidgetClass, box,
				    XtNheight, FILM_HEIGHT, XtNwidth, 80,
				    XtNtop, XtChainTop, 
				    XtNbottom, XtChainBottom,
				    XtNright, XtChainLeft, 
				    XtNleft, XtChainLeft,
				    XtNfromVert, anglebox,
				    XtNfromHoriz, NULL,
				    NULL);

  /* numerical labels for colorbar bar */
  percentbar = XtVaCreateManagedWidget("percentbar", canvasWidgetClass, box,
                                       XtNheight, 20, XtNwidth, FILM_WIDTH,
                                       XtNtop, XtChainTop,
                                       XtNbottom, XtChainTop,
                                       XtNright, XtChainRight,
                                       XtNleft, XtChainLeft,
                                       XtNfromVert, NULL,
                                       XtNfromHoriz, anglebar,
                                       NULL);

  /* colors of color scale bar */
  colorbar = XtVaCreateManagedWidget("colorbar", canvasWidgetClass, box,
                                     XtNheight, 20, XtNwidth, FILM_WIDTH,
                                     XtNbackground, black,
                                     XtNtop, XtChainTop,
                                     XtNbottom, XtChainTop,
                                     XtNright, XtChainRight,
                                     XtNleft, XtChainLeft,
                                     XtNfromVert, percentbar,
                                     XtNfromHoriz, anglebox,
                                     NULL);

  /* horizontal scale bar */
  horizbar = XtVaCreateManagedWidget("horizbar", canvasWidgetClass, box,
				     XtNheight, 60, XtNwidth, FILM_WIDTH,
				     XtNtop, XtChainTop, 
				     XtNbottom, XtChainTop,
				     XtNright, XtChainRight,
				     XtNleft, XtChainLeft,
				     XtNfromVert, colorbar,
				     XtNfromHoriz, anglebox,
				     NULL);
  
  /* viewport to hold picture */
  viewport = XtVaCreateManagedWidget("viewport", viewportWidgetClass, box,
				     XtNallowHoriz, True, XtNallowVert, True,
				     XtNuseBottom, True, XtNuseRight, True,
				     XtNheight, FILM_HEIGHT, 
				     XtNwidth, FILM_WIDTH,
				     XtNbottom, XtChainBottom,
				     XtNtop, XtChainTop,
				     XtNleft, XtChainLeft, 
				     XtNright, XtChainRight,
				     XtNfromVert, horizbar,
				     XtNfromHoriz, vertbar,
				     NULL);
  
  /* main picture */
  canvas = XtVaCreateManagedWidget("canvas", canvasWidgetClass, viewport, 
				   XtNwidth, (movie ? FILM_WIDTH : 1000),
				   XtNheight, (movie ? FILM_HEIGHT : 1000),
				   NULL);

  /* progress indicator for time-consuming activities */
  progressbox = XtVaCreateManagedWidget("progressbox", formWidgetClass, box,
					XtNdefaultDistance, 0,
					XtNborderWidth, 0,
					XtNtop, XtChainBottom,
					XtNbottom, XtChainBottom,
					XtNleft, XtChainLeft,
					XtNright, XtChainLeft,
					XtNfromVert, vertbar,
					XtNfromHoriz, NULL,
					NULL);
  labelprogress = XtVaCreateManagedWidget("label", labelWidgetClass, 
					  progressbox, XtNborderWidth, 0,
					  XtNlabel, "abcdefghij", 
					  XtNforeground, white, NULL);
  scrollprogress = XtVaCreateManagedWidget("scroll", scrollbarWidgetClass,
					   progressbox, 
					   XtNsensitive, False, 
					   XtNminimumThumb, 0,
					   XtNorientation, XtorientHorizontal,
					   XtNwidth, 80,
					   XtNborderWidth, 0,
					   XtNfromVert, labelprogress,
					   NULL);
 
  /* list of basin slice ages available */
  slicelist = XtVaCreateManagedWidget("slicelist", listWidgetClass, box,
				      XtNdefaultColumns, 6, 
				      XtNforceColumns, True,
				      XtNcolumnSpacing, 26,
				      XtNborderWidth, 0,
				      XtNbottom, XtChainBottom,
				      XtNtop, XtChainBottom,
				      XtNright, XtChainLeft,
				      XtNleft, XtChainLeft,
				      XtNfromVert, viewport,
				      XtNfromHoriz, progressbox,
				      NULL);
  XtAddCallback(slicelist, XtNcallback, (movie ? movieloop : readslice), NULL);
  slices(slicelist);

  console = XtVaCreateManagedWidget("console", formWidgetClass, box,
				    XtNborderWidth, 0,
				    XtNbottom, XtChainBottom,
				    XtNtop, XtChainBottom,
				    XtNright, XtChainLeft,
				    XtNleft, XtChainLeft,
				    XtNfromVert, slicelist,
				    XtNfromHoriz, progressbox,
				    NULL);
  
  fillbox = XtVaCreateManagedWidget("fillbox", formWidgetClass, console,
				    XtNfromVert, NULL, XtNfromHoriz, NULL,
				    XtNborderWidth, 0, XtNdefaultDistance, 0,
				    NULL);

  filldatabtn = XtVaCreateManagedWidget
    ("filldatabtn", menuButtonWidgetClass, fillbox,
     XtNfromVert, NULL, XtNfromHoriz, NULL, 
     XtNlabel, (XtArgVal)"fill:  composition (W)",
     NULL);
  filldatamenu = XtVaCreatePopupShell("menu", simpleMenuWidgetClass, 
				      filldatabtn, XtNinput, True, NULL);
  for (i = 0; i < nchoices; ++i) {
    datachoice[i].obj[0] = 
      XtVaCreateManagedWidget(datachoice[i].label[0], smeBSBObjectClass, 
			      filldatamenu,
			      XtNlabel, (XtArgVal)datachoice[i].label[0],
			      NULL);
    n = datachoice[i].major[0];
    if ((!compaction && POROSITY == n) ||
	(!condsand && (CONDUCT == n || THERM == n || THGRAD == n || TTI == n)))
      XtSetSensitive(datachoice[i].obj[0], FALSE);
    XtAddCallback(datachoice[i].obj[0], XtNcallback, 
		  setdata, (XtPointer)(0 + i));
  }

  scalefill = XtVaCreateManagedWidget("scale", toggleWidgetClass, 
				      fillbox,
				      XtNfromVert, NULL,
				      XtNfromHoriz, filldatabtn,
				      XtNsensitive, False,
				      NULL);
  XtAddCallback(scalefill, XtNcallback, setscaled, (XtPointer)0); /* 0==fill */
  
  logbox = XtVaCreateManagedWidget("logbox", formWidgetClass, console,
				   XtNfromVert, NULL, XtNfromHoriz, fillbox,
				   XtNborderWidth, 0, XtNdefaultDistance, 0,
				   XtNleft, XtChainLeft,
				   XtNright, XtChainLeft,
				   NULL);

  logdatabtn = XtVaCreateManagedWidget
    ("logdatabtn", menuButtonWidgetClass, logbox,
     XtNfromVert, NULL, XtNfromHoriz, fillbox, 
     XtNlabel, (XtArgVal)"log:  composition (W)", 
     NULL);
  logdatamenu = XtVaCreatePopupShell("menu", simpleMenuWidgetClass,
				     logdatabtn, XtNinput, True, NULL);
  for (i = 0; i < nchoices; ++i) {
    datachoice[i].obj[1] =
      XtVaCreateManagedWidget(datachoice[i].label[0], smeBSBObjectClass, 
			      logdatamenu,
			      XtNlabel,(XtArgVal)datachoice[i].label[0], 
			      NULL);   
    n = datachoice[i].major[0];
    if ((!compaction && POROSITY == n) ||
	(!condsand && (CONDUCT == n || THERM == n || THGRAD == n || TTI == n))
	|| (!nisotopes && ISOTOPE == n))
      XtSetSensitive(datachoice[i].obj[1], False);
    XtAddCallback(datachoice[i].obj[1], XtNcallback, 
		  setdata, (XtPointer)(10000 + i));
  }

  scalelog = XtVaCreateManagedWidget("scale", toggleWidgetClass, logbox,
				     XtNfromVert, NULL,
				     XtNfromHoriz, logdatabtn,
				     XtNsensitive, False,
				     NULL);
  XtAddCallback(scalelog, XtNcallback, setscaled, (XtPointer)1); /* 1 == log */


  logampbtn = XtVaCreateManagedWidget("logampbtn", commandWidgetClass, logbox,
				      XtNfromVert, NULL, 
				      XtNfromHoriz, scalelog,
				      XtNsensitive, False,
				      XtNlabel, "10",
				      NULL);
  XtOverrideTranslations(logampbtn, XtParseTranslationTable
			 ("<Btn1Down> : logamp(plus)"));
  XtOverrideTranslations(logampbtn, XtParseTranslationTable
			 ("<Btn3Down> : logamp(minus)"));
  XtOverrideTranslations(logampbtn, XtParseTranslationTable
			 ("<Btn2Down> : logamp(default)"));

  conbox = XtVaCreateManagedWidget("conbox", formWidgetClass, console,
				   XtNfromVert, NULL, XtNfromHoriz, logbox,
				   XtNborderWidth, 0, XtNdefaultDistance, 0,
				   XtNleft, XtChainLeft, 
				   XtNright, XtChainRight,
				   NULL);

  condatabtn = XtVaCreateManagedWidget
    ("condatabtn", menuButtonWidgetClass, conbox,
     XtNfromVert, NULL, XtNfromHoriz, NULL,
     XtNlabel, (XtArgVal)"contour:  composition (W)",
     NULL);
  condatamenu = XtVaCreatePopupShell("menu", simpleMenuWidgetClass,
				     condatabtn, XtNinput, True, NULL);

  condial = XtVaCreateManagedWidget("condial", dialogWidgetClass, conbox,
				    XtNfromVert, NULL, 
				    XtNfromHoriz, condatabtn,
				    XtNlabel, "", XtNvalue, "",
				    XtNdefaultDistance, 0,
				    XtNsensitive, False,
				    NULL);
  XtDestroyWidget(XtNameToWidget(condial, "label"));
  conval = XtNameToWidget(condial, "value");
  XtVaSetValues(conval, XtNfromVert, NULL, XtNfromHoriz, NULL, 
		XtNwidth, 80, XtNdisplayCaret, False, XtNborderWidth, 0,
		NULL);
  XtOverrideTranslations(conval, XtParseTranslationTable
			 ("<Key>Return : setcontour()"));

  for (i = 0; i < nchoices; ++i) 
    if (datachoice[i].major[0] != SIGNAL &&
	datachoice[i].major[0] != ISOTOPE) {
      datachoice[i].obj[2] =
	XtVaCreateManagedWidget(datachoice[i].label[0], smeBSBObjectClass,
				condatamenu,
				XtNlabel, (XtArgVal)datachoice[i].label[0],
				NULL);
      n = datachoice[i].major[0];
      if ((!compaction && POROSITY == n) ||
	  (!condsand && (CONDUCT == n || THERM ==n || THGRAD ==n || TTI ==n))) 
	XtSetSensitive(datachoice[i].obj[2], False);
      XtAddCallback(datachoice[i].obj[2], XtNcallback, setdata,
		    (XtPointer)(20000 + i));
    }
  
  axisbtn = XtVaCreateManagedWidget("axis", menuButtonWidgetClass, console,
				    XtNfromVert, fillbox, 
				    XtNfromHoriz, NULL,
				    XtNlabel, (XtArgVal)"axis: depth    ", 
				    NULL);
  axismenu = XtVaCreatePopupShell("menu", simpleMenuWidgetClass, axisbtn,
				  XtNinput, True, NULL);
  XtAddCallback(XtVaCreateManagedWidget("depth", smeBSBObjectClass, axismenu,
					NULL),
		XtNcallback, setaxis, (XtPointer)DEPTH);
  XtAddCallback(XtVaCreateManagedWidget("twtime", smeBSBObjectClass, axismenu, 
					NULL),
		XtNcallback, setaxis, (XtPointer)TWTIME); 
  XtVaCreateManagedWidget("line", smeLineObjectClass, axismenu, NULL);
  XtAddCallback(XtVaCreateManagedWidget("wheeler", smeBSBObjectClass, axismenu,
					NULL),
		XtNcallback, setaxis, (XtPointer)WHEELER);

  boolbtn = XtVaCreateManagedWidget("misc", menuButtonWidgetClass, console,
				    XtNfromVert, fillbox, 
				    XtNfromHoriz, axisbtn,
				    NULL);
  boolmenu = XtVaCreatePopupShell("menu", simpleMenuWidgetClass, boolbtn,
				  XtNinput, True, NULL);
  XtAddCallback
    (XtVaCreateManagedWidget("timelines:  shown", smeBSBObjectClass, boolmenu,
			     NULL),
     XtNcallback, setboolean, (XtPointer)&linesvis);
  XtAddCallback
    (XtVaCreateManagedWidget("shoreline:  off", smeBSBObjectClass, boolmenu,
			     NULL),
     XtNcallback, setboolean, (XtPointer)&shorevis);
  XtVaCreateManagedWidget("line", smeLineObjectClass, boolmenu, NULL);
  XtAddCallback
    (XtVaCreateManagedWidget("ricker displacement", smeBSBObjectClass,boolmenu,
			     NULL),
     XtNcallback, setboolean, (XtPointer)&funcmode);
  XtAddCallback
    (XtVaCreateManagedWidget("logs:  curve", smeBSBObjectClass, boolmenu,
			     NULL),
     XtNcallback, setboolean, (XtPointer)&logdisp);
  XtAddCallback
    (XtVaCreateManagedWidget("sig:  red / blue", smeBSBObjectClass, boolmenu,
			     NULL),
     XtNcallback, setboolean, (XtPointer)&sigmode);
  XtAddCallback
    (XtVaCreateManagedWidget("reflec coeffs:  off", smeBSBObjectClass,boolmenu,
			     NULL),
     XtNcallback, setboolean, (XtPointer)&reflecmode);
  XtAddCallback
    (XtVaCreateManagedWidget("seabottom:  ignore", smeBSBObjectClass, boolmenu,
			     NULL),
     XtNcallback, setboolean, (XtPointer)&watermode);
  XtVaCreateManagedWidget("linetwo", smeLineObjectClass, boolmenu, NULL);
  XtAddCallback
    (XtVaCreateManagedWidget(greyvis ? "paint:  grey" : "paint:  color",
			     smeBSBObjectClass, boolmenu,
			     XtNsensitive, usecolor = !greyvis,
			     NULL),
     XtNcallback, setboolean, (XtPointer)&usecolor);

  param_button = 
    XtVaCreateManagedWidget("parameters", commandWidgetClass, console, 
			    XtNfromVert, fillbox, 
			    XtNfromHoriz, boolbtn, NULL);
  XtAddCallback(param_button, XtNcallback, setparams, NULL);
  paramstuff(param_button, argc, argv);

  dump_button = 
    XtVaCreateManagedWidget("dump", menuButtonWidgetClass, console,
			    XtNfromVert, fillbox, 
			    XtNfromHoriz, param_button,
			    NULL);
  dumpmenu = XtVaCreatePopupShell("menu", simpleMenuWidgetClass, dump_button,
				  XtNinput, True, 
				  NULL);
  XtAddCallback
    (XtVaCreateManagedWidget("entire", smeBSBObjectClass, dumpmenu, NULL),
     XtNcallback, makedump, (XtPointer) 0);
  XtAddCallback
    (XtVaCreateManagedWidget("view", smeBSBObjectClass, dumpmenu, NULL),
     XtNcallback, makedump, (XtPointer) 1);
  XtAddCallback
    (XtVaCreateManagedWidget("canvas", smeBSBObjectClass, dumpmenu, NULL),
     XtNcallback, makedump, (XtPointer) 2);

  alsograb[0] = help_button = 
    XjCreateManagedHelpButton("help", console, plothelp, NULL, "general",
			      (ArgList) NULL, 0);
  XtVaSetValues(help_button, XtNfromVert, fillbox, 
		XtNfromHoriz, dump_button,
		NULL);

  freezetog = XtVaCreateManagedWidget("freeze", toggleWidgetClass, console,
				      XtNfromVert, fillbox, 
				      XtNfromHoriz, help_button, 
				      XtNstate, frozen, NULL);
  XtAddCallback(freezetog, XtNcallback, setfrozen, NULL);
  
  update_button = 
    XtVaCreateManagedWidget("update", commandWidgetClass, console,
			    XtNfromVert, fillbox, XtNfromHoriz, freezetog,
			    XtNsensitive, frozen, NULL);
  XtAddCallback(update_button, XtNcallback, update, NULL);

  alsograb[1] = exit_button = 
    XtVaCreateManagedWidget("exit", commandWidgetClass, console,
			    XtNfromVert, fillbox, 
			    XtNfromHoriz, update_button,
			    NULL);
  XtAddCallback(exit_button, XtNcallback, goodbye, NULL);
  
  alsograb[2] = (Widget) NULL;

  XtRealizeWidget(toplevel);

  if (1 == modelnum) modelgood();

  XtVaGetValues(slicelist, XtNfont, &fontstruct, NULL);

  XtVaSetValues(filldatabtn, XtNlabel, "fill:  empty", NULL);
  XtVaSetValues(logdatabtn, XtNlabel, "log:  empty", NULL);
  XtVaSetValues(condatabtn, XtNlabel, "contour:  empty", NULL);

  gcmask = GCLineWidth | GCBackground | GCFont;
  gcvalues.line_width = 0;  /* default anyway */
  gcvalues.background = white;
  gcvalues.font = fontstruct->fid;
  gc = XCreateGC(display, XtWindow(toplevel), gcmask, &gcvalues);

  if (movie) {
    XtVaGetValues(toplevel, XtNwidth, &width, XtNheight, &height, NULL);
    sizehints.min_width = sizehints.max_width = width;
    sizehints.min_height = sizehints.max_height = height;
    sizehints.flags = PMinSize | PMaxSize;
    XSetNormalHints(display, XtWindow(toplevel), &sizehints);
  }

  XtAppMainLoop(app);
}
