
/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris  */
/* see file strata.h */

#include "strata.h"

#include <stdio.h>
#include <math.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Dialog.h>

#include "Xj/Canvas.h"

#include "paramheader.h"
#include "globals.h"

extern Pixel white;
extern Widget carbshell, carbcanvas, carbdial, carbdraw, carbval;

Boolean iscarbgraphup = False;
Boolean redrawon = False;
GC gc = NULL;

static char dot[] = {1, 7};
static char dash[] = {5, 3};

void carbgraph(void)  /* only ever called when carb group is up */
{
  int i;
  Group g = &group[curgroupnum];  /* CarbGroup */
  double maxdepth = 0;
  double epdepth = 0, ocdepth = 0, ocdecay = -1;  
  double x, wide, high, mult;
  char s[100];
  Dimension width, height;
  XPoint *pt;
  XGCValues values;
  Pixmap pmap = CanvasPixmap(carbcanvas);

  if (!iscarbgraphup)
    return;

  XtSetSensitive(carbdraw, redrawon = False); 

  getgroupvalues();
  sscanf(idtoval(g, EpeiricDepth), "%lf", &epdepth);
  sscanf(idtoval(g, OceanicDecay), "%lf", &ocdecay);
  sscanf(idtoval(g, OceanicDepth), "%lf", &ocdepth);
     
  if (epdepth > 0)
    maxdepth = epdepth * 10;
  if (ocdecay >= 0 && ocdepth > 0) {
    if ((x = 5 / ocdecay) < 2 * ocdepth)
      x = 2 * ocdepth;
    if (x > maxdepth)
      maxdepth = x;
  }
  if (maxdepth <= 0) {
    ClearCanvas(carbcanvas);
    ExposeCanvas(carbcanvas);
    return;
  }

  if (sscanf(XawDialogGetValueString(carbdial), "%lf", &x) == 1 && x > 0) 
    maxdepth = x;
  else {
    sprintf(s, "%g", maxdepth);
    XtVaSetValues(carbdial, XtNvalue, s, NULL); 
    XawTextSetInsertionPoint(carbval, strlen(s));
  }

  if (!gc) gc = XCreateGC(display, XtWindow(toplevel), 0, 0);

  ClearCanvas(carbcanvas);

  XtVaGetValues(carbcanvas, XtNwidth, &width, XtNheight, &height, NULL);

  wide = (double)width - 40 - 20;
  high = (double)height - 20 - 10;

  pt = (XPoint *) XtCalloc(width, sizeof(XPoint));

  values.line_style = LineOnOffDash;
  XChangeGC(display, gc, GCLineStyle, &values);
  XSetDashes(display, gc, 0, dot, 2);
  XSetForeground(display, gc, white);
  pt[0].x = 40;
  pt[1].x = width;
  for (i = 0; i <= 10; ++i) {
    pt[0].y = pt[1].y = (high * i) / 10.0 + 20;
    XDrawLines(display, pmap, gc, pt, 2, CoordModeOrigin);
    sprintf(s, "%g m", .1 * i * maxdepth);
    XDrawString(display, pmap, gc, 5, pt[0].y + 5, s, strlen(s));
  }
  pt[0].y = 20;
  pt[1].y = height;
  for (i = 0; i <= 10; ++i) {
    pt[0].x = pt[1].x = (wide * i) / 10.0 + 40;
    XDrawLines(display, pmap, gc, pt, 2, CoordModeOrigin);
    sprintf(s, "%3d%%", i * 10);
    XDrawString(display, pmap, gc, pt[0].x - 12, 12, s, strlen(s));
  }  

  for (i = 0; i < high; ++i)
    pt[i].y = i + 20;

  mult = maxdepth / (double)(high - 1);

  XSetDashes(display, gc, 0, dash, 2);
  if (epdepth > 0) {
    for (i = 0; i < high; ++i) {
      x = i * mult / epdepth;
      pt[i].x = 40 + wide * x * exp(1 - x);
    }
    XDrawLines(display, pmap, gc, pt, i, CoordModeOrigin);
  }
  values.line_style = LineSolid;
  XChangeGC(display, gc, GCLineStyle, &values);
  if (ocdepth > 0 && ocdecay >= 0) {
    for (i = 0; i < high; ++i) 
      if ((x = i * mult) <= ocdepth)
	pt[i].x = 40 + wide;
      else
	pt[i].x = 40 + wide * exp(- ocdecay * (x - ocdepth));
    XDrawLines(display, pmap, gc, pt, i, CoordModeOrigin);
  }

  ExposeCanvas(carbcanvas);

  XFree((void *)pt);
}

void carbview(Widget w, XtPointer clientData, XtPointer callData)
{  
  XEvent event;

  XtPopup(carbshell, XtGrabNone);
  XRaiseWindow(XtDisplay(carbshell), XtWindow(carbshell));
  iscarbgraphup = True;
  while (!CanvasPixmap(carbcanvas)) {
    XtAppNextEvent(app, &event);
    XtDispatchEvent(&event);
  }
  carbgraph(); 
}

void carbdown(Widget w, XtPointer clientData, XtPointer callData)
{
  XtPopdown(carbshell);
}

void carbaction(Widget w, XEvent *e, String *s, Cardinal *c)
{ carbgraph(); }

void carbcallback(Widget w, XtPointer clientData, XtPointer callData)
{ carbgraph(); }
