
/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris  */
/* see file strata.h */
#include "strata.h"

#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>

#include "paramheader.h"
#include "help.h"
#include "globals.h"


static String **text;
static Widget *menu;
static Widget **menuobj;
static int *nmenuobj;
static Dimension menuobjheight;
static int idtonum[1000];

String *menuname;

extern Widget helptext, helpview, helpsub;
extern Window rootwin;

extern void sethelptext(Widget, XtPointer, XtPointer);

void createhelp(Widget helpall, Widget helpsub)
{
  int i, j, k, nhelp, n[100], m;
  Boolean flag;
  String **list;
  char s[10000];

  for (i = 0; i <= ngroups; ++i) 
    n[i] = 0;

  nhelp = XtNumber(help);
  for (i = 0; i < nhelp; ++i) {
    for (flag = False, j = 0; !flag && j <= ngroups; ++j)
      if ((flag = (help[i].id == groupid[j])))
	n[j]++;
    if (--j < ngroups && help[i].desc == group[j].name)
      n[ngroups]++;
  }

  list = (String **) XtCalloc(ngroups + 1, sizeof(String *));
  text = (String **) XtCalloc(ngroups + 1, sizeof(String *));
  for (i = 0; i <= ngroups; ++i) {
    list[i] = (String *) XtCalloc(n[i], sizeof(String));
    text[i] = (String *) XtCalloc(n[i], sizeof(String));
  }

  for (i = 0; i <= ngroups; ++i) 
    n[i] = 0;

  for (i = 0; i < nhelp; ++i) {
    help[i].ref[0] = help[i].ref0, help[i].ref[1] = help[i].ref1;
    help[i].ref[2] = help[i].ref2, help[i].ref[3] = help[i].ref3;
    help[i].ref[4] = help[i].ref4, help[i].ref[5] = help[i].ref5;
    help[i].ref[6] = help[i].ref6, help[i].ref[7] = help[i].ref7;
    help[i].ref[8] = help[i].ref8, help[i].ref[9] = help[i].ref9;
    m = 40 - (int) strlen(help[i].desc) / 2;  
    strcpy(s, "");
    for (j = 0; j < m; ++j) strcat(s, " ");
    strcat(s, help[i].desc);
    strcat(s, "\n\n\n");
    strcat(s, help[i].entry);
    for (flag = False, j = 0; !flag && j <= ngroups; ++j) 
      if ((flag = (help[i].id == groupid[j]))) {
	list[j][n[j]] = help[i].desc;
	text[j][n[j]] = XtNewString(s);
	for (k = 0; k < 10; ++k)
	  if (help[i].ref[k])
	    idtonum[help[i].ref[k]] = n[j];  
	++n[j];
      }
    if (--j < ngroups && help[i].desc == group[j].name) {
      list[ngroups][n[ngroups]] = help[i].desc;
      text[ngroups][n[ngroups]] = XtNewString(s);
      ++n[ngroups];
    }
  }

  menu = (Widget *) XtCalloc(ngroups + 1, sizeof(Widget));
  menuname = (String *) XtCalloc(ngroups + 1, sizeof(String));
  menuobj = (Widget **) XtCalloc(ngroups + 1, sizeof(Widget *));
  nmenuobj = (int *) XtCalloc(ngroups + 1, sizeof(int));

  for (i = 0; i <= ngroups; ++i) {
    if (i < ngroups) sprintf(s, "menu%d", i);
    else strcpy(s, "menu");
    menuname[i] = XtNewString(s);
    menu[i] = XtVaCreatePopupShell(menuname[i], simpleMenuWidgetClass,
				   i < ngroups ? helpsub : helpall,
				   XtNinput, True, 
				   NULL);
    menuobj[i] = (Widget *) XtCalloc(n[i], sizeof(Widget));
    nmenuobj[i] = n[i];
    for (j = 0; j < n[i]; ++j) {
      menuobj[i][j] = XtVaCreateManagedWidget("menu object", 
					      smeBSBObjectClass, menu[i],
					      XtNlabel, list[i][j], 
					      NULL);
      XtAddCallback(menuobj[i][j], XtNcallback, sethelptext, 
		    (XtPointer)(i < ngroups ? j : -(j+1)));
    }
  }
  XtVaGetValues(menuobj[0][0], XtNheight, &menuobjheight, NULL);

  for (i = 0; i <= ngroups; ++i) 
    XtFree((XtPointer)list[i]);
  XtFree((XtPointer)list);
}


static int whichobj[100];

extern void xj_fake_click(Widget, int);  /* stolen from XjMessenger */
void sethelptext(Widget w, XtPointer clientData, XtPointer callData)
{
  int choice = (int)clientData;
  int n = ((choice < 0) ? ngroups : curgroupnum);
  int m = ((choice < 0) ? -(1 + choice) : choice);

  XtVaSetValues(helptext, XtNstring, text[n][m], NULL);
  whichobj[n] = (m + 1) % nmenuobj[n];

  /* scroll the viewport to the top */
  xj_fake_click(XtNameToWidget(helpview, "vertical"), 2);
}

void varhelp(Widget w, XEvent *e, String *s, Cardinal *c)
{
  int n;

  sscanf(*s, "%d", &n);
  sethelptext
    (NULL, 
     (XtPointer)
     idtonum[ n < 0 ? groupid[curgroupnum] : group[curgroupnum].id[n]],
     NULL);
}

void placemenu(Widget w, XEvent *e, String *s, Cardinal *c)
{
  int n = (w != helpsub ? ngroups : curgroupnum);
  int x, y;
  Window junk;
  
  XTranslateCoordinates(display, XtWindow(w), rootwin, 0, 0, &x, &y, &junk);
  
  XtVaSetValues(menu[n],
		XtNx, x,
		XtNy, y - whichobj[n] * menuobjheight,
		NULL);
}
