
/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris  */
/* see file strata.h */

#include "strata.h"

struct {
  UniqueID id;
  String desc;
  String entry;
  /* nasty, nasty, nasty! */
  UniqueID ref0;
  UniqueID ref1;
  UniqueID ref2;
  UniqueID ref3;
  UniqueID ref4;
  UniqueID ref5;
  UniqueID ref6;
  UniqueID ref7;
  UniqueID ref8;
  UniqueID ref9;
  UniqueID ref[10];
} help[] = {
  {GeneralID, "Strata",
  "Welcome to the Basin Simulation programs. These programs are designed \n\
to produce and show simulated two-dimensional deposition in a sedimentary \n\
basin.  You are currently running the setbasin program, which is to set up \n\
up library (.dat) files for the simulator program, simbasin, to run on.\n\
The program plotbasin is used to view the results. \n\
\n\
Once you are satisfied with your library, use simbasin on it to model the basin\n\
and plotbasin on simbasin's output to view the results graphically.  See the\n\
user manual for more detailed information on these and other matters.\n\
\n\
\n\
Stratigraphic Modeling of Sedimentary Basins\n\
\n\
In an ongoing research program, funded in part by the Chevron Oil \n\
Field Research Company, we have developed a forward model of the \n\
stratigraphic evolution of sedimentary basins. The stratigraphic \n\
model has been developed in C with an X Windows interface on a \n\
Unix system.  The model has been compiled and used successfully on\n\
at least some Suns, IBMs, HPs, Decs, SGIs, RS6000s, RTs, and \n\
NetBSD machines; it may or may not work on other machines.\n\
\n\
The model describes sedimentation as a diffusive process, \n\
with the diffusion parameter controlled by the water depth \n\
during deposition. Carbonates are simulated with a depth-dependent\n\
productivity algorithm. A key component of the model is the ability\n\
to control a wide variety of basin-scale processes.  Among them are:\n\
\n\
1) Flexural compensation on an elastic lithosphere.\n\
2) Lithology-dependent compaction based on empirical compaction laws.\n\
3) A user-defined sea-level function which can be a simple sinusoidal\n\
        function, a eustatic sea-level curve (as defined by Exxon Production\n\
        Research), or a user-specified sea-level history.\n\
4) A user-defined subsidence function which can be a simple constant\n\
        value or can vary spatially and temporally.\n\
5) A user-defined clastic sediment flux history, either constant or \n\
        temporally varying.\n\
6) A user-defined carbonate sediment flux history, either constant or\n\
        spatially varying.\n\
\n\
The model predicts a rich range of stratigraphic response as a function\n\
of the driving variables decribed above.  Use of the model permits an \n\
enhanced understanding of general processes that influence stratigraphic\n\
architecture.  These include the distribution and timing of unconformities\n\
as related to sea-level change, tectonic subsidence, and sediment flux.\n\
\n"},

  {GeneralID, "License", 
"Setbasin and the rest of the Strata suite have been put under the Gnu\n\
General Public License, of which you should have copies both in the Strata\n\
user manual and the source code of each program.  This license includes the\n\
following notice:\n\
\n\
                           NO WARRANTY\n\
\n\
BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR THE\n\
PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN OTHERWISE\n\
STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE\n\
PROGRAM 'AS IS' WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,\n\
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND\n\
FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND\n\
PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE,\n\
YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.\n\
\n\
IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL\n\
ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE\n\
THE PROGRAM AS PERMITTED [elsewhere in the License], BE LIABLE TO YOU FOR\n\
DAMAGED, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES\n\
ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT\n\
LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES BY YOU\n\
OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY PROGRAMS),\n\
EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF\n\
SUCH DAMAGES.\n\
\n\
\n\
The license also specifies exact terms for the modification and/or\n\
redistribution of the code; please refer to it for details on this and other\n\
such matters.\n"},

  {GeneralID, "Setbasin", 
"The purpose of setbasin is to set up defining parameters for a basin \n\
simulation, and create from them a file suitable for simbasin.  Thus its\n\
heart is a list of parameters for which the user gives values (or takes\n\
the default values, or takes values from a file).\n\
\n\
Setbasin considers parameters to fall into groups.  The groups are listed\n\
in the upper left-hand corner.  Parameters are brought up one group at a time,\n\
in the large right-hand space; clicking on a group in the upper left-hand\n\
list causes that group to be brought up.\n\
\n\
At the top right, above the group parameters, are three pieces of information\n\
about the current group.  In the middle is the group name; to the left is\n\
stated whether the group is optional or mandatory; to the right is the current\n\
\"file title\" of that group (these two things are explained in other entries.)\n\
\n\
To vary a parameter, simply type into the box given for it.  You may also\n\
use buttons to do various things with the parameters, as explained under\n\
other help entries.  The buttons in the column on the left apply to all the\n\
parameters at once; the buttons in the row at the bottom right apply only\n\
to the current group."},

  {GeneralID, "Converting libraries",
     "If you have .dat files from version 2.0, you may convert them with\n\
\n\
     setbasin -convert library1 library2 library3...\n\
\n\
Each will be converted to the new format, and the v2.0 library will be saved with\n\
a .orig extension instead of a .dat extension."},

  {GeneralID, "Initial files",
     "By invoking setbasin in the form\n\
\n\
          setbasin library1 library2 library3...\n\
\n\
you can start with parameter values other than the defaults.\n\
Setbasin loads (identically to the operation of the <load> buttons) the\n\
contents of each library specified.  They are loaded in order, so if more\n\
than one gives a value for a certain parameter, it will have the last one.\n"},

  {GeneralID, "Units",
     "Use whatever units you wish as long as you are consistent;\n\
meters / kg / yrs is the recommended system (and the plotter may assume it for\n\
certain displays).  When dimensions are indicated in parentheses, \n\
(L) = length, (T) = time, (M) = mass, (d) = degrees Centigrade, and \n\
( ) = no units (pure number); ^ is used for exponentiation.\n\
To avoid messes of notation, only fundamental dimensions are given, not\n\
terms like 'power' nor specific units; there aren't m / k / yrs standard units\n\
for most of these quantities, anyway.\n\
\n\
The notation (T / F) indicates a Boolean variable, i.e. once which is either\n\
on or off.  Acceptable values for Boolean variables are the following:  \n\
          True        False\n\
          T           F    \n\
          Yes         No   \n\
          Y           N    \n\
          1           0    \n\
          On          Off  \n\
where case is irrelevant.\n"},

  {GeneralID, "File Titles", 
"The setbasin window has a \"file title\" given after the program name, \n\
e.g. \"Strata Setbasin defaults.\"  Each group also gives a file title to the\n\
right of the group name.  The overall file title indicates the file last\n\
associated with parameters in all groups; a group file title indicates the\n\
file last associated with parameters in that group.  If you start up \n\
setbasin without initial files, the file titles are all \"defaults,\" since\n\
the default values are used.  If you start up with initial files, the last\n\
file determines the titles.  When you save to or load from a file, the\n\
titles change to that file's name; if you reset to defaults, that becomes\n\
the title.\n\
\n\
If some of the parameters may have been modified from those in the file\n\
(or defaults), the title will have parentheses around the file name, to\n\
indicate that the parameters may be only loosely based on the file.\n\
\n\
Feel free to ignore file titles entirely if you don't want to think about\n\
them; they're there only to provide extra information for your convenience."},

  {GeneralID, "<warn>", 
"The <warn> buttons tell you if a parameter has a value which setbasin \n\
believes to be unacceptable, or if a parameter that setbasin believes must\n\
have a value is missing entirely.  The overall <warn> button (in the left-hand\n\
column) gives warnings about all parameters; the group <warn> button (in the\n\
bottom row) only gives warnings about parameters in the current group.\n\
\n\
Groups for which parameters must be defined are \"mandatory\" groups, and are\n\
indicated by a \"Mandatory\" note to the left of the group name.  Other groups\n\
are \"optional\" and noted as such.  You may leave a parameter blank if and\n\
only if it is in an optional group.\n\
\n\
Setbasin does not check for problems arising from the interaction of \n\
multiple parameters.  For instance, it will check that all the timeslice\n\
ages are positive, but not that they are in nondecreasing order.  Similarly,\n\
if compaction is turned on, it will still allow blanks in the Compaction group,\n\
because that is an optional group, even though the parameters all are needed\n\
when compaction is turned on.  Setbasin may in the future check for problems\n\
in sets of parameters.\n\
\n\
These warnings are warnings only; simbasin will determine for itself if a\n\
library is unacceptable, and if so abort with appropriate error messages.  A\n\
warning from setbasin means that simbasin will probably abort.\n\
\n\
The warnings generated by loading and saving operations are identical to those\n\
generated by the <warn> buttons."},

  {GeneralID, "<help>", 
"The overall <help> button (in the left-hand column) is for help about the\n\
program in general; the group <help> button (in the bottom row) is for help\n\
about the current group, and so its entries depend upon the selected group."},

  {GeneralID, "<save>", 
"The <save> button (in the left-hand column) saves all parameters in the\n\
file specified by the user, appending a .dat suffix if necessary.  If it\n\
feels any parameters are unacceptable, it will give warnings as specified\n\
under the <warn> button help entry, but will not prevent the save.  The\n\
library files so saved are suitable for use by simbasin.\n\
\n\
To save the data in only one group, rather than all data, use the 'save group'\n\
option under the <group files> button (in the bottow row).  This and 'load group'\n\
allow for mixing and matching of group specifications by experienced users."},

  {GeneralID, "<load>", 
"The <load> button (in the left-hand column) loads data from the specified\n\
library, adding a .dat suffix if necessary, to any and all parameters mentioned\n\
in the library.  The library should be one either created by a save from\n\
setbasin, or converted by setbasin from a v2.0 library file.\n\
\n\
Warnings will be given of any parameters setbasin feels are unacceptable,\n\
as specified under the <warn> button help entry, but they will be loaded regardless.\n\
If loading might cause data to be lost (because changes may have been made\n\
since the last save, load, or reset), setbasin will warn of this and ask for\n\
confirmation to proceed anyway.\n\
\n\
To load data into only one group's parameters, rather to all parameters given\n\
in the library file, use the 'load group' option under the <group files> button\n\
(in the bottom row).  This will ignore any parameters given in the library\n\
which are not in the current group.  Along with 'save group,' this allows\n\
experienced users to mix and match group specifications."},

  {GeneralID, "<group file>", 
"The 'save group' and 'load group' options under this button allow \n\
experienced users to mix and match groups of parameter specifications;\n\
they act as the <save> and <load> buttons, but only the current group\n\
is saved from or loaded to.  See the <save> and <load> button help entries."},

  {GeneralID, "<default>", 
"These buttons will restore the default parameter values.  The overall <defaults>\n\
button (in the left-hand column) operates on all parameters; the group button\n\
(in the bottom row) operates only on the current group.  If resetting might\n\
cause data to be lost (because changes may have been made since the last \n\
save, load, or reset), setbasin will warn of this and ask for confirmation\n\
to proceed anyway."},

  {GeneralID, "<exit>", 
"If an exit might cause data to be lost (because changes may have been made\n\
since the last save, load, or reset), setbasin will warn of this and ask for\n\
confirmation to proceed; otherwise it will simply exit."},

  {GeneralID, "Key Bindings",
"Users highly familiar with setbasin may become annoyed at having to switch\n\
between the mouse and the keyboard.  To ameliorate this, key bindings have \n\
been defined to simulate most mouse actions.\n\
\n\
 Return:                 move to next parameter in the group \n\
 Up arrow:               move to previous parameter in the group \n\
 Down arrow:             move to next parameter in the group \n\
 < (less than):          move to previous group \n\
 > (greater than):       move to next group \n\
 PageUp or Prev:         move to previous group \n\
 PageDown or Next:       move to next group \n\
\n\
 control-x control-w     global warn \n\
 control-x control-s     global save \n\
 control-x control-l     global load \n\
 control-x control-f     also global load \n\
 control-x control-d     global defaults \n\
 control-x control-e     exit \n\
 control-x control-c     also exit \n\
\n\
 control-g control-w     group warn \n\
 control-g control-s     group save \n\
 control-g control-l     group load \n\
 control-g control-f     also group load \n\
 control-g control-d     group defaults \n\
\n\
 control-c, control-a    Confirm or Abort when a \"Messenger\" pops up\n\
\n\
\n\
There are no key bindings to access the help material."},

  {MeasureGroup, MeasureName, 
"The Measures group scales the simulation (duration, basin width) and \n\
describes the resolution of the simulation (spatial and temporal gridding)."},

  {MeasureGroup, "total width covered", 
"This is the total spacial distance covered by the simulation.",
                 TotalWidth},

  {MeasureGroup, "spatial divisions", 
"This is how fine-grained the simulation is in space, i.e. the number of intervals\n\
into which to divide the total width.  The higher the graining, the better\n\
the approximations, of course; but note that runtime is second order, and\n\
file output size first order, in the spatial grain.\n\
\n\
Upon starting, the simulator will print out a Delta X figure, which is the\n\
distance between nodes --- i.e. total width / number of spatial divisions.",
   SpatialDiv},

  {MeasureGroup, "total time covered",
     "This is the total duration of the simulation.\n\
\n\
If the 'use even increments' parameter in the Ages group is turned on (the default),\n\
the timeslice ages will automatically be set to even divisions of the total time.",  TotalTime},

  {MeasureGroup, "temporal divisions",
"This is how fine-grained the simulation is in time, i.e. the number of intervals\n\
into which to divide the total duration.  It must be a multiple of the number\n\
of timelines.  The higher the graining, the better the approximations, of course\n\
(except for the porosity and temperature data, as those are grained by the\n\
timelines, instead; see next help entry); but note that runtime grows linearly\n\
with the temporal grain.\n\
\n\
Upon starting, the simulator will print out two Delta T figures; the smaller\n\
is the duration of each temporal division, i.e. total time / # divisions.\n",
    TimeDiv},

   {MeasureGroup, "# timelines",
   "Each timeline contains the data pertaining to the material laid down at\n\
some time (a constant of the timeline).  The timelines will be evenly\n\
distributed over the temporal divisions (so this might as well be a factor\n\
of the temporal grain).  The density of data available to the plotter, and\n\
the file output size, are both proportional to the number of timelines.\n\
The validity of the model's approximations in general depends on the number\n\
of temporal divisions and not on the number of timelines; however, the data\n\
fields of porosity and temperature (and TTI) are calculated at intervals\n\
fitting the timelines, not at every temporal division, and so the validity\n\
of the approximations of those fields depends on the number of the former\n\
rather than of the latter.\n\
\n\
Upon starting, the simulator will print out two Delta T figures; the larger\n\
is the amount of time between timelines --- i.e. total time / # timelines.\n\
\n\
The #timelines must be a factor of the #temporal divisions.\n",
  NumTimelines},


  

  {AgeGroup, AgeName, 
"The Ages group specifies when in the simulation the state of the basin will\n\
be written as output to a file that can be viewed later with plotbasin."},

  {AgeGroup, "use even increments",
  "If this Boolean variable is set to True, the ages of the time slices\n\
will be set to even divisions of the total time.  In this case,\n\
modification of the 'total time' parameter of the Measures group is \n\
automatically reflected in the slice age parameters.\n\
\n\
If this is turned off, the slice ages may be individually specified.\n\
\n\
Making any change to a slice age will cause this to be turned off\n\
(so as not to override your input).", EvenAges},

  {AgeGroup, "age of # time slice",
     "These are the times that the data will be written to a data file which\n\
later will be viewed in the form of a cross-section. The age is measured\n\
forward in time with 0 years being the start of the simulation.  Exactly\n\
six timeslices of the basin are taken; they should be given in order.\n\
\n\
If 'use even increments' is turned on, these will be set automatically.",
   Age1, Age2, Age3, Age4, Age5, Age6},


  
  {ClasticGroup, ClasticName, 
"Parameters in this group control clastic deposition.\n\
Clastic sedimentation is simulated in two ways:\n\
\n\
1) as a slope-dependent diffusive transport process;\n\
\n\
2) as  constant (pelagic) sedimentation."},

  {ClasticGroup, "non-marine diffusion constant",
"This sets the (slope-dependant) diffusion constant in the non-marine realm\n\
(i.e. above sea level). It is typically of order 1e5 m^2/yr. The higher this\n\
value, the flatter the non-marine profiles will be.", NonmarineDiffus}, 

  {ClasticGroup, "simulate marine sedimentation?",
	  "Boolean.\n\
If false, all diffusion values are set to the nonmarine diffusion value.\n\
If true, diffusion values change across the shoreline as specified under\n\
'decay coeff. for marine diffusion constant.'", MarineSed},

  {ClasticGroup, "marine diffusion constant",
"This sets the (slope-dependant) diffusion constant in the fully marine\n\
setting.  It is typically of order 100 m^2/yr. To make foresets steeper,\n\
decrease this parameter.", MarineDiffus},

  {ClasticGroup, "decay coeff. for marine diffusion constant",
"This is the exponent for exponential decay of the diffusion value as a\n\
function of water depth.\n\
\n\
Above sea level, the diffusion value is set to K_nonmarine.  Below sealevel the\n\
value decays from K_nonmarine to K_marine according to\n\
\n\
K = K_Marine + ((K_Nonmarine - Kmarine) * e^(- diffusion exponent * water depth)\n\
\n\
Thus, IF THE EXPONENT IS ZERO, then all diffusion values will be\n\
set to K_marine.  IF THE EXPONENT IS LARGE, there will be a very abrupt\n\
shift from K_nonmarine to K_marine. IF THE EXPONENT IS NEGATIVE there will\n\
be no exponential decay. All values to the left of the shoreline will be set\n\
 to K_nonmarine.  All to the right will be set to K_marine.\n\
\n\
We find a value of .1 has a smooth transition from Kmarine to Knonmarine. \n\
A value of 1 has a quite abrupt transition, but reduces some of the numerical\n\
noise that is produced by a negative value.\n\
\n\
We have had some problems with this algorithm.\n\
If the simulator hangs, try turning this off. \n", MarineDecay},

  {ClasticGroup, "clastic sediment fluxes",
              "These are applied to the left and right edges of the basin.\n\
Each can be specified as a number, which is then taken to be a constant\n\
flux, or as a file, by name.  The value of zero, or the filename \"none\", \n\
both indicate no flux at that edge.\n\
\n\
The format of the flux data files is pairs of x-y values, one pair to a\n\
line.  Age is in the 1st column, flux is in the 2nd column. The program will\n\
linearly interpolate the flux values between the assigned values, extending\n\
the ends outward.  Sample flux file:\n\
\n\
0       10      \n\
1e6     17.08   \n\
2e6     20.     \n\
3e6     17.08   \n\
4e6     10      \n\
5e6     2.93    \n\
6e6     0.      \n\
\n\
Exact spacing is unimportant.\n\
\n\
We recommend giving clastic flux files a .clas extension.",
    LeftFlux, RightFlux}, 

   {ClasticGroup, "pelagic sedimentation rate",
"This is the pelagic sedimentation rate.  It is applied as a constant rate in every\n\
location in the marine zone.  Negative values may be given to represent erosion.",
    Pelagic},
       


  {CarbGroup, CarbonateName, 
"Parameters in this group control carbonate sedimentation.  This is simulated\n\
as a source term dependant on water depth.  Two different depth-dependant\n\
functions are possible (\"epeiric\" and \"oceanic\") and allow for the\n\
specification of lag-depths and/or lag-times.  The nature of the carbonate\n\
deposition may vary spatially via the carbonate specification file; however,\n\
there is never variation in time in the carbonate deposition function.\n\
\n\
Carbonate deposition will occur if (and only if) a carbonate file is given.\n\
In this case, the parameters for any type of carbonate deposition (epeiric or\n\
oceanic) which occurs in the file must be specified, or simbasin will abort\n\
when it needs to use them.  If the file is \"none\" (the default) or left blank,\n\
there will be no carbonate deposition.\n\
\n\
\n\
Oceanic (lag-depth) deposition is governed by the equation\n\
\n\
            0                      for W < 0\n\
  dsed/dt = C1                     for 0 < W < Wo\n\
            C1 * e^(- C2 * (W - Wo))    for W > Wo\n\
\n\
where C1 is the surface rate, C2 is the scale, Wo is the depth of \n\
maximum sedimentation (lag/cutoff depth), and W is the current water depth.\n\
\n\
\n\
Epeiric deposition is governed by the equation\n\
\n\
  dsed/dt = C1 * (W / Wo) * e^(1 - (W / Wo))) (for W >= 0)\n\
\n\
where C1 is the surface rate and Wo is the depth of maximum sedimentation\n\
(and, of course, also the length scale of the exponential falloff).\n\
\n\
\n\
In either case, the constants are taken from the parameters in the carbonates\n\
group and the carbonates file; the latter also controls which depositions \n\
are operating.\n\
\n\
The <graph> button under the carbonates group may be used to see the shapes\n\
of the curves for a given set of parameters; see the help entry on 'viewing\n\
carbonates.'"},

  {CarbGroup, "viewing carbonates",
     "The <graph> button under the carbonate group brings up a window for\n\
viewing the carbonate sedimentation functions. \n\
\n\
See the group help entry for the functions and their parameters.\n\
The curves are based on the values specified in the specs window as of\n\
the last time you redrew; if they have changed, the <redraw> button will \n\
be available.\n\
\n\
The sedimentations are displayed as percentages of the maxima, so those\n\
do not affect the display (they wouldn't affect the shape of the curves,\n\
anyway).  If any of the other carbonate values are invalid, the display will\n\
lack one or both curves.  The epeiric curve is displayed in a dashed line,\n\
the oceanic in a solid line.\n\
\n\
You may set the total depth to whatever you wish; hit <redraw> or press Return\n\
to view the graph with the new range.  Doing this without a total depth specified\n\
yields a default depth of the maximum of ten times the epeiric depth of \n\
maximum rate, twice the oceanic depth of maximum rate, and five over the\n\
oceanic decay constant; this defaulting always occurs the first time you\n\
bring up the graph."},

  {CarbGroup, "carbonate file", 
     "The file to input carbonate-type data from, or \"none\" for none, in \n\
which case there is no carbonate deposition.  Leaving this parameter blank\n\
also results in no carbonate deposition.\n\
\n\
The data file must be of the form\n\
\n\
0     E   0  0 \n\
10e3  O   5  0 \n\
20e3  O  10  5 \n\
25e3  N   0  0 \n\
\n\
where the first column is position, the second is type of deposition ('E'\n\
for epeiric, 'O' for oceanic, or 'N' for neither), the third is the lagtime,\n\
and the fourth is the lag depth.  The carbonate deposition specified by the \n\
second, third, and fourth columns is used from the position in the first \n\
column until the next position.  \n\
\n\
The first entry must be at position 0.  Exact spacing of the file is\n\
unimportant.\n\
\n\
We recommend giving carbonate files a .carb extension.",
   CarbFile}, 

   {CarbGroup, "epeiric max rate",
"This specifies the max rate (at Wo) of deposition in the epeiric deposition\n\
equation (C1 in the equation as written in the group help entry).",
  EpeiricMax},

   {CarbGroup, "epeiric depth of max rate",
"This specifies the water depth of maximum sedimentation rate in the epeiric\n\
deposition equation (Wo in the equation as written in the group help entry).",
     EpeiricDepth},

   {CarbGroup, "oceanic max rate",
"This specifies the surface rate to use in the oceanic deposition equation\n\
(C1 in the equation as written in the group help entry).", OceanicMax},

   {CarbGroup, "oceanic decay constant",
"This specifies the scale to use in the oceanic deposition equation (C2 in\n\
the equation as written in the group help entry).", OceanicDecay},

   {CarbGroup, "oceanic depth of max rate",
     "This specifies the lag depth to use in the oceanic deposition equation\n\
(Wo in the equation as written in the group help entry).", OceanicDepth},

  {CarbGroup, "lag on zero water depth",
"This controls the behavior of carbonate deposition when the system shoals\n\
to sealevel.  If it is set to True, the lag (depth and/or time) is initiated\n\
when the depositional surface exactly matches (or exceeds) the sealevel.\n\
If it is False, the lag is only initiated when the depositional surface is\n\
above the sealevel. \n\
\n\
The effect of setting this option to True is thus that multiple shoaling cycles\n\
(even with no external forcing) are possible as carbonate deposition rises to\n\
sealevel, cuts off, and restarts when the lag is satisfied.  With this option\n\
False, the system will continually shoal to sealevel, and lag will never be\n\
initiated (unless by other forms of deposition).\n\
\n\
(This variable also controls whether lag is initiated when non-carbonate\n\
effects happen to leave the elevation and sealevel exactly equal, but this is\n\
a very rare case.)", Skim},

  {CarbGroup, "isotope signal file", 
   "If this is not 'none,' it specifies a file defining one or more isotope signals\n\
as functions of time.  Isotope signals are recorded in surface material;\n\
subsequent *vertical* movement of stratigraphy inhomogenously stretches and\n\
compresses the signal, but no other dynamics are taken into effect.  Incoming\n\
signals are always taken to be spatially homogenous.  The final (horizontally\n\
varying) distorted signals can be examined to test isochronicity fitting.\n\
\n\
The format of an isotope signal file is as follows:\n\
\n\
Isotope:  Carbon\n\
Units:  ppt\n\
Fit:  Cubic\n\
-551e6  -2\n\
-550e6   4\n\
[etc]\n\
\n\
Isotope:  Oxygen\n\
[etc]\n\
\n\
More than one isotope signal can be specified in the same file.\n\
Each starts with the word 'Isotope:' followed by space followed by the isotope\n\
name (which can include whitespace).  The next line is 'Units:' followed by space\n\
followed by the units; then comes 'Fit:' followed by space followed by the name\n\
of the fit to be used in interpolating between data points.  Currently available\n\
fits are:\n\
\n\
Linear --- straightforward linear interpolation\n\
Cubic --- natural cublic spline fitting (cubics with continuous first and second\n\
          derivatives, with the boundary condition of zero second derivative)\n\
Waves --- a set of sine waves to be summed, as per wave-type sealevel files\n\
\n\
Following this may be lines giving fit-specific parameters; neither linear,\n\
cubic, nor wave fits require such additional parameter lines.\n\
\n\
Subsequent lines specify the data.  For non-wave fits, this is in the form of\n\
'time versus datum,' times being measured from the present (hence nonpositive).\n\
These times will all be offset by the 'isotope offset time' parameter;\n\
after the offset is applied, they *must* span the duration of the simulation.\n\
\n\
Data for a waves fit is given in the same form as sealevel waves:  rows of\n\
   start_time   end_time   amplitude   period   offset\n\
The sum of all the waves between whose starting and ending times a certain time\n\
falls is the signal at that time.  The per-wave offset time is in addition to the\n\
offset time applied to all isotope signals.  The data need not span the duration\n\
of the simulation; at times when no waves apply the signal is simply zero.\n\
\n\
We recommend giving isotope signal files a .iso suffix.", Isotopes},

  {CarbGroup, "isotope offset time",
   "Like the sealevel offset time, this specifies an amount of time to add to the\n\
model's internal time --- in this case, for isotope calculations.  For instance,\n\
if this is -550e6, then isotope signals are calculated as if the time were\n\
the model's time - 550e6 years.\n\
\n\
Since isotope signal data will often be taken from historical data and therefore\n\
have highly negative times, the isotope offset time will typically be highly\n\
negative to shift the historical data to cover the duration of the simulation.",
   IsoTime},

  {SeaGroup, SeaName, 
"Parameters in this group control eustatic sealevel through time.  A simple\n\
sinusoid may be specified directly, and more complex functions with a\n\
sealevel specification file.  The Haq/Exxon eustatic curve is a special\n\
case of such a file, which should be present as part of Strata."},

  {SeaGroup, "sealevel file",
"This specifies the file from which sealevel data should be taken.  If it\n\
is given as \"none\" or left blank, no file will be consulted, and the\n\
sealevel will be calculated as a pure sine wave.  If it is given as \"exxon.sea,\"\n\
then simbasin will expect to find the Haq (Exxon) Sealevel Curve in that file and\n\
will use it to define the sealevel; any other file name is taken to be a\n\
user-created sealevel file, in the format defined below.  Note that a negative\n\
time offset is required when using the Haq curve (a time offset is optional\n\
with a user-created sealevel file).\n\
\n\
A user-created sealevel file can be specified in two ways, as waves or points.\n\
\n\
A sealevel file of points has the form\n\
\n\
Points\n\
.5e6	100\n\
1e6 	-50\n\
2e6	55\n\
\n\
where the first column is age and the second sealevel.\n\
It will linearly interpolate between the values, extending the ends outward.\n\
The word \"Points\" at the start of the file is literal.\n\
Exact spacing in the file is unimportant.\n\
\n\
\n\
A sealevel file of waves has the form\n\
\n\
Waves\n\
1e6	5e6	75	1e6	0\n\
5e6	20e6	30	1e6	.25e6\n\
10e6	20e6	20	2e6 	0\n\
0	0	0	0	0\n\
\n\
where the first two columns are the range of time in which this sine wave\n\
contributes towards the sealevel, the third is the amplitude, the fourth is\n\
the period, and the fifth is the offset.  Terminate with a row of 0s.\n\
The sealevel at some time is equal to the 'datum for sealevel oscillation'\n\
plus the value at that time of each sine wave whose starting and ending \n\
times bracket the time.  The word \"Waves\" at the start of the file is literal.\n\
You may give up to 1000 waves.  Exact spacing in the file is unimportant.\n\
\n\
We recommend giving sealevel files a .sea extension.\n\
\n\
\n\
Note that (as always) the time offset is applied to the model's time before\n\
it is compared to the times listed in your file; keep that in mind if you\n\
use both non-exxon sealevel file and a time offset (this is rare).\n",
   SeaFile},

   {SeaGroup, "time offset",
"This specifies an amount of time to offset all sealevel calculations by.\n\
For instance, if this is -1000, and your time units are years, then the\n\
sealevel will be calculated as if the time were the model's time - 1000 yrs.\n\
\n\
If you are using the Haq (Exxon) Sealevel Curve, you must specify a highly\n\
negative offset time, because the Haq Curve is specified in terms of years\n\
before the present.  That is, with the Exxon curve, if you specify -1e6 as\n\
the time offset, the model will begin (model time = 0) with the level\n\
specified by Haq for 1e6 years in the past, and go forward from there.\n\
Therefore, with the Exxon curve, make the time offset negative and of\n\
magnitude at least equal to the total time of the simulation.\n\
\n\
With other sealevel files (whether of waves or points), or just the pure\n\
sine wave sealevel, the time offset still has the same effect, setting\n\
time (as far as the sea is concerned) to be model's time plus offset.\n\
For instance, with a time offset of one-quarter the period, a pure sine\n\
wave will be transformed to a cosine; sin(t + period / 4) = cos(t).\n",
    SeaTime},

   {SeaGroup, "datum for sealevel oscillation",
"This is the water depth about which sealevel oscillates.  For instance, \n\
if this is set to 100 meters and amplitude is set to 50 m, the sealevel\n\
will oscillate between 50 and 150 meters.  This parameter is also the baseline\n\
for user-created sealevel files of waves; it is not relevant to sealevel \n\
files of points, nor to the Haq sealevel curve.  (See the help entry on\n\
sealevel files.)", SeaDatum},

   {SeaGroup, "sealevel oscillation amplitude",
"This specifies the amplitude of the sealevel as a sinusoid; it is only\n\
relevant if the sealevel is not specified by a file.", SeaAmp},

   {SeaGroup, "sealevel oscillation period",
"This specifies the period of the sealevel as a sinusoid, if the sealevel\n\
is not specified by a file.", SeaPeriod},


  {SubsidGroup, SubsidenceName, 
"This group controls subsidence and isostatic behavior.  Simple subsidence\n\
geometries (i.e. cratonic, foreland, or passive) are specified directly.\n\
Subsidence histories varying both spatially and temporally may be given\n\
in a subsidence specification file.  Essentially any vertical dependence\n\
can be simulated in such a history; however, there is never any dependence\n\
laterally (such as might be used to simulate fault movement).\n\
\n\
Whether the planet is to behave isostatically can also be specified.\n\
A simple elastic model is used where the properties of the lithosphere do\n\
not change in space or time."},

  {SubsidGroup, "subsidence rate",
"This is the driving subsidence applied to the basin.  If specified as a number,\n\
it is taken to be a constant subsidence; if as a file (by name), it is taken\n\
as described below.  The value zero and the filename \"none\" both indicate no\n\
subsidence.\n\
\n\
A file specifying subsidence well data has the form\n\
\n\
Well one:\n\
0e3\n\
-20e6   0.\n\
0       2e3\n\
\n\
Well two:\n\
600e3\n\
-20e6   0\n\
0       2e3\n\
\n\
end:  -1\n\
\n\
Each well is prefaced with an arbitrary amount of comment, possibly none.\n\
This is terminated by a colon (so don't put colons in your comments!).\n\
The next number is the horizontal position of the well.  The wells are in\n\
order from left to right.\n\
\n\
The next numbers are (age, depth) pairs.  Age is measured from the present \n\
and will therefore be nonpositive.  Depth is nonnegative.  The ages of the \n\
first point in each well are equal and are the starting time of the \n\
simulation; if the starting ages are not all equal, simbasin will complain\n\
and abort.  If the last age is not zero, its depth will be used until t = 0.\n\
Within each well age monotonically increases.  After the last well put\n\
      end:  -1 \n\
or something similar --- any comment you want, followed by a colon, followed\n\
by -1.\n\
\n\
We recommend giving subsidence files a .sub extension.\n\
\n\
\n\
The reason for this format is to be able to include directly \"backstripped\"\n\
results.  For example, a typical output of a backstripped one-dimensional\n\
analysis is the depth to basement through time. This is exactly the data in\n\
these subsidence files. The age-depth pairs record the position of basement\n\
through time starting at the beginning of the basin's evolution.",
   SubsidDrive},

   {SubsidGroup, "profile",
"This is one of three types, specified by name:\n\
\n\
'foreland' geometry: subsidence is maximum at left edge of basin and\n\
decreases linearly to zero at the right edge.\n\
\n\
'cratonic' (flat) geometry:  subsidence is constant across the basin.\n\
\n\
'passive' margin profile:  subsidence is maximum at the right edge of the\n\
basin and decreases linearly to zero at the left edge.\n\
\n\
\n\
\n\
With a positive subsidence rate, these look something like:\n\
\n\
\n\
                     passive profile \n\
-------------------------------------------------------------- \n\
\\                                                               | \n\
        \\                                                       | subsidence\n\
                 \\                                              | \n\
                         \\                                     \\|/  \n\
                                  \\ \n\
                                            \\ \n\
                                                      \\ \n\
                                                               \\ \n\
\n\
\n\
                      cratonic profile \n\
---------------------------------------------------------------- \n\
   |                   |                   |                    | \n\
   |                   |                   |                    | subsidence\n\
   |                   |                   |                    | \n\
  \\|/                 \\|/                 \\|/                  \\|/ \n\
----------------------------------------------------------------  \n\
\n\
\n\
\n\
\n\
                       foreland profile \n\
------------------------------------------------------------------ \n\
 |                                                              / \n\
 |subsidence                                              /          \n\
 |                                                 /      \n\
\\|/                                         /             \n\
                                     /                  \n\
                              /                          \n\
                      /                                 \n\
             / \n\
    / \n\
", Profile},



   {SubsidGroup, "flexural isostatic compensation",
"This Boolean parameter sets whether the sediment and water loads should be\n\
isostatically compensated.  If so, a Turcotte and Schubert analytic solution\n\
is used for the flexure of an elastic beam.  One effect of an infinite elastic\n\
beam is that the left side always appears 'underloaded' because there is no\n\
loading outside the range of the basin.", FlexComp},

   {SubsidGroup, "flexural rigidity",
"This specifies the flexural rigidity of the planet's crust, if flexural\n\
isostatic compensation has been turned on.  It is applied in an elastic\n\
flexure solution after Turcotte and Shubert.\n\
\n\
Our main isostasy algorithm does not handle low-rigidity cases well.  \n\
We are not sure whether this is a physical effect or a artifact of the\n\
algorithm chosen.  (Low rigidities are approximately those below, in \n\
m/kg/yrs units, 1e20.)  A rigidity of zero may be specified to simulate\n\
very-low-rigidity cases, as that case is handled specially.", FlexRigid},

   {SubsidGroup, "densities",
"The densities of air, crust (sediment), mantle, and water are used\n\
to calculate load compensation displacement.\n", DenseAir, DenseCrust,
  DenseMantle, DenseWater},

   {SubsidGroup, "gravitational constant",
"This specifies the acceleration due to gravity at planetary surface.", Grav},



   {CompactGroup, CompactionName, 
"This group controls both compaction behavior and lithologic determination.\n\
Compaction behavior may be left out to simplify the simulation.  If it is\n\
incorporated, porosity is taken to be an exponential function of depth (after\n\
Sclater and Christie), and sand and shale each have scaling constants that\n\
describe their compaction.  The porosity of a sand-shale mixture is assumed to\n\
vary linearly (as a function of percent sand) between that of the pure sand and\n\
shale end-members.\n\
\n\
Note that the simulation time is increased quite a bit when compaction is used\n\
(by about an order of magnitude); also the output files are significantly larger\n\
with compaction than without.  On the other hand, by incorporating compaction\n\
the thermal evolution (if any) and acoustic response of the basin will be\n\
much more accurately simulated.\n\
\n\
Lithology can be specified as either a function of the depositional diffusion\n\
constants (which are dependant on water depth) or as an exponential function\n\
of water depth (after a cutoff depth)."},

   {CompactGroup, "compact sediments?",
"This Boolean variable sets whether sediments will be compacted.  If not,\n\
porosities are set to zero everywhere.", CompactSed},

   {CompactGroup, "let erosion affect compaction?",
"This Boolean variable sets whether to assume irreversible compaction.",
   ErodeCompact},

   {CompactGroup, "material constants",
          "The porosities of sand and shale are calculated as\n\
\n\
           phi_s = phi_0_s * e^(c_s * zmax)\n\
\n\
where s is sand or shale, phi is the porosity, phi_0 is the specified\n\
initial porosity, c_s is the specified compaction decay constant, and\n\
zmax is the maximum depth that sediment layer has reached.\n\
\n\
Typical values for these constants are:\n\
\n\
material              c_s          phi_0\n\
  shale                .51e-3       .63\n\
  sand                 .27e-3       .49\n\
\n\
These are therefore the default values.", 
ShaleDecay, ShalePhi, SandDecay, SandPhi},

    {CompactGroup, "cutoff for sand composition",
"This sets how the sand/shale composition is calculated.  If it is negative,\n\
composition is determined by diffusion constants (with a linear fit between\n\
pure compositions at the exact marine and nonmarine diffusion constants).\n\
If it is zero or above, the composition is calculated from water depth.\n\
In this case, deposits made at a water depth less than the cutoff are taken to\n\
be pure sand; deposits made at a depth greater than the cutoff exponentially\n\
decay to shale with depth, as per the composition decay constant parameter.\n\
\n\
If sediments are being compacted, this affects the porosity calculations; if\n\
not, it does not affect the simulation at all.  Either way, however, plotbasin\n\
uses it when calculating composition.", SandCutoff},

   {CompactGroup, "decay constant for composition",
"This is only relevant if the sand composition cutoff is nonnegative; in this\n\
case, this sets the decay constant with which deposits decay from sand to shale\n\
with depth below the cutoff.", CompDecay},

  
  {HeatGroup, HeatName, 
"Parameters in this group control heat flow history.  Basal heat flux can be\n\
directly specified as a constant, or varied through time via a thermal flux\n\
specification file.  Basal heat flux does not vary spatially.  It may be\n\
left out of the model for simplicity.\n\
\n\
If a thermal flux is specified, simbasin performs thermal flow calculations.\n\
This is done with a simplistic steady-state model restricted to vertical\n\
transport only, solving the equation \n\
\n\
thermal flux = conductivity * (dT / dz).\n\
\n\
The conductivity is calculated from the parameters for the conductivities\n\
of sand, shale, and fluid, and from the porosity and sand/shale composition\n\
at each point.  Boundary conditions on the temperature are given by the\n\
surface temperature parameter, with another parameter defining a linear\n\
falloff in water.\n\
\n\
See the next help entry for a note on units in heat flow."},

  {HeatGroup, "units note",
"The units for thermal flux and thermal conductivity are typically given in\n\
mks, not m/k/yrs.  However, since they only enter the simulation in ratios\n\
--- specifically, thermal flux over conductivity --- it is just as valid to\n\
regard all values as being in m/k/yrs units instead of watts / meter degree\n\
and watts / meter^2.  The upshot is that if you have values in the latter\n\
units, you don't need to convert to m/k/yrs, as long as you give all of the\n\
thermal fluxes and conductivities in the same units.\n"},

  {HeatGroup, "thermal flux",
"If specified as a number, this is taken to be a constant thermal flux\n\
to be applied to the base of the basin.  If specified as a file (by name),\n\
that file is read for base thermal flux as a function of time.  The value\n\
zero or the name \"none\" both indicate no thermal flux; in this case no\n\
thermal calculations whatsoever are performed, and no thermal data is recorded.\n\
\n\
\n\
The format of a thermal flux file is as follows:\n\
\n\
0      66e-3\n\
1e6    60e-3\n\
5e6    70e-3\n\
\n\
where the first column is age and the second is thermal flux.  Exact spacing\n\
is unimportant.\n\
\n\
Typical continental heat fluxes are 0.056 and 0.078 W/m^2 in the oceanic and\n\
continental regimes respectively.\n\
\n\
Simbasin will linearly interpolate between (time, thermal flux) points to\n\
find what thermal flux should be applied to the base of the basin, extending\n\
the ends outward.\n\
\n\
We recommend giving thermal flux files a .therm extension.", 
    ThermalFlux},

   {HeatGroup, "thermal conductivities",
 "The thermal conductivities to use for sand, shale, and fluid when using\n\
the surface temperature and thermal flux to calculate temperatures.\n\
\n\
Thermal conductivities of quartz and shale are typically estimated as being\n\
approximately 3 W/m^2, while that of water is about .5 W/m^2.\n\
\n\
See the help entry on units.", SandConduct, ShaleConduct, FluidConduct},

   {HeatGroup, "surface temperature",
   "This is used as the boundary condition on the temperature at the \n\
surface --- which, if an area is underwater, means the surface of the\n\
*water* --- when solving the temperature equation.  Temperature change\n\
through water is governed by the surface temperature falloff parameter.\n\
\n\
Temperature is taken to be in Celsius.", SurfaceTemp},

   {HeatGroup, "surface temp. falloff",
"The temperature at the water-sediment interface of underwater areas\n\
is taken to be\n\
\n\
surface temperaure - (falloff * depth underwater),\n\
\n\
i.e. a linear decline.  (Temperatures below zero are brought to zero, i.e.\n\
freezing (all temperatures being in Celsius); this will not come into play\n\
in realistic situations on Earth.)\n\
\n\
This is used when solving the steady-state temperature equation.", TempFall} };
  
