
/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris  */
/* see file strata.h */

#include "strata.h"

/* Patch history, starting from v2.1:
   2.11:
           XawChain* changed to XtChain* for R4's sake
   2.12:
           newline added around line 805 in help.h
   2.13:
           helptext stuff messed with to avoid black blotches
	   added -sync option to XSychronize for debugging
	   added PageUp, PageDown bindings
	   fixed <, > bindings
	   void main() --> int
	   'skim' carbonate option added

   2.14:
           Xj & headers shared with simbasin moved to src/shared
	   isotope tracking added
	   misc.c fixed to not corrupt memory in dialog strings
	   int main() --> void (!)
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/List.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/Viewport.h>

#include "Xj/Canvas.c"
#include "Xj/Messenger.c"

#include "paramheader.h"
#include "paramvars.h"  /* needed before params.h */
#include "params.h"
#include "globals.h"
#include "fallbacks.h"

extern void convertlibs(int, char **);  /* for -convert */
extern void docaret(int);

extern void callloadsome(Widget, XEvent *, String *, Cardinal *);
extern void callsavesome(Widget, XEvent *, String *, Cardinal *);
extern void carbaction(Widget, XEvent *, String *, Cardinal *);
extern void carbcallback(Widget, XtPointer, XtPointer);
extern void carbdown(Widget, XtPointer, XtPointer);
extern void carbview(Widget, XtPointer, XtPointer);
extern void createhelp(Widget, Widget);
extern void defaultsome(Widget, XtPointer, XtPointer);
extern void docaret(int);
extern void goodbye(Widget, XtPointer, XtPointer);
extern Boolean loadfile(String, Boolean, Widget);
extern void loadsome(Widget, XtPointer, XtPointer);
extern void mybeep(Widget, XEvent *, String *, Cardinal *);
extern void placemenu(Widget, XEvent *, String *, Cardinal *);
extern void savesome(Widget, XtPointer, XtPointer);
extern void selectgroup(Widget, XtPointer, XtPointer);
extern void sethelptext(Widget, XtPointer, XtPointer);
extern void shiftcaret(Widget, XEvent *, String *, Cardinal *);
extern void shiftgroup(Widget, XEvent *, String *, Cardinal *);
extern void takecaret(Widget, XEvent *, String *, Cardinal *);
extern void textchange(Widget, XtPointer, XtPointer);
extern void varhelp(Widget, XEvent *, String *, Cardinal *);
extern void warnsome(Widget, XtPointer, XtPointer);

XtActionsRec action[] = {
  {"callloadsome", callloadsome},
  {"callsavesome", callsavesome},
  {"carbaction", carbaction},
  {"mybeep", mybeep},
  {"placemenu", placemenu},
  {"shiftcaret", shiftcaret},
  {"shiftgroup", shiftgroup},
  {"takecaret", takecaret},
  {"varhelp", varhelp} };

Group group;
int ngroups;
int curgroupnum = 0;
Widget sublabel, *dialog, toplevel, helpsub, helptext, helpview;
Widget alsograb[5], special_button, subtitle, needlabel, grouplist;
Widget carbshell, carbdial, carbcanvas, carbdraw, carbval, filesub;
XtAppContext app;
Display *display;
Window rootwin;
UniqueID *groupid;
Pixel white, black;
extern Boolean *anychange;

#define APPNAME "setbasin"
#define APPNAMEUP "Setbasin"

void main(int argc, char **argv)
{
  int i, j, k, n, ne, maxname, maxelem, maxdesc;
  int screenint;
  Widget topbox, fixedbox, highbox, subbox;
  Widget buttonbox, saveall, loadall, helpall, exitall, warnall, defall;
  Widget subbuttonbox, warnsub, defsub, filemenu, dialogbox;
  Widget carbbox, carbaxes, carbexit;
  Widget w, temp;
  char s[100], longname[100];
  Group g;
  Boolean flag;
  Dimension dim, height;
  XSizeHints sizehints;
  String *groupname;  

  if (argc > 1 && !strcasecmp(argv[1], "-help")) {
    printf("\n\tUsage:  %s [library1 library2 ...]\n\
A .dat suffix will be assumed for library arguments lacking one.\n\
\t%s -convert <library1> <library2>...\n\
will convert the library files' formats from version 2.0 format to the present.\n\n", APPNAME, APPNAME);
    exit(0);
  }

  printf("\n\
Strata version %g:  %s\n\
Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, and J. E. Morris.\n\
%s and the rest of Strata comes with ABSOLUTELY NO WARRANTY.  See the\n\
License help entry for details.\n\
This is free software, and you are welcome to redistribute it under certain\n\
conditions; please see the Gnu General Public License, of which you should\n\
have copies in the user manual and the source code, for details.\n\n",
	 STRATA_VERSION, APPNAME, APPNAMEUP);
  
  ngroups = 0;
  maxname = 0;
  maxdesc = 0;
  for (i = 0; param[i].elem != Endgroups; ++i)
    if (Newgroup == param[i].elem) {
      ++ngroups;
      if ((n = strlen(param[i].defval)) > maxname) {
	maxname = n;
	strcpy(longname, param[i].defval);
      }
    }
    else 
      if ((n = strlen(param[i].elem)) > maxdesc)
	maxdesc = n;
  
  group = (Group) XtCalloc(ngroups, sizeof(*group));
  g = &group[0];
  groupname = (String *) XtCalloc(ngroups, sizeof(*groupname));
  groupid = (UniqueID *) XtCalloc(ngroups + 1, sizeof(*groupid));
  groupid[ngroups] = GeneralID;
  title = (String *) XtCalloc(ngroups + 1, sizeof(String));
  anychange = (Boolean *) XtCalloc(ngroups, sizeof(Boolean));

  maxelem = 0;
  for (i = 0, n = 0; param[i].elem != Endgroups; ++n) {
    groupid[n] = param[i].id;
    group[n].name = param[i].defval;
    k = strlen(strcpy(s, group[n].name));
    for (j = k; j < maxname; ++j) strcat(s, " ");
    groupname[n] = XtNewString(s);
    group[n].need = !param[i].ok(group[n].name);
    ++i;
    for (group[n].nelem = 0; 
	 param[i].elem != Newgroup && param[i].elem != Endgroups; ++i) {
      ne = group[n].nelem++;
      k = strlen(strcpy(s, param[i].elem));
      for (j = k; j < maxdesc; ++j) strcat(s, " ");
      group[n].elem[ne] = XtNewString(s);
      group[n].truename[ne] = param[i].truename;
      group[n].val[ne] = XtNewString(param[i].defval);
      group[n].defval[ne] = param[i].defval;
      group[n].id[ne] = param[i].id;
      group[n].ok[ne] = param[i].ok;
    }
    if (group[n].nelem > maxelem)
      maxelem = group[n].nelem;
  }

  flag = False;
  for (i = 0; param[i].elem != Endgroups; ++i)
    if (param[i].elem != Newgroup)
      for (j = i + 1; param[j].elem != Endgroups; ++j)
	if (param[j].elem != Newgroup)
	  if (streq(param[i].truename, param[j].truename)) {
	    if (!flag)
	      printf("Bad, bad, bad, bad, BAD!\n");
	    printf("Params %d and %d have the same True Name, \"%s.\"\n",
		   i, j, param[i].truename);
	    flag = True;
	  }
  if (flag) {
    printf("True Names must be unique.  Aborting.\n");
    exit(1);
  }
  
  if (argc > 1 && !strcasecmp(argv[1], "-convert")) {
    convertlibs(argc, argv);
    exit(0);
  }
  
  /* use these instead of Xt(Va)AppInitialize so that
     XtAppSetFallbackResources can be used between app creation
     and display opening */
  XtToolkitInitialize();
  app = XtCreateApplicationContext();
  XtAppSetFallbackResources(app, fallbacks);
  display = XtOpenDisplay(app, NULL, NULL, APPNAME, 
                          (XrmOptionDescList) NULL, 0, &argc, argv);
  for (i = 1; i < argc; ++i)
    if (streq(argv[i], "-sync"))
      XSynchronize(display, True);
  toplevel =
    XtVaAppCreateShell(NULL, APPNAME, applicationShellWidgetClass, display,
                       XtNinput, True, XtNallowShellResize, True, 
                       NULL);
  XtAppAddActions(app, action, XtNumber(action));
  screenint = DefaultScreen(display);
  rootwin = RootWindow(display, screenint);
  black = BlackPixel(display, screenint);
  white = WhitePixel(display, screenint);

  for (i = 0; i <= ngroups; ++i)
    title[i] = XtNewString("defaults");
  settitle(NULL, True);

  topbox = XtVaCreateManagedWidget("topbox", formWidgetClass, toplevel,
				   XtNdefaultDistance, 0, 
				   XtNbackground, black,
				   NULL);
  
  fixedbox = XtVaCreateManagedWidget("fixedbox", formWidgetClass, topbox,
				     XtNdefaultDistance, 0,
				     XtNtop, XtChainTop,
				     XtNbottom, XtChainTop,
				     XtNborderWidth, 0,
				     NULL);

  highbox = XtVaCreateManagedWidget("highbox", formWidgetClass, fixedbox,
				    XtNdefaultDistance, 4,
				    NULL);

  grouplist = XtVaCreateManagedWidget("grouplist", listWidgetClass, highbox,
				      XtNdefaultColumns, 1,
				      XtNforceColumns, True,
				      XtNbottom, XtChainTop,
				      XtNtop, XtChainTop,
				      XtNright, XtChainRight,
				      XtNleft, XtChainRight,
				      XtNfromVert, NULL,
				      XtNfromHoriz, NULL,
				      XtNlist, groupname,
				      XtNnumberStrings, ngroups,
				      NULL);
  XtAddCallback(grouplist, XtNcallback, selectgroup, NULL);
  XawListHighlight(grouplist, 0);

  buttonbox = XtVaCreateManagedWidget("buttonbox", formWidgetClass, highbox,
				      XtNfromVert, grouplist, 
				      XtNfromHoriz, NULL,
				      XtNborderWidth, 0,
				      XtNdefaultDistance, 6,
				      NULL);

  n = 11;

  warnall = 
    XtVaCreateManagedWidget
      ("  warn   ", commandWidgetClass, buttonbox,
       XtNfromVert, NULL, XtNvertDistance, n,
       XtNaccelerators, XtParseAcceleratorTable
       ("#override Ctrl <Key> X, Ctrl <Key> W : set() notify() unset()"),
       NULL);
  XtAddCallback(warnall, XtNcallback, warnsome, (XtPointer)1);

  helpall = 
    XtVaCreateManagedWidget
      ("  help   ", commandWidgetClass, buttonbox,
       XtNfromVert, warnall, XtNvertDistance, n,
       NULL);
  XtOverrideTranslations
    (helpall, XtParseTranslationTable
     ("<BtnDown> : reset() placemenu() XtMenuPopup(menu)"));

  saveall = 
    XtVaCreateManagedWidget
      ("  save   ", commandWidgetClass, buttonbox,
       XtNfromVert, helpall, XtNvertDistance, n,
       XtNaccelerators, XtParseAcceleratorTable
       ("#override Ctrl <Key> X, Ctrl <Key> S : set() notify() unset()"),
       NULL);
  XtAddCallback(saveall, XtNcallback, savesome, (XtPointer)1);

  loadall = 
    XtVaCreateManagedWidget
      ("  load   ", commandWidgetClass, buttonbox,
       XtNfromVert, saveall, XtNvertDistance, n,
       XtNaccelerators, XtParseAcceleratorTable
       ("#override Ctrl <Key> X, Ctrl <Key> F : set() notify() unset() \n\
                   Ctrl <Key> X, Ctrl <Key> L : set() notify() unset()"), 
       NULL);
  XtAddCallback(loadall, XtNcallback, loadsome, (XtPointer)1);

  defall = 
    XtVaCreateManagedWidget
      ("defaults ", commandWidgetClass, buttonbox,
       XtNfromVert, loadall, XtNvertDistance, n,
       XtNaccelerators, XtParseAcceleratorTable
       ("#override Ctrl <Key> X, Ctrl <Key> D : set() notify() unset()"),
       NULL);
  XtAddCallback(defall, XtNcallback, defaultsome, (XtPointer)1);

  exitall = 
    XtVaCreateManagedWidget
      ("  exit   ", commandWidgetClass, buttonbox,
       XtNfromVert, defall, XtNvertDistance, n,
       XtNaccelerators, XtParseAcceleratorTable
       ("#override Ctrl <Key> X, Ctrl <Key> C : set() notify() unset() \n\
                   Ctrl <Key> X, Ctrl <Key> E : set() notify() unset()"),
       NULL);
  XtAddCallback(exitall, XtNcallback, goodbye, NULL);

  subbox = XtVaCreateManagedWidget("subbox", formWidgetClass, fixedbox,
				   XtNfromHoriz, highbox,
				   NULL);

  needlabel = 
    XtVaCreateManagedWidget("needlabel", labelWidgetClass, subbox,
			    XtNlabel, g->need ? "Mandatory" : " Optional ",
			    XtNfromVert, NULL, XtNfromHoriz, NULL,
			    XtNborderColor, white,
			    XtNhorizDistance, 20, 
			    NULL);

  sublabel = XtVaCreateManagedWidget("sublabel", labelWidgetClass, subbox,
				     XtNlabel, longname,
				     XtNborderColor, white,
				     XtNfromVert, NULL,
				     XtNfromHoriz, needlabel,
				     XtNhorizDistance, 100,
				     NULL);
  XtOverrideTranslations(sublabel, 
			 XtParseTranslationTable("<BtnDown> : varhelp(-1)"));
  
  subtitle = XtVaCreateManagedWidget("title", labelWidgetClass, subbox,
				     XtNlabel, "defaults",
				     XtNwidth, 150, XtNborderColor, white,
				     XtNfromVert, NULL,
				     XtNfromHoriz, sublabel,
				     XtNhorizDistance, 40,
				     NULL);
  
  dialogbox = XtVaCreateManagedWidget("dialogbox", formWidgetClass, subbox,
				      XtNfromVert, sublabel, 
				      XtNdefaultDistance, 0,
				      XtNvertDistance, 1,
				      NULL);

  subbuttonbox = XtVaCreateManagedWidget("subbuttonbox", formWidgetClass,
					 subbox, XtNfromVert, dialogbox,
					 XtNborderColor, white,
					 NULL);
  
  n = 25;

  warnsub = 
    XtVaCreateManagedWidget
      (" warn ", commandWidgetClass, subbuttonbox,
       XtNfromHoriz, NULL, XtNhorizDistance, n,
       XtNaccelerators, XtParseAcceleratorTable
       ("#override Ctrl <Key> G, Ctrl <Key> W : set() notify() unset()"),
       NULL);
  XtAddCallback(warnsub, XtNcallback, warnsome, (XtPointer)0);
  
  helpsub = 
    XtVaCreateManagedWidget
      (" help ", commandWidgetClass, subbuttonbox,
       XtNfromHoriz, warnsub, XtNhorizDistance, n,
       NULL);
  XtOverrideTranslations
    (helpsub, XtParseTranslationTable
     ("<BtnDown> : reset() placemenu() XtMenuPopup(menu0)"));

  filesub = 
    XtVaCreateManagedWidget("group file", menuButtonWidgetClass, subbuttonbox,
			    XtNfromHoriz, helpsub, XtNhorizDistance, n,
			    NULL);

  filemenu = XtVaCreatePopupShell("menu", simpleMenuWidgetClass, filesub,
				  XtNinput, True, NULL);
  XtVaCreateManagedWidget("line", smeLineObjectClass, filemenu, NULL);
  XtAddCallback(XtVaCreateManagedWidget
		("save group", smeBSBObjectClass, filemenu, NULL),
		XtNcallback, savesome, (XtPointer)0);
  XtAddCallback(XtVaCreateManagedWidget
		("load group", smeBSBObjectClass, filemenu, NULL),
		XtNcallback, loadsome, (XtPointer)0);
  
  defsub = 
    XtVaCreateManagedWidget
      ("defaults", commandWidgetClass, subbuttonbox,
       XtNfromHoriz, filesub, XtNhorizDistance, n,
       XtNaccelerators, XtParseAcceleratorTable
       ("#override Ctrl <Key> G, Ctrl <Key> D : set() notify() unset()"),
       NULL);
  XtAddCallback(defsub, XtNcallback, defaultsome, (XtPointer)0);

  special_button = XtVaCreateManagedWidget
    ("graph", commandWidgetClass, subbuttonbox,
     XtNfromHoriz, defsub, XtNhorizDistance, n,
     XtNmappedWhenManaged, CarbGroup == groupid[curgroupnum],
     NULL);
  XtAddCallback(special_button, XtNcallback, carbview, NULL);
  
  helpview = XtVaCreateManagedWidget("viewport", viewportWidgetClass, topbox,
				     XtNfromVert, fixedbox,
				     XtNallowHoriz, False,
				     XtNallowVert, True,
				     XtNresizable, True,
				     XtNheight, (Dimension)200,
				     XtNtop, XtChainTop,
				     XtNbottom, XtChainBottom,
				     NULL);
  
  /* this starts out unmanaged so that its size can be made to match
     that of the toplevel shell after realization */
  helptext = XtVaCreateWidget("help text", asciiTextWidgetClass,
			      helpview, XtNdisplayCaret, False,
			      XtNresizable, True,
			      XtNwidth, (Dimension)535, 
			      XtNheight, (Dimension)700,
			      NULL);
  
  /* helpview has to have *some* child unless helptext is managed, so:  */
  temp = XtVaCreateManagedWidget("temp", coreWidgetClass, helpview,
				 XtNwidth, 1, XtNheight, 1, NULL);

  createhelp(helpall, helpsub);

  carbshell = XtVaCreatePopupShell("carbonates", wmShellWidgetClass,
				   special_button, XtNinput, True, NULL);

  carbbox = XtVaCreateManagedWidget("carbbox", formWidgetClass, carbshell,
				    NULL);

  carbaxes = XtVaCreateManagedWidget
    ("carbaxes", labelWidgetClass, carbbox, XtNlabel, 
     "Depth versus carbonate sedimentation (as a percentage of maximum rate)",
     XtNborderColor, white, XtNfromVert, NULL, XtNfromHoriz, NULL,
     XtNtop, XtChainTop, XtNbottom, XtChainTop,
     XtNleft, XtChainLeft, XtNright, XtChainRight,
     NULL);

  carbcanvas = 
    XtVaCreateManagedWidget("canvas", canvasWidgetClass, carbbox,
			    XtNfromVert, carbaxes, XtNfromHoriz, NULL,
			    XtNleft, XtChainLeft, XtNright, XtChainRight,
			    XtNtop, XtChainTop, XtNbottom, XtChainBottom,
			    XtNheight, 600, XtNwidth, 600,
			    XtNbackground, black, XtNborderWidth, 0,
			    NULL);

  carbdial = 
    XtVaCreateManagedWidget("carbdial", dialogWidgetClass, carbbox,
			    XtNfromVert, carbcanvas, XtNfromHoriz, NULL,
			    XtNleft, XtChainLeft, XtNright, XtChainLeft,
			    XtNtop, XtChainBottom, XtNbottom, XtChainBottom,
			    XtNlabel, "Total Depth:     ", XtNvalue, "",
			    XtNdefaultDistance, 0,
			    NULL);

  carbval = XtNameToWidget(carbdial, "value");
  XtVaSetValues(carbval,
		XtNfromHoriz, XtNameToWidget(carbdial, "label"),
		XtNfromVert, NULL, XtNborderWidth, 0,
		NULL);
  XtOverrideTranslations
    (carbval, XtParseTranslationTable("<Key>Return : carbaction()"));
  
  carbdraw = 
    XtVaCreateManagedWidget("redraw", commandWidgetClass, carbbox,
			    XtNfromVert, carbcanvas, XtNfromHoriz, carbdial,
			    XtNleft, XtChainLeft, XtNright, XtChainLeft,
			    XtNtop, XtChainBottom, XtNbottom, XtChainBottom,
			    XtNhorizDistance, 20,
			    NULL);
  XtAddCallback(carbdraw, XtNcallback, carbcallback, NULL);

  carbexit =
    XtVaCreateManagedWidget("exit", commandWidgetClass, carbbox,
			    XtNfromVert, carbcanvas, XtNfromHoriz, carbdraw,
			    XtNleft, XtChainLeft, XtNright, XtChainLeft,
			    XtNtop, XtChainBottom, XtNbottom, XtChainBottom,
			    XtNhorizDistance, 20,
			    NULL);
  XtAddCallback(carbexit, XtNcallback, carbdown, NULL);

  dialog = (Widget *) XtCalloc(maxelem, sizeof(Widget));
  for (i = 0; i < maxelem; ++i) {
    sprintf(s, "dialog%02d", i);
    flag = (i < g->nelem);
    dialog[i] = 
      XtVaCreateManagedWidget(s, dialogWidgetClass, dialogbox,
			      XtNborderColor, white,
			      XtNlabel, flag ? g->elem[i] : g->elem[0],
			      XtNvalue, flag ? g->val[i] : "",
			      XtNmappedWhenManaged, flag, 
			      XtNfromVert, i ? dialog[i - 1] : NULL,
			      NULL);
    sprintf(s, "<Enter> : takecaret(%d) \n\
                <BtnDown> : varhelp(%d) ", i, i);
    XtOverrideTranslations(dialog[i], XtParseTranslationTable(s)); 
    w = XtNameToWidget(dialog[i], "value");
    XtVaSetValues(w,
		  XtNfromVert, NULL,
		  XtNfromHoriz, XtNameToWidget(dialog[i], "label"),
		  XtNdisplayCaret, False,
		  NULL);
    XtAddCallback(XawTextGetSource(w), XtNcallback, textchange, (XtPointer)i);
    XtOverrideTranslations(w, XtParseTranslationTable
			   ("<Key> Return : shiftcaret(+1) \n\
                             <Key> Tab : mybeep() \n\
                             <Key> space : mybeep() \n\
                             <Key> ' : mybeep() \n\
                             <Key> ` : mybeep() \n\
                             <Key> quotedbl : mybeep() \n\
                             <Key> Down : shiftcaret(+1) \n\
                             <Key> Up : shiftcaret(-1) \n\
                             :<Key> > : shiftgroup(+1) \n\
                             :<Key> < : shiftgroup(-1) \n\
                             <Key> Prior : shiftgroup(-1) \n\
                             <Key> Next : shiftgroup(+1) \n\
                             Ctrl <Key> G : \n\
                             Ctrl <Key> G, Ctrl <Key> S : callsavesome(0) \n\
                             Ctrl <Key> G, Ctrl <Key> L : callloadsome(0) \n\
                             Ctrl <Key> G, Ctrl <Key> F : callloadsome(0) "));
    XtInstallAccelerators(w, warnall);
    
    XtInstallAccelerators(w, saveall);
    XtInstallAccelerators(w, loadall);
    XtInstallAccelerators(w, defall);
    XtInstallAccelerators(w, exitall);
    XtInstallAccelerators(w, warnsub);
    XtInstallAccelerators(w, defsub);
  }
  
  alsograb[0] = exitall;
  alsograb[1] = helpall;
  alsograb[2] = helpsub;
  alsograb[3] = carbexit;
  alsograb[4] = NULL;

  XtRealizeWidget(toplevel);

  XSelectInput(display, XtWindow(toplevel),     /* this is needed */
	       KeyPressMask | XtBuildEventMask(toplevel));  
  
  XtVaGetValues(toplevel, XtNwidth, &dim, NULL);
  XtVaSetValues(helpview, XtNwidth, dim, NULL); /* this is needed too */
/*  XtVaSetValues(helptext, XtNwidth, dim, NULL); */ /* is this ever needed? */
  /* XtVaGetValues(helpview, XtNwidth, &dim, NULL); */ /* mustn't be used */
  XtUnmanageChild(temp);
  XtManageChild(helptext);
  XtVaSetValues(helptext, XtNwidth, dim, NULL); /* need on some screens */

  XtVaGetValues(toplevel, XtNwidth, &dim, XtNheight, &height, NULL);
  sizehints.min_width = sizehints.max_width = dim;
  XtVaGetValues(fixedbox, XtNheight, &dim, NULL);
  sizehints.min_height = dim;
  XtVaGetValues(helptext, XtNheight, &dim, NULL);
  sizehints.max_height = height + dim;
  sizehints.flags = PMinSize | PMaxSize; 
  XSetNormalHints(display, XtWindow(toplevel), &sizehints);  

  XtVaSetValues(sublabel, XtNlabel, g->name, NULL);
  sethelptext(NULL, (XtPointer)-1, NULL);
  
  docaret(0);

  for (i = 1; i < argc; ++i)
    if (argv[i][0] != '-' && argv[i - 1][0] != '-')
      if (loadfile(argv[i], True, loadall))
	settitle(argv[i], True);

  XtAppMainLoop(app);
}


extern Boolean iscarbgraphup, redrawon;
extern void getgroupvalues(void);
extern void agegroup(void);

void textchange(Widget w, XtPointer clientData, XtPointer callData)
{
  int i;
  Group g = &group[curgroupnum];

  getgroupvalues();
  
  switch (g->id[(int)clientData]) {
  case EpeiricDepth:  case OceanicDecay:  case OceanicDepth:
    if (iscarbgraphup && !redrawon) 
      XtSetSensitive(carbdraw, redrawon = True);      
    break;
  case Age1: case Age2: case Age3: case Age4: case Age5: case Age6:
    if (stringtobool(idtoval(g, EvenAges))) {
      XtFree(g->val[i]);
      g->val[i] = XtNewString("No");
      setgroupvalues();
    }
    break;
  case EvenAges: case TotalTime:
    agegroup();
    break;
  default:
    break;
  }  

  settitle("()", True);
  settitle("()", False);
}


static void booltostring(String s, int bool)
{ strcpy(s, bool ? "True" : "False"); }

static void parsevarstostring(String s, String file, double x)
{
  if (!strcasecmp(file, "none"))
    sprintf(s, "%g", x);
  else
    strcpy(s, file);
}

static void vartofile(FILE *fp, String name, void *var, String format)
{
  if (!var || !format) return;

  if (name)
    fprintf(fp, "%s\t ", name);
 
  if ('s' == format[1])
    fprintf(fp, "%s\n", (String)var);
  else if ('d' == format[1])
    fprintf(fp, "%d\n", *(int *)var);
  else
    fprintf(fp, "%g\n", *(double *)var);
}


void convertlibs(int argc, char **argv)
{
  int i, j, nvars;
  FILE *fp;
  char lib[100], orig[100];
  char no[100];
  Boolean trouble;

  if (argc < 2) {
    printf("No libraries given to convert.\n");
    return;
  }

  nvars = XtNumber(oldvar);
  strcpy(no, "No");

  for (i = 2; i < argc; ++i) {
    datstring(strcpy(lib, argv[i]));
    if ((trouble = !(fp = fopen(lib, "r")))) 
      printf("Unable to open %s for reading.\n", lib);
    else { 
      for (j = 0; !trouble && j < nvars; ++j)
	trouble |= (fscanf(fp, oldvar[j].format, oldvar[j].var) != 1);
      fclose(fp);
      if (trouble)
	printf("Old-style library %s badly formatted.\n", lib);
      else {
	undatstring(strcpy(orig, argv[i]));
	strcat(orig, ".orig");
	if (!(fp = fopen(orig, "w")))
	  printf("Unable to open %s to save original library.\n", orig);
	else {
	  for (j = 0; j < nvars; ++j) 
	    vartofile(fp, NULL, oldvar[j].var, oldvar[j].format);
	  fclose(fp);
	}
	if (!(fp = fopen(lib, "w")))
	  printf("Unable to open %s to write new library.\n", lib);
	else {
	  /* convert a few vars by hand */
	  booltostring(stringmarine, marine);
	  booltostring(compensatestring, compensate);
	  booltostring(compactionstring, compaction);
	  booltostring(erosionstring, erosion);
	  parsevarstostring(fluxleftstring, fluxfileleft, fluxleft);
	  parsevarstostring(fluxrightstring, fluxfileright, fluxright);
	  parsevarstostring(subsidencestring, subsidencefile, sub);
	  parsevarstostring(thermalstring, thermfile, thermflux);
	  for (j = 0; param[j].elem != Endgroups; ++j)
	    if (param[j].elem != Newgroup) {
	      if (EvenAges == param[j].id)
		vartofile(fp, param[j].truename, no, "%s");
	      else if (DenseWater == param[j].id)
		vartofile(fp, param[j].truename, param[j].defval, "%s");
	      else
		vartofile(fp, param[j].truename, param[j].var, 
			  param[j].format);
	    }
	  fclose(fp);
	  printf("Converted library %s to new format.\n", lib);
	}
      }
    }
  }
}


int hascaret = 0;

void docaret(int n)
{
  Widget v;
  
  while ((n = (n + group[curgroupnum].nelem) % group[curgroupnum].nelem) < 0);
  
  v = XtNameToWidget(dialog[hascaret], "value");
  XawTextDisplayCaret(v, False);

  hascaret = n;

  v = XtNameToWidget(dialog[hascaret], "value");
  XawTextDisplayCaret(v, True);
  XawTextSetInsertionPoint
    (v, strlen(XawDialogGetValueString(dialog[hascaret])));
  XtSetKeyboardFocus(toplevel, dialog[hascaret]);  
}

void takecaret(Widget w, XEvent *e, String *s, Cardinal *c)
{
  int n;

  sscanf(*s, "%d", &n);

  if (hascaret != n) 
    docaret(n);
}

void shiftcaret(Widget w, XEvent *e, String *s, Cardinal *c)
{
  int shift;

  sscanf(*s, "%d", &shift);

  docaret(hascaret + shift);
}
