/* Based on MIT's copywritten Template widget.  modifications copyright circa
   1992 by James Morris and John Grotzinger.  Permissions to use etc are 
   granted as per those in the X11 copyright.  No warranties or guarantees 
   are made implicity or explicity; no responsiblity is taken for anything 
   involving use of this software; no claims are made as to its performance 
   or any other aspect; use at your own risk.  */

#include <X11/IntrinsicP.h>  
#include <X11/StringDefs.h>  
#include "CanvasP.h"

static void Initialize(Widget request, Widget new, 
		       ArgList arglist, Cardinal *num_args)
{
  ((CanvasWidget)new)->canvas.needpixmap = True;
  return;
}

static void Newpixmap(Widget w, XEvent *junkevent, Region junkregion)
{
  CanvasWidget cw = (CanvasWidget)w;
  Dimension width, height;
  Cardinal depth;
  Pixel fg;
  Display *disp = XtDisplay(w);
  Window win = XtWindow(w);

  if (!(cw->canvas.needpixmap)) return;

  cw->canvas.gc = XCreateGC(disp, win, 0, 0);
  XtVaGetValues(w, XtNwidth, &width, XtNheight, &height,
		XtNdepth, &depth, XtNbackground, &fg, NULL);
  cw->canvas.pixmap = XCreatePixmap(disp, win, width, height, depth);
  XSetForeground(disp, cw->canvas.gc, fg);
  XFillRectangle(disp, cw->canvas.pixmap, cw->canvas.gc, 0, 0, width, height);
  cw->canvas.needpixmap = False;
  XtVaSetValues(w, XtNbackgroundPixmap, cw->canvas.pixmap, NULL);
}

static void Repixmap(Widget w)
{
  Dimension width, height;
  Cardinal depth;
  Pixmap pmap;
  Pixel fg;
  CanvasWidget cw = (CanvasWidget)w;
  Display *disp = XtDisplay(w);

  if (cw->canvas.needpixmap) return;

  XtVaGetValues(w, XtNwidth, &width, XtNheight, &height,
		XtNdepth, &depth, XtNbackground, &fg, NULL);
  pmap = XCreatePixmap(disp, XtWindow(w), width, height, depth);
  XSetForeground(disp, cw->canvas.gc, fg);
  XFillRectangle(disp, pmap, cw->canvas.gc, 0, 0, width, height);
  XCopyArea(disp, cw->canvas.pixmap, pmap, cw->canvas.gc, 
	    0, 0, width, height, 0, 0);
  XFreePixmap(disp, cw->canvas.pixmap);
  XtVaSetValues(w, XtNbackgroundPixmap, cw->canvas.pixmap = pmap, NULL);
}

static Boolean CanvasSetValues(Widget old, Widget request, Widget new, 
			       ArgList args, Cardinal *numargs)
{
  Boolean redisplay = False;
  CanvasWidget cw = (CanvasWidget)new;
  CanvasWidget ocw = (CanvasWidget)old;
  Display *disp = XtDisplay(new);

  /* if backgroundPixmap was changed --- change it back! */
  if (!cw->canvas.needpixmap)
    cw->core.background_pixmap = cw->canvas.pixmap;

  /* if the background color was changed, have canvas fg match */
  if (cw->core.background_pixel != ocw->core.background_pixel) {
    XSetForeground(disp, cw->canvas.gc, cw->core.background_pixel);
    ClearCanvas(cw);
    redisplay = True;
  }

  return redisplay;
}

static void DestroyCanvas(Widget w)
{
  CanvasWidget cw = (CanvasWidget)w;
  Display *disp = XtDisplay(w);

  XFreeGC(disp, cw->canvas.gc);
  XFreePixmap(disp, cw->canvas.pixmap);
}

CanvasClassRec canvasClassRec = {
  { /* core fields */
    /* superclass		*/	(WidgetClass) &widgetClassRec,
    /* class_name		*/	"Canvas",
    /* widget_size		*/	sizeof(CanvasRec),
    /* class_initialize		*/	NULL,
    /* class_part_initialize	*/	NULL,
    /* class_inited		*/	FALSE,
    /* initialize		*/	Initialize,
    /* initialize_hook		*/	NULL,
    /* realize			*/	XtInheritRealize,
    /* actions			*/	NULL,
    /* num_actions		*/	0,
    /* resources		*/	NULL,
    /* num_resources		*/	0,
    /* xrm_class		*/	NULLQUARK,
    /* compress_motion		*/	TRUE,
    /* compress_exposure	*/	TRUE,
    /* compress_enterleave	*/	TRUE,
    /* visible_interest		*/	FALSE,
    /* destroy			*/	DestroyCanvas,
    /* resize			*/	Repixmap,
    /* expose			*/	Newpixmap,
    /* set_values		*/	CanvasSetValues,
    /* set_values_hook		*/	NULL,
    /* set_values_almost	*/	XtInheritSetValuesAlmost,
    /* get_values_hook		*/	NULL,
    /* accept_focus		*/	NULL,
    /* version			*/	XtVersionDontCheck,
    /* callback_private		*/	NULL,
    /* tm_table			*/	NULL,
    /* query_geometry		*/	XtInheritQueryGeometry,
    /* display_accelerator	*/	XtInheritDisplayAccelerator,
    /* extension		*/	NULL
  },
  { /* canvas fields */
    /* empty			*/	0
  }
};

WidgetClass canvasWidgetClass = (WidgetClass)&canvasClassRec;

void ClearCanvas(Widget w)
{
  CanvasWidget cw = (CanvasWidget)w;
  Dimension width, height;
  Cardinal depth;
  Pixel fg;
  Display *disp = XtDisplay(w);

  XtVaGetValues(w, XtNwidth, &width, XtNheight, &height,
		XtNdepth, &depth, XtNbackground, &fg, NULL);
  XSetForeground(disp, cw->canvas.gc, fg);
  XFillRectangle(disp, cw->canvas.pixmap, cw->canvas.gc, 0, 0, width, height);
}

void ExposeCanvas(Widget w)
{
  XClearArea(XtDisplay(w), XtWindow(w), 0, 0, 0, 0, True);
}
  
Pixmap CanvasPixmap(Widget w)
{
  return ((CanvasWidget)w)->canvas.pixmap;
}
