/* This code utilized the copywritten MIT Athena X widget set and other than
   that is under copyright by James Morris and John Grotzinger circa 1993.  
   Permissions to use etc are granted as per those in the X11 copyright.  
   No warranties or guarantees are made implicity or explicitly, nor is any 
   responsiblity is taken for anything involving use of this software; no 
   claims are made as to its performance or any other aspect; 
   use at your own risk. */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Dialog.h>

/* make it safe to include more than once */
#ifndef _XjMessenger_defined
#define _XjMessenger_defined

/*  This file contains everything needed to use the function

Boolean XjMessenger(parent, message, allow_abort, answer, suggest, grab_list);
     Widget parent;            widget to use as parent of messenger shell
     String message;           string to display 
     Boolean allow_abort;      whether to have 'abort' button
     char answer[];            string to return user input; NULL if no
				input wanted, otherwise allocated char[] 
     char suggest[];           string to start with in text input; pointless
                                if answer==NULL, of course.  can use NULL
				instead of "".
     Widget *grab_list;        NULL-terminated list of widgets to add to
				the grab list so that they will also function
				while the message is up; or NULL for none

	A transient shell is created as a child of parent, with a dialog for
a child.  The dialog displays message as its label.  The dialog has a confirm
button, and if allow_abort is True it has an abort button.  If answer is not
NULL it has a text space for user input.  The shell does an exclusive grab,
adding those widgets in grab_list, or none if grab_list is NULL.  The widgets
in grab_list are the only ones other than the messenger that will receive
keyboard and pointer events until the messenger pops down.  If the confirm 
button is pressed, XjMessenger returns with a value of True and the user-given
string in answer (if answer != NULL); MAKE SURE the answer array is long enough
to hold the string, or you'll get a segmentation error!  If the abort button is
pressed XjMessenger returns with a value of False.  The messenger pops down 
when either button is pressed.
	You may have more than one messenger up at a time.
*/


static void xjmsg_response(Widget w, XtPointer clientData, XtPointer callData)
{
  int *response = (int *) clientData;

  *response = (strcmp(XtName(w), "confirm") ? 0 : 1);
}


extern void xj_fake_click(Widget, int);

Boolean XjMessenger(Widget parent, String message, Boolean allow_abort, 
		    char answer[], char suggest[], 
		    Widget hasfocus, Widget *grab_list)
{
  Widget shell, dialog, confirm, abrt, val, top, w;
  int response, i, x, y;
  Display *disp = XtDisplay(parent);
  Window junkwin, root = XRootWindow(disp, XDefaultScreen(disp));
  XtAppContext context = XtWidgetToApplicationContext(parent);

  XTranslateCoordinates(disp, XtWindow(parent), root, 0, 0, &x, &y, &junkwin);

  shell = XtVaCreatePopupShell("message", transientShellWidgetClass, parent,
			     XtNinput, True, XtNx, x, XtNy, y, NULL);

  dialog = XtVaCreateManagedWidget
    ("dialog", dialogWidgetClass, shell,
     XtNlabel, message,
     XtNvalue, answer ? (suggest ? suggest : "") : NULL,
     NULL);
  
  for (w = parent; w; w = XtParent(w))
    top = w;
  XtSetKeyboardFocus(top, dialog); 

  if (answer != NULL)     
    XtVaSetValues(val = XtNameToWidget(dialog, "value"),
		  XtNborderWidth, 0, NULL);
  
  XawDialogAddButton(dialog, "confirm", xjmsg_response, &response);
  XtVaSetValues(confirm = XtNameToWidget(dialog, "confirm"), 
		XtNaccelerators, XtParseAcceleratorTable
		("#override <Key> Return : set() notify() unset() \n\
                              Ctrl <Key> C : set() notify() unset() "),
		NULL);
  XtInstallAccelerators(answer ? val : dialog, confirm);
  
  if (allow_abort) {
    XawDialogAddButton(dialog, "abort", xjmsg_response, &response);
    XtVaSetValues(abrt = XtNameToWidget(dialog, "abort"), 
		  XtNaccelerators, XtParseAcceleratorTable
		  ("#override Ctrl <Key> A : set() notify() unset() "), 
		  NULL);
    XtInstallAccelerators(answer ? val : dialog, abrt);
  }    

  XtPopupSpringLoaded(shell);

  if (suggest) {
    xj_fake_click(val, 1);
    XawTextSetInsertionPoint(val, strlen(suggest));
  }
 
  if (grab_list != NULL)
    for (i = 0; grab_list[i] != NULL; ++i)
      XtAddGrab(grab_list[i], False, False);

  for (response = -1; -1 == response; XtAppProcessEvent(context, XtIMAll));
  
  if (answer != NULL)
    strcpy(answer, XawDialogGetValueString(dialog));

  XtSetKeyboardFocus(top, hasfocus ? hasfocus : top);
  
  XtPopdown(shell);
  XtDestroyWidget(shell);

  return (response ? True : False);
}

/* don't redefine xj_fake_click */
#ifndef _xj_fake_click_defined
#define _xj_fake_click_defined

/* send a fake ButtonPress and ButtonRelease from button # button
   to 0, 0 on a widget */
void xj_fake_click(Widget widget, int button)
{
  XEvent event;
  Window junk;

  event.type = ButtonPress;
  event.xany.serial = (int) NULL;
  event.xany.send_event = True;
  event.xany.display = XtDisplay(widget);
  event.xbutton.window = XtWindow(widget);
  event.xbutton.root = XRootWindow(event.xany.display,
                                   XDefaultScreen(event.xany.display));
  event.xbutton.subwindow = None;
  event.xbutton.time = XtLastTimestampProcessed(event.xany.display);
  event.xbutton.x = 0;
  event.xbutton.y = 0;
  XTranslateCoordinates(event.xany.display, event.xbutton.window,
                        event.xbutton.root, event.xbutton.x, event.xbutton.y,
                        &event.xbutton.x_root, &event.xbutton.y_root, &junk);
  event.xbutton.state = 0;
  event.xbutton.button = button;
  event.xbutton.same_screen = True;

  XtDispatchEvent(&event);

  event.type = ButtonRelease;
  event.xbutton.time = XtLastTimestampProcessed(event.xany.display);
  XtDispatchEvent(&event);

  return;
}
#endif  /* _xj_fake_click_defined */

#endif /* _XjMessenger_defined */
