/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris  */
/* see file strata.h */
#include "strata.h"


/*   This needs to be the SAME file in simbasin and setbasin!     */



#include "paramheader.h"
#include "paramfuncs.h"


char Endgroups[] = "-";
char Newgroup[] = "+";

/* this must remain the same, since simbasin and setbasin communicate w it */
char NoFile[] = "none";

char f[] = "%lf";
char s[] = "%s";

char MeasureName[] = "Measures";
char AgeName[] = "Ages";
char ClasticName[] = "Clastics";
char CarbonateName[] = "Carbonates";
char SeaName[] = "Sealevel";
char SubsidenceName[] = "Subsidence";
char CompactionName[] = "Compaction";
char HeatName[] = "Heat Flow";


struct {
  String elem;
  UniqueID id;
  String truename;      /* must stay the same, since simbasin uses them! */
  String defval;
  
  /* for a var, whether value is ok; for group, whether it's ok to punt
     elements of the group; note always ok to punt NULL vars */
  Boolean (*ok)(String);  
  String format;
  void *var;
} param[] = {

  {Newgroup, MeasureGroup, NULL, MeasureName, notfine},
  {"total width covered (L):", TotalWidth, "total_width", "500e3", 
     posdef, f, &fullspace},
  {"spatial divisions ( ): ", SpatialDiv, "spatial_divisions", "50", 
     posint, f, &doublennodes},
  {"total time covered (T):", TotalTime, "total_time", "10e6",
     posdef, f, &fulltime},
  {"temporal divisions ( ):", TimeDiv, "time_divisions", "1000", 
     posint, f, &doublethrust},
  {"# timelines ( ):", NumTimelines, "num_timelines", "100",
     posint, f, &doublentlines},

  {Newgroup, AgeGroup, NULL, AgeName, notfine},
  {"use even increments:", EvenAges, "even_ages", "Yes", bool, NULL, NULL},
  {"age of 1st time slice:", Age1, "age_1", "1.67e6", posdef, f, cut+1},
  {"age of 2nd time slice:", Age2, "age_2", "3.33e6", posdef, f, cut+2},
  {"age of 3rd time slice:", Age3, "age_3", "5.00e6", posdef, f, cut+3},
  {"age of 4th time slice:", Age4, "age_4", "6.67e6", posdef, f, cut+4},
  {"age of 5th time slice:", Age5, "age_5", "8.33e6", posdef, f, cut+5},
  {"age of 6th time slice:", Age6, "age_6", "10.00e6", posdef, f, cut+6},
  
  {Newgroup, ClasticGroup, NULL, ClasticName, notfine},
  {"nonmarine diffusion constant (L^2 / T):", NonmarineDiffus, 
     "nonmarine_diffusion_constant", "1e5", posdef, f, &ksed},
  {"simulate marine sedimentation? (T / F):", MarineSed, "sim_marine_sed",
     "True", bool, s, stringmarine},
  {"marine diffusion constant (L^2 / T):", MarineDiffus, 
     "marine_diffusion_constant", "100", posdef, f, &kmarine},
  {"decay coeff. for marine diffusion constant (1 / L):", MarineDecay, 
     "marine_diffusion_decay", "0.1", fine, f, &lambdadif},
  {"left clastic flux (L^2 / T):", LeftFlux, "left_flux", "5", fine, 
     s, fluxleftstring},
  {"right clastic flux (L^2 / T):", RightFlux, "right_flux", NoFile, fine,
     s, fluxrightstring}, 
  {"pelagic sedimentation rate (L / T):", Pelagic, "pelagic_rate", "0", fine,
     f, &pelagic},

  {Newgroup, CarbGroup, NULL, CarbonateName, fine},
  {"carbonate file:", CarbFile, "carbonate_file", NoFile, fine,
     s, carbonatefile},
  {"epeiric CaCO3 sed. - max. rate (L / T):", EpeiricMax, "epeiric_max",
     "1e-3", nonneg, f, &carbepeiricsur},
  {"epeiric CaCO3 sed. - depth of max rate (L):", EpeiricDepth, 
     "epeiric_depth", "1.0", posdef, f, &carbepeiricdep},
  {"oceanic CaCO3 sed. - max. rate (L / T):", OceanicMax, "oceanic_max",
     "1e-3", nonneg, f, &carboceanicsur}, 
  {"oceanic CaCO3 sed. - exp. decay constant (1 / L):", OceanicDecay, 
     "oceanic_decay", "0.1", nonneg, f, &carboceanicsca},
  {"oceanic CaCO3 sed. - depth of max. rate (L):", OceanicDepth, 
     "oceanic_depth", "1.0", posdef, f, &carboceanicdep},
  {"lag on zero water depth", Skim, "carb_skim", "True", bool, s, skimstring},
  {"isotope signal file", Isotopes, "isotope_file", NoFile, fine, s,
   isotopefile},
  {"isotope offset time", IsoTime, "isotope_time", "0", nonpos, f, &isostart},
  
  {Newgroup, SeaGroup, NULL, SeaName, fine},
  {"sealevel file:", SeaFile, "sealevel_file", NoFile, fine,
     s, eustaticfile},
  {"time offset (T):", SeaTime, "sea_time_offset", "0", fine, f, &start},
  {"datum for sealevel oscillation (L):", SeaDatum, "sea_datum", "0", fine,
     f, &depth},
  {"sealevel oscillation amplitude (L):", SeaAmp, "sea_amp", "0", nonneg,
     f, &amplitude},
  {"sealevel oscillation period (T):", SeaPeriod, "sea_period", "1e6", posdef,
     f, &period},
  
  {Newgroup, SubsidGroup, NULL, SubsidenceName, notfine},
  {"subsidence rate (L / T):", SubsidDrive, "subsidence", "-0.0001", fine,
     s, subsidencestring},
  {"profile (name):", Profile, "profile_name", "passive", testprofile,
     s, profile},
  {"flexural isostatic compensation? (T / F):", FlexComp, "flex_compensate",
     "False", bool, s, compensatestring},
  {"flexural rigidity (M L^2 / T^2):", FlexRigid, "flex_rigidity", "1e23", 
     nonneg, f, &flex},
  {"density of air (M / L^3):", DenseAir, "density_air", "1e-4", posdef,
     f, &da},
  {"density of crust (M / L^3):", DenseCrust, "density_crust", "2300", posdef,
     f, &dc},
  {"density of mantle (M / L^3):", DenseMantle, "density_mantle", "3300", 
     posdef, f, &dm},
  {"density of water (M / L^3):", DenseWater, "density_water", "1000",
     posdef, f, &dw},
  {"gravitational constant, g (L / T^2):", Grav, "local_grav", "9.81", posdef,
     f, &g},

  {Newgroup, CompactGroup, NULL, CompactionName, fine},
  {"compact sediments? (T / F):", CompactSed, "compact_sediments", "False", 
     bool, s, compactionstring},
  {"let erosion affect compaction? (T / F):", ErodeCompact, 
     "erosion_affects_compaction", "True", bool, s, erosionstring},
  {"decay constant for shale compaction (1 / L):", ShaleDecay, "shale_decay",
     ".51e-3", posdef, f, &cshale},
  {"initial porosity (phi) for shale ( ):", ShalePhi, "shale_phi", ".63", 
     isfraction, f, &phi0_shale},
  {"decay constant for sand compaction (1 / L):", SandDecay, "sand_decay", 
     ".27e-3", posdef, f, &csand},
  {"initial porosity (phi) for sand ( ):", SandPhi, "sand_phi", ".49", 
     isfraction, f, &phi0_sand},
  {"cutoff for sand composition (L):", SandCutoff, "sand_cutoff", "-1", fine,
     f, &compcut},
  {"decay constant for composition (1 / L):", CompDecay, 
     "compaction_composition_decay", ".1", posdef, f, &compdecay},

  {Newgroup, HeatGroup, NULL, HeatName, fine},
  {"thermal flux (M / T^3):", ThermalFlux, "thermal_flux", NoFile, fine, 
     s, thermalstring},
  {"thermal conductivity, sand (M L / d T^3):", SandConduct, 
     "sand_conductivity", "3.00", posdef, f, &sandtherm},
  {"thermal conductivity, shale (M L / d T^3):", ShaleConduct, 
     "shale_conductivity", "3.01", posdef, f, &shaletherm},
  {"thermal conductivity, fluid (M L / d T^3):", FluidConduct, 
     "fluid_conductivity", "0.50", posdef, f, &fluidtherm},
  {"surface temperature (d):", SurfaceTemp, "surface_temperature", "20", fine,
     f, &surtemp},
  {"surface temp falloff (d / L):", TempFall, "surface_temp_falloff", "0", 
     nonneg, f, &surtempfall},
  
  {Endgroups},
};


  
