
/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see file strata.h                                                 */
#include "strata.h"


/* create a linear subsidence pattern, where subsidence is a maximum at
   the left edge and goes to zero at the right edge */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "header.h"

double delta, sub, *subsid;
char subsidencefile[100];
double lowfluxright, highfluxright;
double lowfluxleft, highfluxleft;
double lowfluxtimeright, highfluxtimeright;
double lowfluxtimeleft, highfluxtimeleft;
double fluxright, fluxleft;
char fluxfileright[100], fluxfileleft[100];
char profile[20];

void barnsub(FILE *fpfluxleft, FILE *fpfluxright)
{
  int i, j, x, t[3];
  double height, drop, y[2];

  if (strcasecmp(subsidencefile, NoFile)) {  
    for (x = i = 0; i < nnodes; ++i) {
      while (i * delta > well[0][x+1][0]) x++;
      for (t[j = 0] = 1; j < 2; t[++j] = 1) 
	while (time1 > well[0][x+j][t[j] + 1] - well[0][x+j][1]) t[j]++;
      for (j = 0; j < 2; ++j)
	y[j] = (well[1][x+j][t[j]+1] - well[1][x+j][t[j]]) /
	  (well[0][x+j][t[j]+1] - well[0][x+j][t[j]]);
      subsid[i] = -(y[0] + (y[1] - y[0]) * (i * delta - well[0][x][0]) /
		    (well[0][x+1][0] - well[0][x][0])) * timedelt;
    }
  }
  else {
    drop = sub * timedelt;
    if (!strcasecmp(profile, "cratonic"))
      for (i = 0; i < nnodes; ++i)
	subsid[i] = drop;
    else if (!strcasecmp(profile, "foreland"))
      for (i = 0; i < nnodes; ++i)
	subsid[i] = drop * (1 - (double)(i + 1) / nnodes);
    else /* ought to be "passive" */
      for (i = 0; i < nnodes; ++i)
	subsid[i] = drop * (double) (i + 1) / nnodes;
  }
  for (i = 0; i < nnodes; ++i) {
    elev[i] += subsid[i];
    /* chelev ignores subsidence */
    tchelev[i] += subsid[i];
  }
  
  if (strcasecmp(fluxfileleft, NoFile)) {
    while (time1 < lowfluxtimeleft || time1 > highfluxtimeleft) {
      lowfluxtimeleft = highfluxtimeleft;
      lowfluxleft = highfluxleft;
       if (fscanf(fpfluxleft, "%lf %lf", 
		  &highfluxtimeleft, &highfluxleft) != 2) {
	 highfluxtimeleft = lowfluxtimeleft + fulltime;
	 highfluxleft = lowfluxleft;
       }
    }
    height = (timedelt / delta) * 
      (lowfluxleft + (highfluxleft - lowfluxleft) *
       (time1 - lowfluxtimeleft) / (highfluxtimeleft - lowfluxtimeleft));
  }
  else
    height = timedelt * fluxleft / delta;
  elev[2] += height;
  chelev[2] += height;
  
  if (strcasecmp(fluxfileright, NoFile)) {
    while (time1 < lowfluxtimeright || time1 > highfluxtimeright) {
      lowfluxtimeright = highfluxtimeright;
      lowfluxright = highfluxright;
      if (fscanf(fpfluxright, "%lf %lf",
		 &highfluxtimeright, &highfluxright) != 2) {
	highfluxtimeright = lowfluxtimeright + fulltime;
	highfluxright = lowfluxright;
      }
    }
    height = (timedelt / delta) * 
      (lowfluxright + (highfluxright - lowfluxright) *
       (time1 - lowfluxtimeright) / (highfluxtimeright - lowfluxtimeright));
  }
  else
    height = timedelt * fluxright / delta;
  elev[nnodes - 3] += height;
  chelev[nnodes - 3] += height;

  return;
}


