
/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see file strata.h                                                 */
#include "strata.h"

/* main driver for basin model
   reads input files
   sets up output files
   calls subroutines in loops
   writes output to data file for later graphics
   worked on by jeff masek to incorporate compaction */

/* patch history is in header.h */

#include <stdio.h>
#include <stdlib.h>
#include <math.h> 
#include <string.h>
#include <unistd.h>
#include "header.h"

#define indicatefilm "-MakeIntoFilm"

extern void barncomp(void);
extern void barnsub(FILE *, FILE *);
extern void carbread(void);
extern void deposit(void);
extern void doisotopes(void);
extern void friendly(int);
extern void moviesave(int); 
extern void reprop(FILE *);
extern void sdatout(void);
extern void seta(void);
extern void settime(FILE *);
extern void solve(void);
extern void unitdisp(void);
extern void wellread(void);

char *tag;
char **path;
int npaths;
  
double doublennodes, doublentlines, doublethrust;
double fulltime, fullspace;
int tlstep, timeloop[7];
double **timeline[7], timedelt, time1;
Boolean movie;
Boolean compensate;
int nframes, thrust, ntlines, nnodes;
char library[1000];
char ExxonSea[] = "exxon.sea";

extern double *dep;
extern Boolean skim;

extern int nisotopes;
extern String isoname[];
extern String isounits[];
extern double *isotope[];
extern double isostart;

Tempstring stringmarine, fluxleftstring, fluxrightstring, subsidencestring;
Tempstring compensatestring, compactionstring, erosionstring, thermalstring;
Tempstring skimstring;

#include "paramheader.h"
#include "params.h"  /* needs to be after var decls */

extern void parsestringtovars(String, String, double *);

	/* basmod.c */
int main(int argc, char **argv)
{
  int i, j, n;
  int sock;
  double movienexttime, movietimedelt, x;
  char temp[1000], s[100], v[100];
  FILE *fp, *fpsea, *fpfluxright, *fpfluxleft, *fptherm;
  Boolean flag, got[1000], trouble, mark;
  Boolean (*ok)(String);
  UniqueID id;

  if (1 == argc)  friendly(1);
  if (!strcasecmp(argv[1], "help")) friendly(2);

  /* cut .dat if present */
  strcpy(library, argv[1]);
  if (((i = strlen(library)) > 4) && !strcmp(library + i - 4, ".dat"))
    library[i - 4] = '\0';

  if ((movie = (argc > 2 && !strcmp(argv[2], indicatefilm)))) { 
    tag = argv[3];
    sscanf(argv[4], "%d", &nframes);
    sscanf(argv[5], "%d", &sock);
  }

  /* accept user-given paths */
  if ((npaths = argc - (n = (movie ? 5 : 2))) > 0) {
    path = (char **) calloc(npaths, sizeof(char *));
    for (i = n; i < argc; ++i) 
      path[i-n]=strcpy((char*)calloc(strlen(argv[i])+1, sizeof(char)),argv[i]);
  }

  if (!movie) 
    friendly(0);

  trouble = False;
  for (i = 0; param[i].elem != Endgroups; ++i)
    if (param[i].elem != Newgroup)
      for (j = i + 1; param[j].elem != Endgroups; ++j)
	if (param[j].elem != Newgroup)
	  if (streq(param[i].truename, param[j].truename)) {
	    if (!trouble)
	      printf("Bad, bad, bad, bad, BAD!\n");
	    printf("Params %d and %d have the same True Name, \"%s.\"\n",
		   i, j, param[i].truename);
	    trouble = True;
	  }
  if (trouble) {
    printf("True Names must be unique.  Aborting.\n");
    exit(1);
  }
  
  /* hand-init some strings so they can be checked after loading for
     certain values, even if they didn't actually get loaded */
  strcpy(eustaticfile, NoFile);
  strcpy(thermalstring, NoFile);
  strcpy(isotopefile, NoFile);

  for (i = 0; param[i].elem != Endgroups; ++i)
    got[param[i].id] = False;
  fp = openfile(strcat(strcpy(temp, library), ".dat"));
  trouble = False;
  while (myfgets(temp, 1000, fp))
    if (sscanf(temp, "%s %s", s, v) == 2)
      for (i = 0, flag = False; !flag && param[i].elem != Endgroups; ++i)
	if (param[i].elem != Newgroup)
	  if ((flag = !strcmp(param[i].truename, s)))
	    if (param[i].format && param[i].var)
	      if (sscanf(v, param[i].format, param[i].var) == 1) {
		got[param[i].id] = True;
		if (!param[i].ok(v)) {
		  printf("Unacceptable value of %s  %s\n",
			 param[i].elem, v);
		  trouble = True;
		}
	      }
  fclose(fp);
  for (i = 0; param[i].elem != Endgroups; ++i)
    if (Newgroup == param[i].elem)
      ok = param[i].ok;
    else if (param[i].var != NULL && !got[param[i].id]) 
      if (!ok(param[i].elem)) {
	printf("Unacceptable value of %s  (no value found)\n", param[i].elem);
	trouble = True;
      }
  /* check that things aren't fucked up in sets
     be careful to process any vars that need it before checking them... */
  flag = mark = True;
  for (i = 1; i < 7; ++i) {
    if (flag && cut[i] > fulltime) {
      printf("Slice ages must be no more than simulation age.\n");
      trouble = True;
      flag = False;
    }
    for (j = 1; mark && j < i; ++j)
      if (cut[i] < cut[j]) {
	printf("Slices cannot be younger than previous slices.\n");
	trouble = True;
	mark = False;
      }
  }
  if (!strcasecmp(eustaticfile, ExxonSea)) 
    if (start >= 0) {
      printf("For exxon.sea, offset time must be negative.\n");
      trouble = True;
    }
  if (dm <= da) {
    printf("Mantle must be denser than air.\n");
    trouble = True;
  }
  parsestringtovars(thermalstring, thermfile, &thermflux);
  if (got[ThermalFlux] && !strcmp(thermfile, NoFile) && thermflux < 0) {
    printf("Thermal flux may not be negative.\n");
    trouble = True;
    strcpy(thermalstring, NoFile);
  } 
  id = GeneralID;
  compaction = stringtobool(compactionstring);
  mark = True;
  if (compaction || (flag = strcmp(thermalstring, NoFile)))
    for (i = 0; param[i].elem != Endgroups; ++i)
      if (param[i].elem != Newgroup) {
	if (mark && compaction && (CompactGroup == id) && !got[param[i].id]) {
	  printf("With compaction, all compaction variables must be set.\n");
	  trouble = True;
	  mark = False;
	}
	if (flag && (HeatGroup == id) && !got[param[i].id]) {
	  printf("With heat flow, all heat flow variables must be set.\n");
	  trouble = True;
	  flag = False;
	}
      }
      else
	id = param[i].id;
  if (trouble) {
    printf("Library errors fatal.  Exiting.\n");
    exit(1);
  }

  /* translate library variables into internal variables */
  nnodes = (int) doublennodes;
  ntlines = (int) doublentlines;
  thrust = (int) doublethrust;
  delta = fullspace / (double)nnodes;
  timedelt = fulltime / (double)thrust;
  if (thrust % ntlines) {
    printf("Fatal error:  #temporal divisions (%d) is not", thrust);
    printf(" a multiple of #timelines (%d)\n", ntlines);
    exit(1);
  }
  tlstep = thrust / ntlines;
  if (movie) printf("\n");
  printf("Delta x %.1g; delta t %.1g (%.1g)\n", 
	 delta, timedelt, timedelt * tlstep);
  marine = stringtobool(stringmarine);
  compensate = stringtobool(compensatestring);
  compaction = stringtobool(compactionstring);
  erosion = stringtobool(erosionstring);
  skim = stringtobool(skimstring);
  parsestringtovars(fluxleftstring, fluxfileleft, &fluxleft);
  parsestringtovars(fluxrightstring, fluxfileright, &fluxright);
  parsestringtovars(subsidencestring, subsidencefile, &sub);
  parsestringtovars(thermalstring, thermfile, &thermflux);

  /* this has to go here so it's done before movie, so that the conductivities
     are set to zero if they should be, so the plotter knows */

  if (strcasecmp(thermfile, NoFile)) {
    fptherm = openfile(thermfile);
    lowthermtime = 2 * fulltime;
    highthermtime = -2 * fulltime;
    lowtherm = hightherm = 0;
    thermdo = TRUE;
  }
  else {
    fptherm = NULL;
    if (thermflux > 0)
      thermdo = TRUE;
    else {  
      sandtherm = shaletherm = fluidtherm = 0;  /* tells plotter therm off */
      thermdo = FALSE;
    }
  }

  if (movie) {
    strcat(library, ".film.");
    strcat(library, tag);
    if (nframes > ntlines) 
      printf("Lowering # frames to # timelines, %d.\n", nframes = ntlines);
    else if (!nframes)
      nframes = ntlines;
    else if (thrust % nframes)
      printf("Warning:  # frames is not a factor of # timesteps (%d).\n", 
	     thrust);
    movienexttime = movietimedelt = thrust * timedelt / nframes;
    printf("Taking snapshot every %.1g years\n", movietimedelt);

    for (i = 1; i <= 6; ++i) cut[i] = 0;
    sdatout();

    /* tell the plotter how many digits the highest age has, for slicelist */
    sprintf(temp, "%2d", (int) floor(log10((double)(thrust * timedelt))) + 1);
    write(sock, temp, 2);

    /* tell the plotter how many frames */
    sprintf(temp, "%d", nframes);
    strcat(temp, ".");
    write(sock, temp, strlen(temp));
  }

  if (!period && !strcasecmp(eustaticfile, NoFile))
    if (amplitude) {
      printf("Zero period (for sealevel) is unacceptable.\n");
      exit(1);
    }
    else period = 1e6;  /* arbitrary; doesn't matter since amplitude is 0 */

  /* pretty much everything here really should be init'd to zero explicitly
     calloc usually does that but shouldn't be relied upon */
  for (i = 0; i < 3; ++i) {
    aa[i] = (double *) calloc(nnodes, sizeof(double));
    bb[i] = (double *) calloc(nnodes, sizeof(double));
  }
  dep = (double *) calloc(nnodes, sizeof(double));
  difconst = (double *) calloc(nnodes, sizeof(double));
  disp = (double *) calloc(nnodes, sizeof(double));
  elev = (double *) calloc(nnodes, sizeof(double));
  chelev = (double *) calloc(nnodes, sizeof(double));
  for (i = 0; i < nnodes; ++i)
    chelev[i] = elev[i] = 0;
  exposed = (double *) calloc(nnodes, sizeof(double));
  cutoff = (Boolean *) calloc(nnodes, sizeof(Boolean));
  for (i = 0; i < nnodes; ++i) {
    exposed[i] = 0;
    cutoff[i] = False;
  }
  load = (double *) calloc(nnodes, sizeof(double));
  newwaterthick = (double *) calloc(nnodes, sizeof(double));
  oldwaterthick = (double *) calloc(nnodes, sizeof(double));
  for (i = 0; i < nnodes; ++i)
    oldwaterthick[i] = 0;
  sealevel = (double *) calloc(ntlines, sizeof(double));
  shoreboundary = (int *) calloc(ntlines, sizeof(int));
  subsid = (double *) calloc(nnodes, sizeof(double));
  tchelev = (double *) calloc(nnodes, sizeof(double));
  waterload = (double *) calloc(nnodes, sizeof(double));
  for (i = 0; i < 7; ++i) {
    timeline[i] = (double **) calloc(ntlines, sizeof(double *));
    for (j = 0; j < ntlines; ++j)
      timeline[i][j] = (double *) calloc(nnodes, sizeof(double));
  }
  xx = (double *) calloc(nnodes, sizeof(double));
  maxspace = (double **) calloc(ntlines, sizeof(double *));
  for (i = 0; i < ntlines; ++i) 
    maxspace[i] = (double *) calloc(nnodes, sizeof(double));    

  if (strcasecmp(eustaticfile, NoFile)) {
    fpsea = openfile(eustaticfile);
    if (strcasecmp(eustaticfile, ExxonSea)) 
      fscanf(fpsea, "%s", seatype);
    else strcpy(seatype, "points");
    switch (seatype[0]) {
    case 'p':  case 'P':
      seatype[0] = 'P';
      lowseatime = 2 * fulltime;
      highseatime = -2 * fulltime;
      lowsea = highsea = 0;
      break;
    case 'w':  case 'W':
      seatype[i = 0] = 'W';
      do fscanf(fpsea, "%lf %lf %lf %lf %lf", seawave[i], seawave[i]+1,
	seawave[i]+2, seawave[i]+3, seawave[i]+4);
      while (seawave[i++][2]);
      break;
    default:  
      printf("File %s not typed 'points' or 'waves'; exiting\n", eustaticfile);
      fclose(fpsea);
      exit(1);
      break;
    }
  }

  if (strcasecmp(subsidencefile, NoFile)) 
    wellread();

  if (strcasecmp(fluxfileleft, NoFile)) 
    fpfluxleft = openfile(fluxfileleft);
  lowfluxtimeleft = 2 * fulltime;
  highfluxtimeleft = -2 * fulltime;
  lowfluxleft = highfluxleft = 0;
  
  if (strcasecmp(fluxfileright, NoFile)) 
    fpfluxright = openfile(fluxfileright);
  lowfluxtimeright = 2 * fulltime;
  highfluxtimeright = -2 * fulltime;
  lowfluxright = highfluxright = 0;
  
  if (strcasecmp(carbonatefile, NoFile)) 
    carbread();

  if (strcasecmp(isotopefile, NoFile)) 
    doisotopes();

  for (i = 0; i < nnodes; ++i) difconst[i] = kmarine;

  timeloop[0] = 0;  /* should this be 1, instead? */
  
  if (compensate) 
    unitdisp();

  time1 = 0;
  for (i = 1; i <= thrust; ++i) {
    barnsub(fpfluxleft, fpfluxright);
    seta();
    solve();
    for (j = 0; j < nnodes; ++j) {
      x = xx[j] - elev[j];
      elev[j] += x;
      chelev[j] += x;
    }
    if (compensate) 
      barncomp();    
    reprop(fpsea);
    deposit();
    time1 = i * timedelt;  /* unfortunately this has to be in the middle */
    if (!(i % tlstep)) {
      timeloop[0]++;
      for (j = 0; j < nnodes; ++j) 
	maxspace[timeloop[0]-1][j] = timeline[3][timeloop[0]-1][j] = 0;
      settime(fptherm);
    }

    if (movie && time1 >= movienexttime) {
      movienexttime += movietimedelt;
      moviesave(sock);
    }
  }

  if (strcasecmp(eustaticfile, NoFile)) fclose(fpsea);
  if (strcasecmp(fluxfileleft, NoFile)) fclose(fpfluxleft);
  if (strcasecmp(fluxfileright, NoFile)) fclose(fpfluxright);

  if (movie) {
    read(sock, temp, 1); /* listen for plotter's last words */
    write(sock, "q", 1); /* tell plotter to quit */
    close(sock);
  }
  else 
    sdatout();

  return 0;
}


void parsestringtovars(String source, String file, double *var)
{
  int n;

  sscanf(source, "%lf%n", var, &n);

  if (n != strlen(source))
    strcpy(file, source);
  else
    strcpy(file, NoFile);
}


void sdatout()
{
  FILE *fp;
  int i, n;
  char temp[1000];
  
  fp = fopen(strcat(strcpy(temp, library), "dat.out"), "w");
  fprintf(fp, "%d %g\n", MODEL, STRATA_VERSION);
  fprintf(fp, "%g %d %d %d %g\n", delta, nnodes, thrust, tlstep, timedelt);
  fprintf(fp, "%g %g %g %g %g %g\n", cut[1], cut[2], cut[3], 
	  cut[4], cut[5], cut[6]);
  fprintf(fp, "%d %g %g %g %g\n", compaction, ksed, kmarine, 
	  compcut, compdecay);
  fprintf(fp, "%g %g %g\n", sandtherm, shaletherm, fluidtherm);
  fprintf(fp, "%d\n", nisotopes);
  for (n = 0; n < nisotopes; ++n) 
    fprintf(fp, "%s %s\n", isoname[n], isounits[n]);
  fclose(fp);
  
  fp = fopen(strcat(strcpy(temp, library), "s.out"), "w");
  for (i = 1; i < 7; ++i)
    fprintf(fp, "%d\n", timeloop[i]);
  for (i = 0; i < timeloop[6]; ++i)
    fprintf(fp, "%d\n", shoreboundary[i]);
  for (i = 0; i < timeloop[6]; ++i)
    fprintf(fp, "%g\n", sealevel[i]);
  for (n = 0; n < nisotopes; ++n) 
    for (i = 0; i < timeloop[6]; ++i)
      fprintf(fp, "%g\n", isotope[n][i]); 
  fclose(fp);
}
