
/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see file strata.h                                                 */
#include "strata.h"


/* subroutine of settime to reduce stratal thicknesses through 
   gravitational compaction */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "header.h"

double cshale, csand;
double compcut, compdecay;
Boolean erosion;
double **maxspace;
double phi0_sand, phi0_shale;

void compact()
{
  int i, j;
  double persand, pershale, difdif, x;
  double zold[2], znew[2], volpornew, volporold;
  double compsum, top1, top2;
  double *shrink;

  if (timeloop[0] < 3) 
    return;

  shrink = (double *) calloc(ntlines, sizeof(double));
  
  difdif = ksed - kmarine;
  
  for(i = 0; i < nnodes; ++i) {
    maxspace[0][i] = 0;
    top1 = timeline[0][timeloop[0]-1][i];
    top2 = timeline[0][timeloop[0]-2][i];
    for(j = 0; j < timeloop[0] - 1; ++j) {
      if (compcut < 0)
	persand = (timeline[4][j][i] - kmarine) / difdif;
      else if ((x = (timeline[2][j][i] - compcut)) < 0)
	persand = 1;
      else
	persand = exp(- compdecay * x);
      pershale = 1 - persand;
      znew[0] = top1 - timeline[0][j+1][i];
      znew[1] = top1 - timeline[0][j][i];
      zold[0] = top2 - timeline[0][j+1][i];
      zold[1] = top2 - timeline[0][j][i];
      volpornew = persand * (phi0_sand / csand) * 
	(exp(-csand * znew[0]) - exp(-csand * znew[1]))
	  + pershale * (phi0_shale / cshale) *
	    (exp(-cshale * znew[0]) - exp(-cshale * znew[1]));
      volporold = persand * (phi0_sand / csand) * 
	(exp(-csand * zold[0]) - exp(-csand * zold[1]))
	  + pershale * (phi0_shale / cshale) *
	    (exp(-cshale * zold[0]) - exp(-cshale * zold[1]));
      shrink[j + 1] = volporold - volpornew;
      if (erosion) {
	if (znew[0] > maxspace[j][i])
	  maxspace[j][i] = znew[0];
	else
	  shrink[j + 1] = 0;
      }
      timeline[1][j][i] = 
	persand * phi0_sand * exp(-csand * maxspace[j][i])
	  + pershale * phi0_shale * exp(-cshale * maxspace[j][i]);
    }
    compsum = 0;
    for(j = 1; j < timeloop[0]; ++j) 
      timeline[0][j][i] -= (compsum += shrink[j]);
    timeline[1][timeloop[0]-1][i] = persand*phi0_sand + pershale*phi0_shale;
  }

  for (i = 0; i < nnodes; ++i) 
    /* chelev ignores compaction */
    elev[i] = timeline[0][timeloop[0]-1][i];
  
  free(shrink);
}



