
/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see file strata.h                                                 */
#include "strata.h"


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "header.h"

Boolean *cutoff, skim;
double pelagic;
double *exposed;
double *dep;

/* deposit.c */
void deposit(void)
{
  int i, j;
  double w, change;
  
  for (i = 0; i < nnodes; ++i)
    dep[i] = 0;
  
  for (i = shoreline; i < nnodes; ++i) 
    if (elev[i] < waterlevel)
      dep[i] += pelagic * timedelt;

  if (strcasecmp(carbonatefile, NoFile)) 
    for (i = j = 0; i < nnodes; ++i) {
      if (waterlevel < elev[i] || (skim && waterlevel == elev[i]))
	exposed[i] = time1, cutoff[i] = TRUE;
      while (i * delta >= carbpos[j + 1]) ++j;
      if (time1 - exposed[i] >= carblagtime[j]) 
	if ((w = waterlevel - elev[i]) > 0)
	  if (w >= carblagdepth[j] || !cutoff[i]) {
	    cutoff[i] = FALSE;
	    switch (carbtype[j]) {
	    case 'E':     /* epeiric carbonates */
	      dep[i]  += timedelt * carbepeiricsur * (w / carbepeiricdep) * 
		exp (1 - (w / carbepeiricdep));
	      break;
	    case 'O':     /* oceanic carbonates */
	      /* carbosur is the max. sed. rate. This occurs at carbodep 
		 (the water depth of the maximum sedimentation rate). This
		 sedimentation rate is constant between sea-level and carbodep
		 and declines exponentially after */	    
	      dep[i] += timedelt * carboceanicsur * 
		((w < carboceanicdep) ? 1 
		 : exp(-carboceanicsca * (w - carboceanicdep)));
	      break;
	    default:      /* no carbonates --- should be 'N' */
	      break;
	    }
	  }
    }
  
  for (i = 0; i < nnodes; ++i) {
    w = waterlevel - elev[i]; 
    if (dep[i] < w)
      change = dep[i];
    else if (w > 0) {
      change = w;
      if (skim) {
	exposed[i] = time1;
	cutoff[i] = TRUE;
      }
    }
    else
      change = 0;
    elev[i] += change;
    chelev[i] += change;
  }
}


