
/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see file strata.h                                                 */
#include "strata.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "header.h"

# define MAXISOTOPES 100

typedef enum { LIN, NCS, WAV } Fit;
  

char isotopefile[1000];
int nisotopes = 0;           /* max MAXISOTOPES */
String isoname[MAXISOTOPES];
String isounits[MAXISOTOPES];
double *isotope[MAXISOTOPES];
double isostart;

void doisotopes() {
  FILE *fp;
  int npts[MAXISOTOPES], maxnpts, i, j, k;
  Fit fit[MAXISOTOPES];
  char s[1000], str[1000], val[1000];
  double *f, *t, *cub, *bet, a, b, c, d, l, del, wav[5];

  fp = openfile (isotopefile);
  
  while (myfgets (str, 1000, fp)) {
    sscanf(str, "%s %s", s, val);
    if (!strcasecmp(s, "isotope:")) {  /* new isotope */
      ++nisotopes;
      isoname[nisotopes-1] = strdup(val);  
      
      if (fscanf(fp, "%s %s", s, val) != 2 || strcasecmp(s, "units:")) {
	printf ("Fatal error in %s, isotope %s:  bad/missing units\n",
		isotopefile, isoname[nisotopes-1]);
	exit(1);
      }
      isounits[nisotopes-1] = strdup(val);
      
      if (fscanf(fp, "%s %s", s, val) != 2 || strcasecmp(s, "fit:")) {
	printf ("Fatal error in %s, isotope %s:  bad/missing fit\n",
		isotopefile, isoname[nisotopes-1]);
	exit(1);
      }
      if (!strcasecmp(val, "linear")) {
	fit[nisotopes-1] = LIN;
	/* no parameters to read for linear fit */
      }
      else if (!strcasecmp(val, "cubic")) {
	fit[nisotopes-1] = NCS;
	/* no parameters to read for Natural Cubic Spline fit */
      }
      else if (!strcasecmp(val, "waves")) {
	fit[nisotopes-1] = WAV;
	/* no parameters to read for sine wave sum */
      }
      else {
	printf ("Fatal error in %s, isotope %s:  fit %s unknown\n",
		isotopefile, isoname[nisotopes-1], val);
	exit(1);
      }

      npts[nisotopes - 1] = 0;
    }
    else if (*str) {   /* data */
      if (!nisotopes) {
	printf ("Fatal error in %s:  doesn't start with isotope\n", 
		isotopefile);
	exit(1);
      }
      ++npts[nisotopes - 1];
    }
  }
  fclose (fp);

  maxnpts = 0;
  for (i = 0; i < nisotopes; ++i) {
    isotope[i] = (double *) calloc (ntlines, sizeof(double));
    if (npts[i] > maxnpts)
      maxnpts = npts[i];
  }

  t = (double *) calloc (maxnpts, sizeof(double));  /* raw times */
  f = (double *) calloc (maxnpts, sizeof(double));  /* raw data  */

  fp = openfile (isotopefile);

  for (i = 0; i < nisotopes; ++i) {
    while (myfgets(str, 1000, fp) && (!*str || strchr(str, ':')));
    if (fit[i] != WAV) {
      for (j = 0; j < npts[i]; ++j) {
	if (sscanf(str, "%lf %lf", t+j, f+j) != 2) {
	  printf ("Fatal error in %s:  %s datum %d missing or bad\n",
		  isotopefile, isoname[i], j+1);
	  exit(1);
	}
	t[j] -= isostart;      /* offset time */
	t[j] /= (tlstep * timedelt);  /* "timeline" units */
	while (myfgets(str, 1000, fp) && !*str);
      }
      if (t[0] > 0 || t[npts[i]-1] < ntlines - 1) {
	printf ("Fatal error in %s:  %s times (offset %g) do not span run\n",
		isotopefile, isoname[i], isostart);
	exit(1);
      }
      switch (fit[i]) {   /* fit */
      case LIN:
	j = 0;
	for (k = 0; k < ntlines; ++k) {
	  while (k > t[j+1]) ++j;
	  isotope[i][k] = f[j] + 
	    (f[j+1] - f[j]) * (k - t[j]) / (t[j+1] - t[j]);
	}
	break;
      case NCS:
	bet = (double *) calloc (npts[i], sizeof(double));
	cub = (double *) calloc (npts[i], sizeof(double));
	cub[0] = 0;
	l = 0;
	for (j = 1; j < npts[i] - 1; ++j) {
	  a = (t[j] - t[j-1]) / 6.0;
	  c = (t[j+1] - t[j]) / 6.0;
	  cub[j] = ((f[j+1] - f[j]) / c - (f[j] - f[j-1]) / a) / 6.0
	    - l * cub[j-1];
	  bet[j] = 2 * (a + c) - l * a;
	  l = c / bet[j];
	}
	cub[j] = 0;
	while (--j > 0) 
	  cub[j] = (cub[j] - cub[j+1] * (t[j+1] - t[j]) / 6.0) / bet[j];
	j = 0;
	for (k = 0; k < ntlines; ++k) {
	  while (k > t[j+1]) ++j;
	  del = t[j+1] - t[j];
	  a = (t[j+1] - k) / del;
	  b = (k - t[j]) / del;
	  c = (a*a*a - a) * del*del / 6.0;
	  d = (b*b*b - b) * del*del / 6.0;
	  isotope[i][k] = a * f[j] + b * f[j+1] + c * cub[j] + d * cub[j+1];
	}
	free(bet);
	free(cub);
	break;
      default:  /* WAV doesn't even get into this switch, remember */
	printf ("Fatal error in %s:  unable to do fit %d for %s\n",
		isotopefile, fit[i], isoname[i]);
	exit(1);
      } 
    }
    else {  /* WAV */
      for (k = 0; k < ntlines; ++k)
	isotope[i][k] = 0;      
      for (j = 0; j < npts[i]; ++j) {
	if (sscanf(str, "%lf %lf %lf %lf %lf", 
		   wav, wav+1, wav+2, wav+3, wav+4) != 5) {
	  printf("Fatal error in %s:  %s wave %d missing or bad\n",
		 isotopefile, isoname[j], j+1);
	  exit(1);
	}
	for (k = 0; k < 2; ++k) {  /* start and end times */
	  wav[k] -= isostart;            /* offset time */
	  wav[k] /= (tlstep * timedelt); /* "timeline" units */
	}
	wav[0] = (wav[0] < 0 ? 0 : ceil((double)wav[0]));
	wav[1] = (wav[1] > ntlines-1 ? ntlines-1 : floor((double)wav[1]));
	wav[3] /= (2 * 3.1415926535 * tlstep * timedelt);  /* period */
	wav[4] /= (tlstep * timedelt);   /* extra personal offset */
	for (k = wav[0]; k <= wav[1]; ++k) 
	  isotope[i][k] += wav[2] * sin((k + wav[4]) / wav[3]);
	while (myfgets(str, 1000, fp) && !*str);
      }
    }
  }
  
  fclose(fp); 
  free (t);
  free (f);
}
