

/* Copyright (C) 1995 P. B. Flemings, J. P. Grotzinger, J. E. Morris
   see file strata.h                                                 */
#include "strata.h"


/* set the timelines for the individual strata sequences */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "header.h"

extern void compact(void);
extern void movetimelines(void);
extern void settime_save(FILE *);
extern void thermal(FILE *);

Boolean written[7] = {FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE};
Boolean compaction;
double cut[7];
int *shoreboundary;
double *sealevel;

void settime(FILE *fptherm)
{
  int i;
  char temp[1000];
  FILE *fp;

  shoreboundary[timeloop[0]-1] = shoreline;
  sealevel[timeloop[0]-1] = waterlevel;
  movetimelines();
  if (compaction) 
    compact();
  if (thermdo)
    thermal(fptherm);

  if (!movie) 
    for (i = 1; i < 7; ++i)
      if (time1 >= cut[i]) 
	if (!written[i]) {
	  written[i] = TRUE;
	  cut[i] = time1;
	  timeloop[i] = timeloop[0];
	  sprintf(temp, "%s%d.out", library, i);
	  if ((fp = fopen(temp, "w"))) {
	    settime_save(fp);
	    fclose(fp);
	  }
	  else {
	    printf("Fatal error opening %s for output.  Simbasin exiting.\n",
		   temp);
	    exit(1);
	  }
	}
  
  for(i = 0; i < nnodes; ++i)
    tchelev[i] = 0;
}

extern void settime_spit(FILE *fp, double);
void settime_save(FILE *fp)
{
  int i, j, k;
  Boolean havedata[7];

  havedata[0] = TRUE;         /* elevation   */
  havedata[1] = compaction;   /* porosity    */
  havedata[2] = TRUE;         /* water depth */
  havedata[3] = TRUE;         /* max space   */
  havedata[4] = TRUE;         /* diff const  */
  havedata[5] = thermdo;      /* temperature */
  havedata[6] = thermdo;      /* TTI         */

  for (i = 0; i < 7; ++i)
    if (havedata[i])
      for (j = 0; j < timeloop[0]; ++j)
	for (k = 0; k < nnodes; ++k)
	  settime_spit(fp, timeline[i][j][k]);
}

void settime_spit(FILE *fp, double x)
{
  Boolean negmantis, negexpon;
  unsigned int mantis, expon;
  unsigned char buf[3];
  double y;

  if ((negmantis = (x < 0)))
    x = -x;
  else if (!x) {
    fwrite("\0\0\0", 1, 3, fp);
    return;
  }

  y = log(x) / .69314718 - 17;   /* binary log(x) = ln(x) / ln(2)  */
  if ((int)y == y) ++y;
  if ((negexpon = (y < 0)))
    expon = -y;
  else
    expon = y;

  if (expon > 31) 
    if (negexpon) {
      fwrite("\0\0\0", 1, 3, fp);
      return;
    }
    else {
      printf("Simbasin crashes and burns:  can't handle %g (too big)\n", x);
      exit(1);
    }
  
  if (negexpon)
    mantis = x * (1 << expon);
  else
    mantis = ((int)x) >> expon;

  buf[1] = (mantis & 65280) >> 8;
  buf[2] = mantis & 255;

  buf[0] = expon;
  if (negexpon) buf[0] += 32;
  if (negmantis) buf[0] += 64;
  if (mantis & 65536) buf[0] += 128;

  fwrite(buf, 1, 3, fp);
}

